@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,7 +25,7 @@ using namespace std;
  *  The History class, it represents an execution record of a Virtual Machine.
  */
 
-class History:public ObjectSQL
+class History:public ObjectSQL, public ObjectXML
 {
 public:
     enum MigrationReason
@@ -40,13 +40,14 @@ public:
     History(int oid, int _seq = -1);
 
     History(
-        int             oid,
-        int             seq,
-        int             hid,
-        string&         hostname,
-        string&         vm_dir,
-        string&         vmm,
-        string&         tm);
+        int oid,
+        int seq,
+        int hid,
+        const string& hostname,
+        const string& vm_dir,
+        const string& vmm,
+        const string& vnm,
+        const string& tm);
 
     ~History(){};
 
@@ -55,14 +56,6 @@ public:
      */
     friend ostream& operator<<(ostream& os, const History& history);
 
-    /**
-     * Function to print the History object into a string in
-     * plain text
-     *  @param str the resulting string
-     *  @return a reference to the generated string
-     */
-    string& to_str(string& str) const;
-
     /**
      * Function to print the History object into a string in
      * XML format
@@ -78,52 +71,15 @@ private:
     // ----------------------------------------
     // DataBase implementation variables
     // ----------------------------------------
-    enum ColNames
-    {
-        VID             = 0,
-        SEQ             = 1,
-        HOSTNAME        = 2,
-        VM_DIR          = 3,
-        HID             = 4,
-        VMMMAD          = 5,
-        TMMAD           = 6,
-        STIME           = 7,
-        ETIME           = 8,
-        PROLOG_STIME    = 9,
-        PROLOG_ETIME    = 10,
-        RUNNING_STIME   = 11,
-        RUNNING_ETIME   = 12,
-        EPILOG_STIME    = 13,
-        EPILOG_ETIME    = 14,
-        REASON          = 15,
-        LIMIT           = 16
-    };
 
     static const char * table;
 
     static const char * db_names;
 
-    static const char * extended_db_names;
-
     static const char * db_bootstrap;
 
     void non_persistent_data();
 
-    static string column_name(const ColNames column)
-    {
-        switch (column)
-        {
-        case HID:
-            return "hid";
-        case ETIME:
-            return "etime";
-        case RUNNING_ETIME:
-            return "retime";
-        default:
-            return "";
-        }
-    }
-
     // ----------------------------------------
     // History fields
     // ----------------------------------------
@@ -136,6 +92,7 @@ private:
     int     hid;
 
     string  vmm_mad_name;
+    string  vnm_mad_name;
     string  tm_mad_name;
 
     time_t  stime;
@@ -167,7 +124,12 @@ private:
      *    @param db pointer to the database.
      *    @return 0 on success.
      */
-    int insert(SqlDB * db, string& error_str);
+    int insert(SqlDB * db, string& error_str)
+    {
+        error_str.clear();
+
+        return insert_replace(db, false);
+    }
 
     /**
      *  Reads the history record from the DB
@@ -181,7 +143,10 @@ private:
      *    @param db pointer to the database.
      *    @return 0 on success.
      */
-     int update(SqlDB * db);
+     int update(SqlDB * db)
+     {
+        return insert_replace(db, true);
+     }
 
     /**
      *  Removes the all history records from the DB
@@ -208,15 +173,35 @@ private:
     int select_cb(void *nil, int num, char **values, char **names);
 
     /**
-     *  Function to unmarshall a History object into an output stream with XML
-     *  format.
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
+     *  Rebuilds the object from an xml node
+     *    @param node The xml node pointer
+     *
+     *    @return 0 on success, -1 otherwise
      */
-    static int dump(ostringstream& oss, int  num, char **names, char **values);
+    int from_xml_node(const xmlNodePtr node)
+    {
+        ObjectXML::update_from_node(node);
+
+        return rebuild_attributes();
+    }
+
+    /**
+     *  Rebuilds the object from an xml formatted string
+     *    @param xml_str The xml-formatted string
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int from_xml(const string &xml_str)
+    {
+        ObjectXML::update_from_str(xml_str);
+
+        return rebuild_attributes();
+    }
+
+    /**
+     *  Rebuilds the internal attributes using xpath
+     */
+    int rebuild_attributes();
 };
 
 #endif /*HISTORY_H_*/
