@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)           */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)           */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -20,7 +20,6 @@
 #include "PoolSQL.h"
 #include "HostTemplate.h"
 #include "HostShare.h"
-#include "ClusterPool.h"
 
 using namespace std;
 
@@ -45,32 +44,19 @@ public:
     };
 
     /**
-     *  Function to write a Host on an output stream
+     * Function to print the Host object into a string in XML format
+     *  @param xml the resulting XML string
+     *  @return a reference to the generated string
      */
-     friend ostream& operator<<(ostream& os, Host& h);
-
-	/**
-	 * Function to print the Host object into a string in plain text
-	 *  @param str the resulting string
-	 *  @return a reference to the generated string
-	 */
-	string& to_str(string& str) const;
-
-	/**
-	 * Function to print the Host object into a string in XML format
-	 *  @param xml the resulting XML string
-	 *  @return a reference to the generated string
-	 */
-	string& to_xml(string& xml) const;
+    string& to_xml(string& xml) const;
 
     /**
-     * Get the Host unique identifier HID, that matches the OID of the object
-     *    @return HID Host identifier
+     *  Rebuilds the object from an xml formatted string
+     *    @param xml_str The xml-formatted string
+     *
+     *    @return 0 on success, -1 otherwise
      */
-    int get_hid() const
-    {
-        return oid;
-    };
+    int from_xml(const string &xml_str);
 
     /**
      *  Check if the host is enabled
@@ -120,15 +106,6 @@ public:
         state = INIT;
     };
 
-    /**
-     *  Returns host host_name
-     *     @return host_name Host's hostname
-     */
-    const string& get_hostname() const
-    {
-        return hostname;
-    };
-
     /** Update host counters and update the whole host on the DB
      *    @param parse_str string with values to be parsed
      *    @return 0 on success
@@ -153,6 +130,15 @@ public:
         return vmm_mad_name;
     };
 
+    /**
+     * Retrives VNM mad name
+     *    @return string vnm mad name
+     */
+    const string& get_vnm_mad() const
+    {
+        return vnm_mad_name;
+    };
+
     /**
      * Retrives TM mad name
      *    @return string tm mad name
@@ -189,73 +175,6 @@ public:
         return last_monitored;
     };
 
-    /**
-     *  Sets the cluster for this host
-     *    @return time_t last monitored time
-     */
-    int set_cluster(const string& cluster_name)
-    {
-        cluster = cluster_name;
-        return 0;
-    };
-
-    // ------------------------------------------------------------------------
-    // Template
-    // ------------------------------------------------------------------------
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        string& name,
-        vector<const Attribute*>& values) const
-    {
-        return host_template.get(name,values);
-    };
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        const char *name,
-        vector<const Attribute*>& values) const
-    {
-        string str=name;
-        return host_template.get(str,values);
-    };
-
-    /**
-     *  Gets a string based host attribute
-     *    @param name of the attribute
-     *    @param value of the attribute (a string), will be "" if not defined
-     */
-    void get_template_attribute(
-        const char *    name,
-        string&         value) const
-    {
-        string str=name;
-        host_template.get(str,value);
-    }
-
-    /**
-     *  Gets a string based host attribute
-     *    @param name of the attribute
-     *    @param value of the attribute (an int), will be 0 if not defined
-     */
-    void get_template_attribute(
-        const char *    name,
-        int&            value) const
-    {
-        string str=name;
-        host_template.get(str,value);
-    }
-
     // ------------------------------------------------------------------------
     // Share functions
     // ------------------------------------------------------------------------
@@ -367,6 +286,14 @@ public:
         return host_share.test(cpu,mem,disk);
     }
 
+    /**
+     *  Factory method for host templates
+     */
+    Template * get_new_template()
+    {
+        return new HostTemplate;
+    }
+
 private:
 
     // -------------------------------------------------------------------------
@@ -378,9 +305,6 @@ private:
     // -------------------------------------------------------------------------
     // Host Description
     // -------------------------------------------------------------------------
-
-    string      hostname;
-
     /**
      *  The state of the Host
      */
@@ -396,101 +320,47 @@ private:
      */
 	string      vmm_mad_name;
 
+    /**
+     *  Name of the VN driver used to manage networking in this host
+     */
+    string      vnm_mad_name;
+
 	/**
      *  Name of the TM driver used to transfer file to and from this host
      */
 	string      tm_mad_name;
 
 	/**
-     *  If Host State = MONITORED  last time it got fully monitored or 1 Jan 1970
+     *  If Host State= MONITORED  last time it got fully monitored or 1 Jan 1970
      *     Host State = MONITORING last time it got a signal to be monitored
      */
     time_t      last_monitored;
 
-    /**
-     *  Name of the cluster this host belongs to.
-     */
-    string      cluster;
-
     // -------------------------------------------------------------------------
     //  Host Attributes
     // -------------------------------------------------------------------------
-
-    /**
-     *  The Host template, holds the Host attributes.
-     */
-    HostTemplate    host_template;
-
     /**
      *  The Share represents the logical capacity associated with the host
      */
     HostShare       host_share;
 
-    // *************************************************************************
-    // DataBase implementation (Private)
-    // *************************************************************************
-
-    /**
-     *  Execute an INSERT or REPLACE Sql query.
-     *    @param db The SQL DB
-     *    @param replace Execute an INSERT or a REPLACE
-     *    @return 0 one success
-    */
-    int insert_replace(SqlDB *db, bool replace);
-
-    /**
-     *  Callback function to unmarshall a Host object (Host::select)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int select_cb(void *nil, int num, char **values, char **names);
-
-    /**
-     *  Bootstraps the database table(s) associated to the Host
-     */
-    static void bootstrap(SqlDB * db)
-    {
-        ostringstream oss_host(Host::db_bootstrap);
-        ostringstream oss_share(HostShare::db_bootstrap);
-
-        db->exec(oss_host);
-        db->exec(oss_share);
-    };
-
-protected:
-
     // *************************************************************************
     // Constructor
     // *************************************************************************
 
-    Host(int     id=-1,
-         string _hostname="",
-         string _im_mad_name="",
-         string _vmm_mad_name="",
-         string _tm_mad_name="");
+    Host(int           id=-1,
+         const string& hostname="",
+         const string& im_mad_name="",
+         const string& vmm_mad_name="",
+         const string& vnm_mad_name="",
+         const string& tm_mad_name="");
 
     virtual ~Host();
 
     // *************************************************************************
-    // DataBase implementation
+    // DataBase implementation (Private)
     // *************************************************************************
 
-    enum ColNames
-    {
-        OID              = 0,
-        HOST_NAME        = 1,
-        STATE            = 2,
-        IM_MAD           = 3,
-        VM_MAD           = 4,
-        TM_MAD           = 5,
-        LAST_MON_TIME    = 6,
-        CLUSTER          = 7,
-        TEMPLATE         = 8,
-        LIMIT            = 9
-    };
-
     static const char * db_names;
 
     static const char * db_bootstrap;
@@ -498,42 +368,45 @@ protected:
     static const char * table;
 
     /**
-     *  Reads the Host (identified with its OID=HID) from the database.
-     *    @param db pointer to the db
+     *  Execute an INSERT or REPLACE Sql query.
+     *    @param db The SQL DB
+     *    @param replace Execute an INSERT or a REPLACE
+     *    @param error_str Returns the error reason, if any
+     *    @return 0 one success
+     */
+    int insert_replace(SqlDB *db, bool replace, string& error_str);
+
+    /**
+     *  Bootstraps the database table(s) associated to the Host
      *    @return 0 on success
      */
-    virtual int select(SqlDB *db);
+    static int bootstrap(SqlDB * db)
+    {
+        ostringstream oss_host(Host::db_bootstrap);
+
+        return db->exec(oss_host);
+    };
 
     /**
      *  Writes the Host and its associated HostShares in the database.
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    virtual int insert(SqlDB *db, string& error_str);
+    int insert(SqlDB *db, string& error_str)
+    {
+        return insert_replace(db, false, error_str);
+    };
 
     /**
      *  Writes/updates the Hosts data fields in the database.
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    virtual int update(SqlDB *db);
-
-    /**
-     *  Drops host from the database
-     *    @param db pointer to the db
-     *    @return 0 on success
-     */
-    virtual int drop(SqlDB *db);
-
-    /**
-     *  Function to output a Host object in to an stream in XML format
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    static int dump(ostringstream& oss, int num, char **values, char **names);
+    int update(SqlDB *db)
+    {
+        string error_str;
+        return insert_replace(db, true, error_str);
+    };
 };
 
 #endif /*HOST_H_*/
