@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)           */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)           */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -47,25 +47,15 @@ public:
         INIT      = 0, /** < Initialization state */
         READY     = 1, /** < Image ready to use */
         USED      = 2, /** < Image in use */
-        DISABLED  = 3  /** < Image can not be instantiated by a VM */
+        DISABLED  = 3, /** < Image can not be instantiated by a VM */
+        LOCKED    = 4, /** < FS operation for the Image in process */
+        ERROR     = 5  /** < Error state the operation FAILED*/
     };
 
-    /**
-     *  Function to write an Image on an output stream
-     */
-     friend ostream& operator<<(ostream& os, Image& i);
-
     // *************************************************************************
     // Image Public Methods
     // *************************************************************************
 
-    /**
-     * Function to print the Image object into a string in plain text
-     *  @param str the resulting string
-     *  @return a reference to the generated string
-     */
-    string& to_str(string& str) const;
-
     /**
      * Function to print the Image object into a string in XML format
      *  @param xml the resulting XML string
@@ -74,169 +64,211 @@ public:
     string& to_xml(string& xml) const;
 
     /**
-     * Get the Image unique identifier IID, that matches the OID of the object
-     *    @return IID Image identifier
+     *  Rebuilds the object from an xml formatted string
+     *    @param xml_str The xml-formatted string
+     *
+     *    @return 0 on success, -1 otherwise
      */
-    int get_iid() const
-    {
-        return oid;
-    };
-
-    /**
-     * Gets the uid of the owner of the Image
-     * @return uid
-     **/
-    int get_uid()
-    {
-        return uid;
-    }
-
-    /**
-     *  Returns Image's name
-     *     @return name Image's name
-     */
-    const string& get_name() const
-    {
-        return name;
-    };
-
-    /**
-     *  Returns true if the image is public
-     *     @return true if the image is public
-     */
-    bool isPublic()
-    {
-        return (public_img == 1);
-    };
+    int from_xml(const string &xml_str);
 
     /**
      *  Returns true if the image is persistent
      *     @return true if the image is persistent
      */
-    bool isPersistent()
+    bool isPersistent() const
     {
         return (persistent_img == 1);
     };
 
     /**
-     *  Set enum type
-     *     @return 0 on success, -1 otherwise
+     *  Returns the source path of the image
+     *     @return source of image
      */
-    int set_type(const string& _type)
+    const string& get_source() const
     {
-        int rc = 0;
-
-        if ( _type == "OS" )
-        {
-            type = OS;
-        }
-        else if ( _type == "CDROM" )
-        {
-            type = CDROM;
-        }
-        else if ( _type == "DATABLOCK" )
-        {
-            type = DATABLOCK;
-        }
-        else
-        {
-            rc = -1;
-        }
-
-        return rc;
+        return source;
     }
 
     /**
-     * Get an image to be used in a VM, and updates its state.
-     *  @return 0 if success
+     *  Returns the original path of the image
+     *     @return path of image
      */
-    int acquire_image();
-
-
-    /**
-     * Releases an image being used by a VM
-     *  @return true if the image needs to be updated
-     */
-    bool release_image();
-
-    /**
-     *  Enables the image
-     *    @param to_enable true will enable the image.
-     *    @return 0 on success
-     */
-    int enable(bool to_enable)
+    const string& get_path() const
     {
-        int rc = 0;
-
-        if ( to_enable == true )
-        {
-            if(state == DISABLED)
-            {
-                state = READY;
-            }
-        }
-        else if (state != USED) // to_enable == false
-        {
-            state = DISABLED;
-        }
-        else
-        {
-            rc = -1;
-        }
-
-        return rc;
+        return path;
     }
 
     /**
-     *  Publish or unpublish an image
-     *    @param pub true to publish the image
-     *    @return 0 on success
+     *  Returns the fs_type for the image (defined for datablocks)
+     *     @return fs_type
      */
-    bool publish(bool pub)
+    const string& get_fstype() const
     {
-        bool success = false;
-
-        if (pub == true)
-        {
-            if (!isPersistent())
-            {
-                public_img = 1;
-                success    = true;
-            }
-        }
-        else
-        {
-            public_img = 0;
-            success    = true;
-        }
-
-        return success;
+        return fs_type;
     }
 
+    /**
+     *  Returns the size of the image 
+     *     @return size in mb
+     */
+    int get_size() const
+    {
+        return size_mb;
+    }
+
+    /**
+     *  Sets the source path of the image
+     */
+    void set_source(const string& _source)
+    {
+        source = _source;
+    }
+
+    /**
+     *  Sets the size for the image
+     */
+    void set_size(unsigned int _size_mb)
+    {
+        size_mb = _size_mb;
+    }
+
+    /**
+     *  Returns the type of the image
+     *     @return type
+     */
+    ImageType get_type()
+    {
+        return type;
+    }
+    /**
+     *  Returns the image state
+     *     @return state of image
+     */
+    ImageState get_state()
+    {
+        return state;
+    }
+
+    /**
+     *  Sets the image state
+     *     @param state of image
+     */
+    void set_state(ImageState _state)
+    {
+        state = _state;
+    }
+
+    /**
+     *
+     */
+    int dec_running ()
+    {
+        return --running_vms;
+    }
+
+    /**
+     *
+     */
+    int inc_running()
+    {
+        return ++running_vms;
+    }
+
+    /**
+     *
+     */
+    int get_running()
+    {
+        return running_vms;
+    }
+
+    /**
+     * Sets the Image type.
+     *
+     * @param _type the new type. It will be transformed to upper case
+     * @return 0 on success, -1 otherwise
+     */
+    int set_type(string& _type);
+
+    /**
+     *  Check if the image can be used by other users
+     *  @return true if group or others can access the image
+     */
+    bool isPublic()
+    {
+       return (group_u == 1 || other_u == 1); 
+    }
+
+    /**
+     *  Set permissions for the Image. Extends the PoolSQLObject method
+     *  by checking the persistent state of the image.
+     */
+    int set_permissions(int _owner_u,
+                        int _owner_m,
+                        int _owner_a,
+                        int _group_u,
+                        int _group_m,
+                        int _group_a,
+                        int _other_u,
+                        int _other_m,
+                        int _other_a,
+                        string& error_str)
+    {
+        if ( isPersistent() && (_group_u == 1 || _other_u == 1) )
+        {
+            error_str = "Image cannot be public and persistent.";
+
+            return -1;
+        } 
+
+        return PoolObjectSQL::set_permissions(_owner_u, _owner_m, _owner_a,
+                                              _group_u, _group_m, _group_a,
+                                              _other_u, _other_m, _other_a,
+                                              error_str);
+    };
+
     /**
      *  Set/Unset an image as persistent
      *    @param persistent true to make an image persistent
+     *    @param error_str Returns the error reason, if any
+     *
      *    @return 0 on success
      */
-    bool persistent(bool persis)
+    int persistent(bool persis, string& error_str)
     {
-        bool success = false;
+        if ( running_vms != 0 )
+        {
+            goto error_vms;
+        }
 
         if (persis == true)
         {
-            if (!isPublic() && running_vms == 0)
+            
+            if ( isPublic() )
             {
-                persistent_img = 1;
-                success        = true;
+                goto error_public;
             }
+            
+            persistent_img = 1;
         }
         else
         {
             persistent_img = 0;
-            success        = true;
         }
 
-        return success;
+        return 0;
+
+    error_vms:
+        error_str = "Image cannot be in 'used' state.";
+        goto error_common;
+
+    error_public:
+        error_str = "Image cannot be public and persistent.";
+        goto error_common;
+
+    error_common:
+        return -1;
+
     }
 
     /**
@@ -256,91 +288,12 @@ public:
      */
     int disk_attribute(VectorAttribute * disk, int* index, ImageType* img_type);
 
-    // ------------------------------------------------------------------------
-    // Template
-    // ------------------------------------------------------------------------
-
     /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
+     *  Factory method for image templates
      */
-    int get_template_attribute(
-        string& name,
-        vector<const Attribute*>& values) const
+    Template * get_new_template()
     {
-        return image_template->get(name,values);
-    };
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        const char *name,
-        vector<const Attribute*>& values) const
-    {
-        string str=name;
-        return image_template->get(str,values);
-    };
-
-    /**
-     *  Gets a string based Image attribute
-     *    @param name of the attribute
-     *    @param value of the attribute (a string), will be "" if not defined
-     */
-    void get_template_attribute(
-        const char *    name,
-        string&         value) const
-    {
-        string str=name;
-        image_template->get(str,value);
-    }
-
-    /**
-     *  Gets a string based Image attribute
-     *    @param name of the attribute
-     *    @param value of the attribute (an int), will be 0 if not defined
-     */
-    void get_template_attribute(
-        const char *    name,
-        int&            value) const
-    {
-        string str=name;
-        image_template->get(str,value);
-    }
-
-    /**
-     *  Removes an Image attribute
-     *    @param name of the attribute
-     */
-    int remove_template_attribute(const string&   name)
-    {
-        return image_template->erase(name);
-    }
-
-    /**
-     *  Adds a new attribute to the template (replacing it if
-     *  already defined), the image's mutex SHOULD be locked
-     *    @param name of the new attribute
-     *    @param value of the new attribute
-     *    @return 0 on success
-     */
-    int replace_template_attribute(
-        const string& name,
-        const string& value)
-    {
-        SingleAttribute * sattr;
-
-        image_template->erase(name);
-
-        sattr = new SingleAttribute(name,value);
-        image_template->set(sattr);
-
-        return 0;
+        return new ImageTemplate;
     }
 
 private:
@@ -355,26 +308,11 @@ private:
     // Image Description
     // -------------------------------------------------------------------------
 
-    /**
-     *  Owner if the image
-     */
-    int         uid;
-
-    /**
-     *  The name of the Image
-     */
-    string       name;
-
     /**
      *  Type of the Image
      */
     ImageType    type;
 
-    /**
-     *  Public scope of the Image
-     */
-    int          public_img;
-
     /**
      *  Persistency of the Image
      */
@@ -388,28 +326,33 @@ private:
     /**
      *  Path to the image
      */
-    string      source;
+    string       source;
+
+    /**
+     *  Original Path to the image (optional if source is given or datablock)
+     */
+    string       path;
+
+    /**
+     *  File system type for the image (mandatory for datablocks)
+     */
+    string       fs_type;
+
+    /**
+     *  Size of the image in MB
+     */
+    unsigned int size_mb;
 
      /**
       *  Image state
       */
-    ImageState  state;
+    ImageState   state;
 
     /**
      * Number of VMs using the image
      */
     int running_vms;
 
-    // -------------------------------------------------------------------------
-    //  Image Attributes
-    // -------------------------------------------------------------------------
-
-    /**
-     *  The Image template, holds the Image attributes.
-     */
-    ImageTemplate *  image_template;
-
-
     // *************************************************************************
     // DataBase implementation (Private)
     // *************************************************************************
@@ -418,36 +361,28 @@ private:
      *  Execute an INSERT or REPLACE Sql query.
      *    @param db The SQL DB
      *    @param replace Execute an INSERT or a REPLACE
-     *    @return 0 on success
-    */
-    int insert_replace(SqlDB *db, bool replace);
-
-    /**
-     *  Callback function to unmarshall a Image object (Image::select)
-     *    @param num the number of columns read from the DB
-     *    @param names  the column names
-     *    @param values the column values
+     *    @param error_str Returns the error reason, if any
      *    @return 0 on success
      */
-    int select_cb(void *nil, int num, char **values, char **names);
+    int insert_replace(SqlDB *db, bool replace, string& error_str);
 
     /**
      *  Bootstraps the database table(s) associated to the Image
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * db)
+    static int bootstrap(SqlDB * db)
     {
         ostringstream oss_image(Image::db_bootstrap);
 
-        db->exec(oss_image);
+        return db->exec(oss_image);
     };
 
-
     /**
      *  "Encrypts" the password with SHA1 digest
      *  @param password
      *  @return sha1 encrypted password
      */
-    string sha1_digest(const string& pass);
+    static string sha1_digest(const string& pass);
 
 protected:
 
@@ -455,7 +390,11 @@ protected:
     // Constructor
     // *************************************************************************
 
-    Image(int uid=-1, ImageTemplate *img_template = 0);
+    Image(int            uid,
+          int            gid,
+          const string&  uname,
+          const string&  gname,
+          ImageTemplate* img_template);
 
     virtual ~Image();
 
@@ -463,37 +402,12 @@ protected:
     // DataBase implementation
     // *************************************************************************
 
-    enum ColNames
-    {
-        OID              = 0,    /* Image identifier (IID)      */
-        UID              = 1,    /* Image owner id              */
-        NAME             = 2,    /* Image name                  */
-        TYPE             = 3,    /* 0) OS 1) CDROM 2) DATABLOCK */
-        PUBLIC           = 4,    /* Public scope (YES OR NO)    */
-        PERSISTENT       = 5,    /* Peristency (YES OR NO)      */
-        REGTIME          = 6,    /* Time of registration        */
-        SOURCE           = 7,    /* Path to the image           */
-        STATE            = 8,    /* 0) INIT   1) ALLOCATED      */
-        RUNNING_VMS      = 9,    /* Number of VMs using the img */
-        TEMPLATE         = 10,    /* Image template xml data     */
-        LIMIT            = 11
-    };
-
-    static const char * extended_db_names;
-
     static const char * db_names;
 
     static const char * db_bootstrap;
 
     static const char * table;
 
-    /**
-     *  Reads the Image (identified with its OID=IID) from the database.
-     *    @param db pointer to the db
-     *    @return 0 on success
-     */
-    virtual int select(SqlDB *db);
-
     /**
      *  Writes the Image in the database.
      *    @param db pointer to the db
@@ -507,23 +421,6 @@ protected:
      *    @return 0 on success
      */
     virtual int update(SqlDB *db);
-
-    /**
-     *  Drops Image and associated template from the database
-     *    @param db pointer to the db
-     *    @return 0 on success
-     */
-     virtual int drop(SqlDB *db);
-
-    /**
-     *  Function to output an Image object in to an stream in XML format
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    static int dump(ostringstream& oss, int num, char **values, char **names);
 };
 
 #endif /*IMAGE_H_*/
