@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -38,65 +38,62 @@ class ImagePool : public PoolSQL
 {
 public:
 
-    ImagePool(SqlDB * db,
-              const string&   _source_prefix,
-              const string&   _default_type,
-              const string&   _default_dev_prefix);
+    ImagePool(SqlDB *       db,
+              const string& _default_type,
+              const string& _default_dev_prefix);
 
     ~ImagePool(){};
 
     /**
      *  Function to allocate a new Image object
      *    @param uid the user id of the image's owner
-     *    @param stemplate template associated with the image
+     *    @param gid the id of the group this object is assigned to
+     *    @param uname name of the user
+     *    @param gname name of the group
+     *    @param img_template template associated with the image
      *    @param oid the id assigned to the Image
+     *    @param error_str Returns the error reason, if any
      *    @return the oid assigned to the object,
      *                  -1 in case of failure
      *                  -2 in case of template parse failure
      */
     int allocate (
         int             uid,
+        int             gid,
+        const string&   uname,
+        const string&   gname,
         ImageTemplate * img_template,
         int *           oid,
         string&         error_str);
 
     /**
-     *  Function to get a Image from the pool, if the object is not in memory
+     **  Function to get a Image from the pool, if the object is not in memory
      *  it is loaded from the DB
      *    @param oid Image unique id
      *    @param lock locks the Image mutex
      *    @return a pointer to the Image, 0 if the Image could not be loaded
      */
-    Image * get(
-        int     oid,
-        bool    lock)
+    Image * get(int oid, bool lock)
     {
         return static_cast<Image *>(PoolSQL::get(oid,lock));
     };
 
     /**
-     *  Function to get an Image from the pool using the image name
-     *    @param name of the image
-     *    @param lock locks the User mutex
-     *    @return a pointer to the Image, 0 if the User could not be loaded
+     *  Gets an object from the pool (if needed the object is loaded from the
+     *  database).
+     *   @param name of the object
+     *   @param uid id of owner
+     *   @param lock locks the object if true
+     *
+     *   @return a pointer to the object, 0 in case of failure
      */
-    Image * get(
-        const string&  name,
-        bool           lock)
+    Image * get(const string& name, int uid, bool lock)
     {
-        map<string, int>::iterator     index;
+        return static_cast<Image *>(PoolSQL::get(name,uid,lock));
+    };
 
-        index = image_names.find(name);
-
-        if ( index != image_names.end() )
-        {
-            return get((int)index->second,lock);
-        }
-
-        return 0;
-    }
-
-    /** Update a particular Image
+    /** 
+     *  Update a particular Image
      *    @param image pointer to Image
      *    @return 0 on success
      */
@@ -105,28 +102,13 @@ public:
         return image->update(db);
     };
 
-    /** Drops an image from the DB, the image mutex MUST BE locked
-     *    @param image pointer to Image
-     *    @return 0 on success
-     */
-    int drop(Image * image)
-    {
-        int rc = PoolSQL::drop(image);
-
-        if ( rc == 0)
-        {
-            image_names.erase(image->get_name());
-        }
-
-        return rc;
-    };
-
     /**
      *  Bootstraps the database table(s) associated to the Image pool
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB *_db)
+    static int bootstrap(SqlDB *_db)
     {
-        Image::bootstrap(_db);
+        return Image::bootstrap(_db);
     };
 
     /**
@@ -136,7 +118,10 @@ public:
      *  @param where filter for the objects, defaults to all
      *  @return 0 on success
      */
-    int dump(ostringstream& oss, const string& where);
+    int dump(ostringstream& oss, const string& where)
+    {
+        return PoolSQL::dump(oss, "IMAGE_POOL", Image::table, where);
+    }
 
     /**
      *  Generates a DISK attribute for VM templates using the Image metadata
@@ -145,23 +130,25 @@ public:
      *    @param index number of datablock images used by the same VM. Will be
      *                 automatically increased.
      *    @param img_type will be set to the used image's type
-     *    @return 0 on success, -1 error, -2 not using the pool
+     *    @param uid of VM owner (to look for the image id within its images)
+     *    @param image_id on success returns the acquired image id
+     *    @return 0 on success, 
+     *            -1 error, 
+     *            -2 not using the pool, 
      */
     int disk_attribute(VectorAttribute *  disk,
                        int                disk_id,
                        int *              index,
-                       Image::ImageType * img_type);
+                       Image::ImageType * img_type,
+                       int                uid,
+                       int&               image_id);
     /**
      *  Generates an Authorization token for the DISK attribute
      *    @param disk the disk to be authorized
+     *    @param uid of owner (to look for the image id within her images)
      *    @param ar the AuthRequest
      */
-    void authorize_disk(VectorAttribute * disk, AuthRequest * ar);
-
-    static const string& source_prefix()
-    {
-        return _source_prefix;
-    };
+    void authorize_disk(VectorAttribute * disk, int uid, AuthRequest * ar);
 
     static const string& default_type()
     {
@@ -177,57 +164,28 @@ private:
     //--------------------------------------------------------------------------
     // Configuration Attributes for Images
     // -------------------------------------------------------------------------
-    /**
-     * Path to the image repository
-     **/
-    static string       _source_prefix;
 
     /**
      * Default image type
      **/
-    static string       _default_type;
+    static string  _default_type;
 
     /**
      * Default device prefix
      **/
-    static string       _default_dev_prefix;
+    static string  _default_dev_prefix;
 
     //--------------------------------------------------------------------------
     // Pool Attributes
     // -------------------------------------------------------------------------
-    /**
-     *  This map stores the association between IIDs and Image names
-     */
-    map<string, int>    image_names;
-
     /**
      *  Factory method to produce Image objects
      *    @return a pointer to the new Image
      */
     PoolObjectSQL * create()
     {
-        return new Image;
+        return new Image(-1,-1,"","",0);
     };
-
-    /**
-     *  Callback function to get output the image pool in XML format
-     *  (Image::dump)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int dump_cb(void * _oss, int num, char **values, char **names);
-
-    /**
-     *  Callback function to build the image_names map
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int init_cb(void *nil, int num, char **values, char **names);
-
 };
 
 #endif /*IMAGE_POOL_H_*/
