@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -22,7 +22,7 @@
 using namespace std;
 
 /**
- *  Image Template class, it represents the attributes of a Host
+ *  Image Template class, it represents the attributes of an Image
  */
 class ImageTemplate : public Template
 {
@@ -30,8 +30,28 @@ public:
     ImageTemplate() : Template(true,'=',"TEMPLATE"){};
 
     ~ImageTemplate(){};
+
+    /**
+     *  Checks the template for RESTRICTED ATTRIBUTES
+     *    @param rs_attr the first restricted attribute found if any
+     *    @return true if a restricted attribute is found in the template
+     */
+    bool check(string& rs_attr);
+
+private:
+    /**
+     *  Number of restricted attributes
+     */
+    const static int    RS_ATTRS_LENGTH;
+
+    /**
+     *  Restricted template attributes in the form
+     *  'SINGLE' or 'VECTOR/ATTR'. Restricted attributes are only
+     *  allowed for ONE_ADMIN Group.
+     */
+    const static string RESTRICTED_ATTRIBUTES[];
 };
-    
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
