@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -18,6 +18,7 @@
 #define LEASES_H_
 
 #include "ObjectSQL.h"
+#include "ObjectXML.h"
 #include "Attribute.h"
 
 #include <map>
@@ -39,8 +40,9 @@ public:
      * @param _oid the virtual network unique identifier
      * @param _size the max number of leases
      */
-    Leases(SqlDB * _db, int _oid, unsigned long _size):
-        oid(_oid), size(_size), db(_db){};
+    Leases(SqlDB * _db, int _oid, unsigned long _size, unsigned int _mac_prefix):
+        ObjectSQL(),
+        oid(_oid), size(_size), n_used(0), mac_prefix(_mac_prefix), db(_db){};
 
     virtual ~Leases()
     {
@@ -100,6 +102,26 @@ public:
     virtual int remove_leases(vector<const Attribute*>& vector_leases,
                               string&                   error_msg) = 0;
 
+    /**
+     * Holds a Lease, marking it as used
+     *  @param vector_leases vector of VectorAttribute objects. For the
+     *         moment, the vector can only contain one LEASE.
+     *  @param error_msg If the action fails, this message contains
+     *         the reason.
+     *  @return 0 on success
+     */
+    int hold_leases(vector<const Attribute*>& vector_leases, string& error_msg);
+
+    /**
+     * Releases a Lease on hold
+     *  @param vector_leases vector of VectorAttribute objects. For the
+     *         moment, the vector can only contain one LEASE.
+     *  @param error_msg If the action fails, this message contains
+     *         the reason.
+     *  @return 0 on success
+     */
+    int free_leases(vector<const Attribute*>& vector_leases, string& error_msg);
+
     // -------------------------------------------------------------------------
     // -------------------------------------------------------------------------
 
@@ -108,7 +130,7 @@ protected:
      *  The Lease class, it represents a pair of IP and MAC assigned to
      *  a Virtual Machine
      */
-    class Lease
+    class Lease : public ObjectXML
     {
     public:
         /**
@@ -129,13 +151,19 @@ protected:
         * @param _used, the lease is in use
         */
         Lease(unsigned int _ip, unsigned int _mac[], int _vid, bool _used=true)
-            :ip(_ip), vid(_vid), used(_used)
+            :ObjectXML(),ip(_ip), vid(_vid), used(_used)
         {
                 // TODO check size
                 mac[PREFIX]=_mac[PREFIX];
                 mac[SUFFIX]=_mac[SUFFIX];
         };
 
+        /**
+         *  Creates a new empty lease. Method from_xml should be called right
+         *  after this constructor.
+         */
+        Lease():ObjectXML(){};
+
         ~Lease(){};
 
         /**
@@ -172,14 +200,6 @@ protected:
          */
         friend ostream& operator<<(ostream& os, Lease& _lease);
 
-        /**
-         * Function to print the Lease object into a string in
-         * plain text
-         *  @param str the resulting string
-         *  @return a reference to the generated string
-         */
-        string& to_str(string& str) const;
-
         /**
          * Function to print the Lease object into a string in
          * XML format
@@ -188,6 +208,23 @@ protected:
          */
         string& to_xml(string& xml) const;
 
+        /**
+         * Function to print the Lease object into a string in
+         * XML format. The output contains all the internal attributes,
+         * to be saved in the DB as a blob
+         *  @param xml the resulting XML string
+         *  @return a reference to the generated string
+         */
+        string& to_xml_db(string& xml) const;
+
+        /**
+         *  Rebuilds the object from an xml formatted string
+         *    @param xml_str The xml-formatted string
+         *
+         *    @return 0 on success, -1 otherwise
+         */
+        int from_xml(const string &xml_str);
+
         /**
          * Constants to access the array storing the MAC address
          */
@@ -213,13 +250,13 @@ protected:
     // Leases fields
     // -------------------------------------------------------------------------
     /**
-    * Leases indentifier. Connects it to a Virtual Network
-    */
+     * Leases identifier. Connects it to a Virtual Network
+     */
     int            oid;
 
     /**
-    * Number of possible leases (free + asigned)
-    */
+     * Number of possible leases (free + assigned)
+     */
     unsigned int  size;
 
     /**
@@ -227,6 +264,16 @@ protected:
      */
     map<unsigned int, Lease *> leases;
 
+    /**
+     * Number of used leases
+     */
+    int n_used;
+
+    /**
+     *  The default MAC prefix for the Leases
+     */
+    unsigned int mac_prefix;
+
     // -------------------------------------------------------------------------
     // DataBase implementation variables
     // -------------------------------------------------------------------------
@@ -235,17 +282,6 @@ protected:
      */
     SqlDB *  db;
 
-    enum ColNames
-    {
-        OID             = 0,
-        IP              = 1,
-        MAC_PREFIX      = 2,
-        MAC_SUFFIX      = 3,
-        VID             = 4,
-        USED            = 5,
-        LIMIT           = 6
-    };
-
     static const char * table;
 
     static const char * db_names;
@@ -275,19 +311,11 @@ protected:
     friend ostream& operator<<(ostream& os, Lease& _lease);
 
     /**
-    * Function to print the Leases object into a string in
-    * plain text
-    *  @param str the resulting string
-    *  @return a reference to the generated string
-    */
-    string& to_str(string& str) const;
-
-    /**
-    * Function to print the Leases object into a string in
-    * XML format
-    *  @param xml the resulting XML string
-    *  @return a reference to the generated string
-    */
+     * Function to print the Leases object into a string in
+     * XML format
+     *  @param xml the resulting XML string
+     *  @return a reference to the generated string
+     */
     string& to_xml(string& xml) const;
 
 private:
