@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -43,11 +43,11 @@ class MySqlDB : public SqlDB
 {
 public:
 
-    MySqlDB(const string& server,
-            int           port,
-            const string& user,
-            const string& password,
-            const char *  database);
+    MySqlDB(const string& _server,
+            int           _port,
+            const string& _user,
+            const string& _password,
+            const string& _database);
 
     ~MySqlDB();
 
@@ -81,6 +81,19 @@ private:
      */
     MYSQL *             db;
 
+    /**
+     *  MySQL Connection parameters
+     */
+    string              server;
+
+    int                 port;
+
+    string              user;
+
+    string              password;
+
+    string              database;
+
     /**
      *  Fine-grain mutex for DB access
      */
@@ -113,7 +126,7 @@ public:
             int    port,
             string user,
             string password,
-            const char * database)
+            string database)
     {
         throw runtime_error("Aborting oned, MySQL support not compiled!");
     };
