@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,6 +25,8 @@
 #include "VirtualNetworkPool.h"
 #include "HostPool.h"
 #include "UserPool.h"
+#include "VMTemplatePool.h"
+#include "GroupPool.h"
 
 #include "VirtualMachineManager.h"
 #include "LifeCycleManager.h"
@@ -34,8 +36,12 @@
 #include "RequestManager.h"
 #include "HookManager.h"
 #include "AuthManager.h"
+#include "AclManager.h"
+#include "ImageManager.h"
 
-class Nebula
+#include "Callbackable.h"
+
+class Nebula : public Callbackable
 {
 public:
 
@@ -75,6 +81,16 @@ public:
         return ipool;
     };
 
+    GroupPool * get_gpool()
+    {
+        return gpool;
+    };
+
+    VMTemplatePool * get_tpool()
+    {
+        return tpool;
+    };
+
     // --------------------------------------------------------------
     // Manager Accessors
     // --------------------------------------------------------------
@@ -109,12 +125,21 @@ public:
         return hm;
     };
 
-
     AuthManager * get_authm()
     {
         return authm;
     };
 
+    ImageManager * get_imagem()
+    {
+        return imagem;
+    };
+
+    AclManager * get_aclm()
+    {
+        return aclm;
+    };
+
     // --------------------------------------------------------------
     // Environment & Configuration
     // --------------------------------------------------------------
@@ -203,9 +228,14 @@ public:
 
     static string version()
     {
-        return "OpenNebula 2.2";
+        return "OpenNebula 3.2.1";
     };
 
+    static string db_version()
+    {
+        return "3.2.1";
+    }
+
     void start();
 
     void get_configuration_attribute(
@@ -223,8 +253,9 @@ private:
     //Constructors and = are private to only access the class through instance
     // -----------------------------------------------------------------------
 
-    Nebula():nebula_configuration(0),db(0),vmpool(0),hpool(0),vnpool(0),upool(0),
-        ipool(0),lcm(0),vmm(0),im(0),tm(0),dm(0),rm(0),hm(0),authm(0)
+    Nebula():nebula_configuration(0),db(0),vmpool(0),hpool(0),vnpool(0),
+        upool(0),ipool(0),gpool(0),tpool(0),lcm(0),vmm(0),im(0),tm(0),
+        dm(0),rm(0),hm(0),authm(0),aclm(0),imagem(0)
     {
         const char * nl = getenv("ONE_LOCATION");
 
@@ -236,7 +267,6 @@ private:
             etc_location     = "/etc/one/";
             log_location     = "/var/log/one/";
             var_location     = "/var/lib/one/";
-            hook_location    = "/usr/share/one/hooks/";
             remotes_location = "/var/lib/one/remotes/";
         }
         else
@@ -252,7 +282,6 @@ private:
             etc_location     = nebula_location + "etc/";
             log_location     = nebula_location + "var/";
             var_location     = nebula_location + "var/";
-            hook_location    = nebula_location + "share/hooks/";
             remotes_location = nebula_location + "var/remotes/";
         }
     };
@@ -284,6 +313,16 @@ private:
             delete ipool;
         }
 
+        if ( gpool != 0)
+        {
+            delete gpool;
+        }
+
+        if ( tpool != 0)
+        {
+            delete tpool;
+        }
+
         if ( vmm != 0)
         {
             delete vmm;
@@ -324,6 +363,16 @@ private:
             delete authm;
         }
 
+        if ( aclm != 0)
+        {
+            delete aclm;
+        }
+
+        if ( imagem != 0)
+        {
+            delete imagem;
+        }
+
         if ( nebula_configuration != 0)
         {
             delete nebula_configuration;
@@ -358,7 +407,7 @@ private:
     // Configuration
     // ---------------------------------------------------------------
 
-    NebulaTemplate *    nebula_configuration;
+    OpenNebulaTemplate *    nebula_configuration;
 
     // ---------------------------------------------------------------
     // Nebula Pools
@@ -370,6 +419,8 @@ private:
     VirtualNetworkPool * vnpool;
     UserPool           * upool;
     ImagePool          * ipool;
+    GroupPool          * gpool;
+    VMTemplatePool     * tpool;
 
     // ---------------------------------------------------------------
     // Nebula Managers
@@ -383,12 +434,42 @@ private:
     RequestManager *        rm;
     HookManager *           hm;
     AuthManager *           authm;
+    AclManager *            aclm;
+    ImageManager *          imagem;
 
     // ---------------------------------------------------------------
     // Implementation functions
     // ---------------------------------------------------------------
 
     friend void nebula_signal_handler (int sig);
+
+    /**
+     *  Bootstraps the database control tables
+     *
+     *    @return 0 on success
+     */
+    int bootstrap();
+
+    /**
+     *  Callback function for the check_db_version method. Stores the read
+     *  version in loaded_db_version
+     *    @param _loaded_db_version returned columns
+     *    @param num the number of columns read from the DB
+     *    @param names the column names
+     *    @param vaues the column values
+     *    @return 0 on success
+     */
+    int select_cb(void *_loaded_db_version, int num, char **values,
+                  char **names);
+
+    /**
+     * Reads the current DB version.
+     *
+     * @return  0 on success,
+     *          -1 if there is a version mismatch,
+     *          -2 if the DB needs a bootstrap
+     */
+    int check_db_version();
 };
 
 #endif /*NEBULA_H_*/
