@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -45,8 +45,9 @@ public:
      *   @param _db a pointer to the database
      *   @param table the name of the table supporting the pool (to set the oid
      *   counter). If null the OID counter is not updated.
+     *   @param with_uid the Pool objects have an owner id (uid)
      */
-    PoolSQL(SqlDB * _db, const char * table);
+    PoolSQL(SqlDB * _db, const char * _table);
 
     virtual ~PoolSQL();
 
@@ -68,9 +69,32 @@ public:
      *
      *   @return a pointer to the object, 0 in case of failure
      */
-    virtual PoolObjectSQL * get(
-        int     oid,
-        bool    lock);
+    PoolObjectSQL * get(int oid, bool lock);
+
+    /**
+     *  Gets an object from the pool (if needed the object is loaded from the
+     *  database).
+     *   @param name of the object
+     *   @param uid id of owner
+     *   @param lock locks the object if true
+     *
+     *   @return a pointer to the object, 0 in case of failure
+     */
+    PoolObjectSQL * get(const string& name, int uid, bool lock);
+
+    /**
+     * Updates the cache name index. Must be called when the owner of an object
+     * is changed
+     *
+     * @param old_name Object's name before the change
+     * @param old_uid Object's owner ID before the change
+     * @param new_name Object's name after the change
+     * @param new_uid Object's owner ID after the change
+     */
+    void update_cache_index(string& old_name,
+                            int     old_uid,
+                            string& new_name,
+                            int     new_uid);
 
     /**
      *  Finds a set objects that satisfies a given condition
@@ -111,12 +135,20 @@ public:
      *  Drops the object's data in the data base. The object mutex SHOULD be
      *  locked.
      *    @param objsql a pointer to the object
-     *    @return 0 on success.
+     *    @param error_msg Error reason, if any
+     *    @return 0 on success, -1 DB error
      */
-    virtual int drop(
-        PoolObjectSQL * objsql)
+    virtual int drop(PoolObjectSQL * objsql, string& error_msg)
     {
-       return objsql->drop(db);
+        int rc = objsql->drop(db);
+
+        if ( rc != 0 )
+        {
+            error_msg = "SQL DB error";
+            return -1;
+        }
+
+        return 0;
     };
 
     /**
@@ -141,28 +173,76 @@ protected:
      */
     SqlDB * db;
 
+    /**
+     *  Dumps the pool in XML format. A filter can be also added to the
+     *  query
+     *  @param oss the output stream to dump the pool contents
+     *  @param elem_name Name of the root xml pool name
+     *  @param table Pool table name
+     *  @param where filter for the objects, defaults to all
+     *
+     *  @return 0 on success
+     */
+    int dump(ostringstream& oss, const string& elem_name,
+             const char * table, const string& where);
+
+    /* ---------------------------------------------------------------------- */
+    /* Interface to access the lastOID assigned by the pool                   */
+    /* ---------------------------------------------------------------------- */
+
+    /**
+     *  Gets the value of the last identifier assigned by the pool
+     *   @return the lastOID of the pool
+     */
+    int get_lastOID()
+    {
+        return lastOID;
+    };
+
+    /**
+     *  Sets the lastOID of the pool and updates the control database
+     *    @param _lastOID for the pool
+     */
+    void set_update_lastOID(int _lastOID)
+    {
+        lastOID = _lastOID;
+
+        update_lastOID();
+    };
+
 private:
 
-    pthread_mutex_t             mutex;
+    pthread_mutex_t mutex;
 
     /**
      *  Max size for the pool, to control the memory footprint of the pool. This
      *  number MUST be greater than the max. number of objects that are
      *  accessed simultaneously.
      */
-    static const unsigned int   MAX_POOL_SIZE;
+    static const unsigned int MAX_POOL_SIZE;
 
     /**
      *  Last object ID assigned to an object. It must be initialized by the
      *  target pool.
      */
-    int                         lastOID;
+    int lastOID;
+
+    /**
+     *  Tablename for this pool
+     */
+    string table;
 
     /**
      *  The pool is implemented with a Map of SQL object pointers, using the
      *  OID as key.
      */
-    map<int,PoolObjectSQL *>    pool;
+    map<int,PoolObjectSQL *> pool;
+
+    /**
+     *  This is a name index for the pool map. The key is the name of the object
+     *  , that may be combained with the owner id.
+     */
+    map<string,PoolObjectSQL *> name_pool;
 
     /**
      *  Factory method, must return an ObjectSQL pointer to an allocated pool
@@ -174,7 +254,7 @@ private:
      *  OID queue to implement a FIFO-like replacement policy for the pool
      *  cache.
      */
-    queue<int>                  oid_queue;
+    queue<int> oid_queue;
 
     /**
      *  Function to lock the pool
@@ -200,6 +280,30 @@ private:
      */
     void replace();
 
+    /**
+     *  Generate an index key for the object
+     *    @param name of the object
+     *    @param uid owner of the object, only used if needed
+     *    
+     *    @return the key, a string
+     */
+    virtual string key(const string& name, int uid)
+    {
+        ostringstream key;
+
+        key << name << ':' << uid;
+
+        return key.str();
+    };
+
+    /**
+     *  Inserts the last oid into the pool_control table
+     */
+    void update_lastOID();
+
+    /* ---------------------------------------------------------------------- */
+    /* ---------------------------------------------------------------------- */
+
     /**
      *  Callback to set the lastOID (PoolSQL::PoolSQL)
      */
@@ -209,6 +313,15 @@ private:
      *  Callback to store the IDs of pool objects (PoolSQL::search)
      */
     int  search_cb(void *_oids, int num, char **values, char **names);
+
+    /**
+     *  Callback function to get output in XML format
+     *    @param num the number of columns read from the DB
+     *    @param names the column names
+     *    @param vaues the column values
+     *    @return 0 on success
+     */
+    int dump_cb(void * _oss, int num, char **values, char **names);
 };
 
 #endif /*POOL_SQL_H_*/
