@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -47,6 +47,20 @@ public:
                  separator(_separator),
                  xml_root(_xml_root){};
 
+    Template(const Template& t)
+    {
+        multimap<string,Attribute *>::const_iterator it;
+
+        replace_mode = t.replace_mode;
+        separator    = t.separator;
+        xml_root     = t.xml_root;
+
+        for (it = t.attributes.begin() ; it != t.attributes.end() ; it++)
+        {
+            attributes.insert(make_pair(it->first,(it->second)->clone()));
+        }
+    }
+
     /**
      *  The class destructor frees all the attributes conforming the template
      */
@@ -114,24 +128,23 @@ public:
      *  attributes MUST be freed by the calling funtion
      *    @param name of the attribute
      *    @param values a vector containing a pointer to the attributes
-     *    @returns the number of attributes removed
+     *    @return the number of attributes removed
      */
     virtual int remove(
         const string&        name,
         vector<Attribute *>& values);
 
-
     /**
      *  Removes an attribute from the template, and frees the attributes.
      *    @param name of the attribute
-     *    @returns the number of attributes removed
+     *    @return the number of attributes removed
      */
     virtual int erase(const string& name);
 
     /**
      *  Gets all the attributes with the given name.
      *    @param name the attribute name.
-     *    @returns the number of elements in the vector
+     *    @return the number of elements in the vector
      */
     virtual int get(
         const string& name,
@@ -140,7 +153,7 @@ public:
     /**
      *  Gets all the attributes with the given name,  non-const version
      *    @param name the attribute name.
-     *    @returns the number of elements in the vector
+     *    @return the number of elements in the vector
      */
     virtual int get(
         const string& name,
@@ -153,7 +166,7 @@ public:
      *    defined or not Single
      */
     virtual void get(
-        string& name,
+        const string& name,
         string& value) const;
 
     /**
@@ -161,9 +174,11 @@ public:
      *    @param name the attribute name.
      *    @param value the attribute value, an int, 0 if the attribute is not
      *    defined or not Single
+     *
+     *    @return True if the Single attribute was found
      */
-    virtual void get(
-        string& name,
+    virtual bool get(
+        const string& name,
         int&    value) const;
 
     friend ostream& operator<<(ostream& os, const Template& t);
@@ -176,6 +191,14 @@ public:
      */
     int from_xml(const string &xml_str);
 
+    /**
+     *  Rebuilds the object from an xml node
+     *    @param node The xml node pointer
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int from_xml_node(const xmlNodePtr node);
+
 protected:
     /**
      *  The template attributes
@@ -215,6 +238,12 @@ private:
      *  Name of the Root element for the XML document
      */
     string							xml_root;
+
+    /**
+     *  Builds the template attribute from the node
+     *    @param root_element The xml element to build the template from.
+     */
+    void rebuild_attributes(const xmlNode * root_element);
 };
 
 /* -------------------------------------------------------------------------- */
