@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -18,6 +18,7 @@
 #define USER_H_
 
 #include "PoolSQL.h"
+#include "UserTemplate.h"
 
 using namespace std;
 
@@ -32,16 +33,14 @@ class User : public PoolObjectSQL
 public:
 
     /**
-     *  Function to write a User on an output stream
+     *  Characters that can not be in a name
      */
-    friend ostream& operator<<(ostream& os, User& u);
+    static const string INVALID_NAME_CHARS;
 
     /**
-     * Function to print the User object into a string in plain text
-     *  @param str the resulting string
-     *  @return a reference to the generated string
+     *  Characters that can not be in a password
      */
-    string& to_str(string& str) const;
+    static const string INVALID_PASS_CHARS;
 
     /**
      * Function to print the User object into a string in XML format
@@ -50,15 +49,6 @@ public:
      */
     string& to_xml(string& xml) const;
 
-    /**
-     * Get the User unique identifier UID, that matches the OID of the object
-     *    @return UID User identifier
-     */
-    int get_uid() const
-    {
-        return oid;
-    };
-
     /**
      *  Check if the user is enabled
      *    @return true if the user is enabled
@@ -68,18 +58,9 @@ public:
         return enabled;
      }
 
-    /**
-     *  Returns user username
-     *     @return username User's hostname
-     */
-    const string& get_username() const
-    {
-        return username;
-    };
-
     /**
      *  Returns user password
-     *     @return username User's hostname
+     *     @return the User's password
      */
     const string& get_password() const
     {
@@ -100,22 +81,73 @@ public:
     void disable()
     {
         enabled = false;
+        invalidate_session();
     };
 
     /**
-     *  Sets user username
+     *  Checks if a name is valid, i.e. it is not empty and does not
+     *  contain invalid characters.
+     *    @param uname Name to be checked
+     *    @param error_str Returns the error reason, if any
+     *    @return true if the string is valid
      */
-    void set_username(string _username)
+    static bool name_is_valid(const string& uname, string& error_str);
+
+    /**
+     *  Checks if a password is valid, i.e. it is not empty and does not
+     *  contain invalid characters.
+     *    @param pass Password to be checked
+     *    @param error_str Returns the error reason, if any
+     *    @return true if the string is valid
+     */
+    static bool pass_is_valid(const string& pass, string& error_str);
+
+    /**
+     *  Sets user password. It checks that the new password does not contain
+     *  forbidden chars.
+     *    @param _password the new pass
+     *    @param error_str Returns the error reason, if any
+     *    @returns -1 if the password is not valid
+     */
+    int set_password(const string& passwd, string& error_str)
     {
-        username = _username;
+        int rc = 0;
+
+        if (pass_is_valid(passwd, error_str))
+        { 
+            password = passwd;
+            invalidate_session();
+        }
+        else
+        {
+            rc = -1;
+        }
+
+        return rc;
     };
 
     /**
-     *  Sets user password
+     *  Returns user password
+     *     @return the user's auth driver
      */
-    void set_password(string _password)
+    const string& get_auth_driver() const
     {
-        password = _password;
+        return auth_driver;
+    };
+
+    /**
+     *  Sets the user auth driver.
+     *
+     *    @param _auth_driver the new auth. driver
+     *    @param error_str Returns the error reason, if any
+     *    @return 0 on success, -1 otherwise
+     */
+    int set_auth_driver(const string& _auth_driver, string& error_str)
+    {
+        auth_driver = _auth_driver;
+        invalidate_session();
+
+        return 0;
     };
 
     /**
@@ -128,12 +160,13 @@ public:
     static int split_secret(const string secret, string& user, string& pass);
 
     /**
-     *  "Encrypts" the password with SHA1 digest
-     *  @param password
-     *  @return sha1 encrypted password
+     *  Factory method for image templates
      */
-    static string sha1_digest(const string& pass);
-
+    Template * get_new_template()
+    {
+        return new UserTemplate;
+    }
+    
 private:
     // -------------------------------------------------------------------------
     // Friends
@@ -145,21 +178,70 @@ private:
     // User Attributes
     // -------------------------------------------------------------------------
 
-    /**
-     *  User's username
-     */
-    string      username;
-
     /**
      *  User's password
      */
     string      password;
 
+    /**
+     *  Authentication driver for this user
+     */
+    string      auth_driver;
+
     /**
      * Flag marking user enabled/disabled
      */
     bool        enabled;
 
+    // *************************************************************************
+    // Authentication session (Private)
+    // *************************************************************************
+
+    /**
+     * Until when the session_token is valid
+     */
+    time_t session_expiration_time;
+
+    /**
+     * Last authentication token validated by the driver, can
+     * be trusted until the session_expiration_time
+     */
+    string session_token;
+
+    /**
+     * Checks if a session token is authorized and still valid
+     *
+     * @param token The authentication token
+     * @return true if the token is still valid
+     */
+    bool valid_session(const string& token)
+    {
+        return (( session_token == token ) &&
+                ( time(0) < session_expiration_time ) );
+    };
+
+    /**
+     * Resets the authentication session
+     */
+    void invalidate_session()
+    {
+        session_token.clear();
+        session_expiration_time = 0;
+    };
+
+    /**
+     * Stores the given session token for a limited time. This eliminates the
+     * need to call the external authentication driver until the time expires.
+     *
+     * @param token The authenticated token
+     * @param validity_time
+     */
+    void set_session(const string& token, time_t validity_time)
+    {
+        session_token           = token;
+        session_expiration_time = time(0) + validity_time;
+    };
+
     // *************************************************************************
     // DataBase implementation (Private)
     // *************************************************************************
@@ -168,98 +250,92 @@ private:
      *  Execute an INSERT or REPLACE Sql query.
      *    @param db The SQL DB
      *    @param replace Execute an INSERT or a REPLACE
+     *    @param error_str Returns the error reason, if any
      *    @return 0 one success
      */
-    int insert_replace(SqlDB *db, bool replace);
-
-    /**
-     *  Callback function to unmarshall a User object (User::select)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int select_cb(void *nil, int num, char **values, char **names);
+    int insert_replace(SqlDB *db, bool replace, string& error_str);
 
     /**
      *  Bootstraps the database table(s) associated to the User
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * db)
+    static int bootstrap(SqlDB * db)
     {
         ostringstream oss_user(User::db_bootstrap);
 
-        db->exec(oss_user);
+        return db->exec(oss_user);
     };
 
+    /**
+     *  Rebuilds the object from an xml formatted string
+     *    @param xml_str The xml-formatted string
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int from_xml(const string &xml_str);
+
+
 protected:
 
     // *************************************************************************
     // Constructor
     // *************************************************************************
 
-    User(int     id=-1,
-         string _username="",
-         string _password="",
-         bool   _enabled=true);
+    User(int           id, 
+         int           _gid, 
+         const string& _uname, 
+         const string& _gname,
+         const string& _password,
+         const string& _auth_driver,
+         bool          _enabled):
+        PoolObjectSQL(id,USER,_uname,-1,_gid,"",_gname,table),
+        password(_password),
+        auth_driver(_auth_driver),
+        enabled(_enabled),
+        session_expiration_time(0),
+        session_token("")
+    {
+        obj_template = new UserTemplate;
+    };
 
-    virtual ~User();
+    virtual ~User()
+    {
+        if (obj_template != 0)
+        {
+            delete obj_template;
+        }
+    };
 
     // *************************************************************************
     // DataBase implementation
     // *************************************************************************
 
-    enum ColNames
-    {
-        OID             = 0,
-        USERNAME        = 1,
-        PASSWORD        = 2,
-        ENABLED         = 3,     // 0 = false, 1 = true
-        LIMIT           = 4
-    };
-
     static const char * db_names;
 
     static const char * db_bootstrap;
 
     static const char * table;
 
-    /**
-     *  Reads the User (identified with its OID=UID) from the database.
-     *    @param db pointer to the db
-     *    @return 0 on success
-     */
-    virtual int select(SqlDB *db);
-
     /**
      *  Writes the User in the database.
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    virtual int insert(SqlDB *db, string& error_str);
+    int insert(SqlDB *db, string& error_str)
+    {
+        return insert_replace(db, false, error_str);
+    };
 
     /**
      *  Writes/updates the User data fields in the database.
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    virtual int update(SqlDB *db);
-
-    /**
-     *  Drops USer from the database
-     *    @param db pointer to the db
-     *    @return 0 on success
-     */
-    virtual int drop(SqlDB *db);
-
-    /**
-     *  Function to output a User object in to an stream in XML format
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    static int dump(ostringstream& oss, int num, char **values, char **names);
+    int update(SqlDB *db)
+    {
+        string error_str;
+        return insert_replace(db, true, error_str);
+    }
 };
 
 #endif /*USER_H_*/
