@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,6 +19,7 @@
 
 #include "PoolSQL.h"
 #include "User.h"
+#include "GroupPool.h"
 
 #include <time.h>
 #include <sstream>
@@ -38,7 +39,8 @@ class UserPool : public PoolSQL
 {
 public:
 
-    UserPool(SqlDB * db);
+    UserPool(SqlDB * db,
+             time_t  __session_expiration_time);
 
     ~UserPool(){};
 
@@ -49,8 +51,11 @@ public:
      */
     int allocate (
         int *   oid,
-        string  hostname,
-        string  password,
+        int     gid,
+        const string& uname,
+        const string& gname,
+        const string& password,
+        const string& auth,
         bool    enabled,
         string& error_str);
 
@@ -59,17 +64,12 @@ public:
      *  it is loaded from the DB
      *    @param oid User unique id
      *    @param lock locks the User mutex
-     *    @return a pointer to the Host, 0 if the User could not be loaded
+     *    @return a pointer to the User, 0 if the User could not be loaded
      */
-    User * get(
-        int     oid,
-        bool    lock)
+    User * get(int oid, bool lock)
     {
-        User * user = static_cast<User *>(PoolSQL::get(oid,lock));
-
-        return user;
-    }
-
+        return static_cast<User *>(PoolSQL::get(oid,lock));
+    };
 
     /**
      *  Function to get a User from the pool, if the object is not in memory
@@ -78,21 +78,24 @@ public:
      *    @param lock locks the User mutex
      *    @return a pointer to the User, 0 if the User could not be loaded
      */
-    User * get(
-        string  username,
-        bool    lock)
+    User * get(string name, bool lock)
     {
-        map<string, int>::iterator     index;
+        // The owner is set to -1, because it is not used in the key() method
+        return static_cast<User *>(PoolSQL::get(name,-1,lock));
+    };
 
-        index = known_users.find(username);
-
-        if ( index != known_users.end() )
-        {
-            return get((int)index->second,lock);
-        }
-
-        return 0;
-    }
+    /**
+     *  Generate an index key for the object
+     *    @param name of the object
+     *    @param uid owner of the object, only used if needed
+     *
+     *    @return the key, a string
+     */
+    string key(const string& name, int uid)
+    {
+        // Name is enough key because Users can't repeat names.
+        return name;
+    };
 
     /** Update a particular User
      *    @param user pointer to User
@@ -103,37 +106,30 @@ public:
         return user->update(db);
     };
 
-
-    /** Drops a user from the DB, the user mutex MUST BE locked
-     *    @param user pointer to User
-     */
-    int drop(User * user)
-    {
-        int rc = PoolSQL::drop(user);
-
-        if ( rc == 0)
-        {
-            known_users.erase(user->get_username());
-        }
-
-        return rc;
-    };
-
     /**
      *  Bootstraps the database table(s) associated to the User pool
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * _db)
+    static int bootstrap(SqlDB * _db)
     {
-        User::bootstrap(_db);
+        return User::bootstrap(_db);
     };
 
     /**
      * Returns whether there is a user with given username/password or not
      *   @param session, colon separated username and password string
-     *   @return -1 if authn failed, uid of the user in other case
+     *   @param uid of the user if authN succeeded -1 otherwise
+     *   @param gid of the user if authN succeeded -1 otherwise
+     *   @param uname of the user if authN succeeded "" otherwise
+     *   @param gname of the group if authN succeeded "" otherwise
+     *
+     *   @return false if authn failed, true otherwise
      */
-    int authenticate(string& session);
-
+    bool authenticate(const string& session, 
+                      int&          uid, 
+                      int&          gid,
+                      string&       uname,
+                      string&       gname);
     /**
      * Returns whether there is a user with given username/password or not
      *   @param ar, an Authorization Request
@@ -149,42 +145,86 @@ public:
      *
      *  @return 0 on success
      */
-    int dump(ostringstream& oss, const string& where);
+    int dump(ostringstream& oss, const string& where)
+    {
+        return PoolSQL::dump(oss, "USER_POOL", User::table, where);
+    };
+
+    /**
+     *  Name for the OpenNebula core authentication process 
+     */
+    static const char * CORE_AUTH;
+
+    /**
+     *  Name for the OpenNebula server (delegated) authentication process 
+     */
+    static const char * SERVER_AUTH;
+
+    /**
+     *  Name for the OpenNebula public authentication process. It only
+     *  allows delegated
+     */
+    static const char * PUBLIC_AUTH;
+
+    /**
+     *  Name for the default auth driver to be used for not registered users
+     */
+    static const char * DEFAULT_AUTH;
+
+    /**
+     *  Name for the default Sunstone server user
+     */
+    static const char * SERVER_NAME;
 
 private:
+    //--------------------------------------------------------------------------
+    // Configuration Attributes for Users
+    // -------------------------------------------------------------------------
+
+    /**
+     * Authentication session expiration time
+     **/
+    static time_t _session_expiration_time;
+
+    /**
+     *  Function to authenticate internal (known) users
+     */
+    bool authenticate_internal(User *        user,
+                               const string& token,
+                               int&          user_id,
+                               int&          group_id,
+                               string&       uname,
+                               string&       gname);
+
+    /**
+     *  Function to authenticate internal users using a server driver
+     */
+    bool authenticate_server(User *        user,
+                             const string& token,
+                             int&          user_id,
+                             int&          group_id,
+                             string&       uname,
+                             string&       gname);
+
+    
+    /**
+     *  Function to authenticate external (not known) users
+     */
+    bool authenticate_external(const string& username,
+                               const string& token,
+                               int&    user_id,
+                               int&    group_id,
+                               string& uname,
+                               string& gname);
     /**
      *  Factory method to produce User objects
      *    @return a pointer to the new User
      */
     PoolObjectSQL * create()
     {
-        return new User;
+        return new User(-1,-1,"","","",UserPool::CORE_AUTH,true);
     };
 
-    /**
-     *  This map stores the association between UIDs and Usernames
-     */
-    map<string, int>	known_users;
-
-    /**
-     *  Callback function to get output the user pool in XML format
-     *  (User::dump)
-     *    @param _oss pointer to the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int dump_cb(void * _oss, int num, char **values, char **names);
-
-    /**
-     *  Callback function to build the knwon_user map (User::User)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int init_cb(void *nil, int num, char **values, char **names);
 };
 
 #endif /*USER_POOL_H_*/
