@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -28,6 +28,8 @@
 
 using namespace std;
 
+class AuthRequest;
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
@@ -114,19 +116,6 @@ public:
         }
     };
 
-    /**
-     *  Function to write a Virtual Machine in an output stream
-     */
-    friend ostream& operator<<(ostream& os, const VirtualMachine& vm);
-
-    /**
-     * Function to print the VirtualMachine object into a string in
-     * plain text
-     *  @param str the resulting string
-     *  @return a reference to the generated string
-     */
-    string& to_str(string& str) const;
-
     /**
      * Function to print the VirtualMachine object into a string in
      * XML format
@@ -135,6 +124,22 @@ public:
      */
     string& to_xml(string& xml) const;
 
+    /**
+     * Function to print the VirtualMachine object into a string in
+     * XML format, with extended information (full history records)
+     *  @param xml the resulting XML string
+     *  @return a reference to the generated string
+     */
+    string& to_xml_extended(string& xml) const;
+
+    /**
+     *  Rebuilds the object from an xml formatted string
+     *    @param xml_str The xml-formatted string
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int from_xml(const string &xml_str);
+
     // ------------------------------------------------------------------------
     // Dynamic Info
     // ------------------------------------------------------------------------
@@ -183,16 +188,6 @@ public:
         }
     };
 
-
-    /**
-     *  Returns the name of the VM
-     *    @return the VM name
-     */
-    const string& get_name() const
-    {
-        return name;
-    };
-
     /**
      *  Returns the deployment ID
      *    @return the VMM driver specific ID
@@ -218,11 +213,12 @@ public:
      *  Adds a new history record an writes it in the database.
      */
     void add_history(
-        int                         hid,
-        string&                     hostname,
-        string&                     vm_dir,
-        string&                     vmm_mad,
-        string&                     tm_mad);
+        int     hid,
+        const string& hostname,
+        const string& vm_dir,
+        const string& vmm_mad,
+        const string& vnm_mad,
+        const string& tm_mad);
 
     /**
      *  Duplicates the last history record. Only the host related fields are
@@ -278,6 +274,26 @@ public:
         return previous_history->vmm_mad_name;
     };
 
+    /**
+     *  Returns the VNM driver name for the current host. The hasHistory()
+     *  function MUST be called before this one.
+     *    @return the VNM mad name
+     */
+    const string & get_vnm_mad() const
+    {
+        return history->vnm_mad_name;
+    };
+
+    /**
+     *  Returns the VNM driver name for the previous host. The hasPreviousHistory()
+     *  function MUST be called before this one.
+     *    @return the VNM mad name
+     */
+    const string & get_previous_vnm_mad() const
+    {
+        return previous_history->vnm_mad_name;
+    };
+
     /**
      *  Returns the TM driver name for the current host. The hasHistory()
      *  function MUST be called before this one.
@@ -542,93 +558,6 @@ public:
     // ------------------------------------------------------------------------
     // Template
     // ------------------------------------------------------------------------
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        string& name,
-        vector<const Attribute*>& values) const
-    {
-        return vm_template->get(name,values);
-    };
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        const char *name,
-        vector<const Attribute*>& values) const
-    {
-        string str=name;
-        return vm_template->get(str,values);
-    };
-
-    /**
-     *  Gets a string based VM attribute (single)
-     *    @param name of the attribute
-     *    @param value of the attribute (a string), will be "" if not defined or
-     *    not a single attribute
-     */
-    void get_template_attribute(
-        const char *    name,
-        string&         value) const
-    {
-        string str=name;
-        vm_template->get(str,value);
-    }
-
-    /**
-     *  Gets an int based VM attribute (single)
-     *    @param name of the attribute
-     *    @param value of the attribute (an int), will be 0 if not defined or
-     *    not a single attribute
-     */
-    void get_template_attribute(
-        const char *    name,
-        int&            value) const
-    {
-        string str=name;
-        vm_template->get(str,value);
-    }
-
-    /**
-     *  Adds a new attribute to the template (replacing it if
-     *  already defined), the vm's mutex SHOULD be locked
-     *    @param name of the new attribute
-     *    @param value of the new attribute
-     *    @return 0 on success
-     */
-    int replace_template_attribute(
-        string& name,
-        string& value)
-    {
-        SingleAttribute * sattr;
-
-        vm_template->erase(name);
-
-        sattr = new SingleAttribute(name,value);
-        vm_template->set(sattr);
-
-        return 0;
-    }
-
-
-    /**
-     *  Generates a XML string for the template of the VM
-     *    @param xml the string to store the XML description.
-     */
-    void template_to_xml(string &xml) const
-    {
-        vm_template->to_xml(xml);
-    }
-
     /**
      *  Parse a string and substitute variables (e.g. $NAME) using the VM
      *  template values:
@@ -636,12 +565,19 @@ public:
      *    @param parsed, the resulting parsed string
      *    @return 0 on success.
      */
-    int  parse_template_attribute(const string& attribute,
-                                  string&       parsed);
+    int  parse_template_attribute(const string& attribute, string& parsed);
+    
+    /**
+     *  Factory method for virtual machine templates
+     */
+    Template * get_new_template()
+    {
+        return new VirtualMachineTemplate;
+    }
+
     // ------------------------------------------------------------------------
     // States
     // ------------------------------------------------------------------------
-
     /**
      *  Returns the VM state (Dispatch Manager)
      *    @return the VM state
@@ -678,16 +614,6 @@ public:
         lcm_state = s;
     };
 
-    /**
-     *  Gets the user id of the owner of this VM
-     *    @return the VM uid
-     */
-    int get_uid() const
-    {
-        return uid;
-    };
-
-
     // ------------------------------------------------------------------------
     // Timers
     // ------------------------------------------------------------------------
@@ -724,7 +650,7 @@ public:
      *  Get all network leases for this Virtual Machine
      *  @return 0 if success
      */
-    int get_network_leases();
+    int get_network_leases(string &error_str);
 
     /**
      *  Releases all network leases taken by this Virtual Machine
@@ -733,6 +659,7 @@ public:
 
     /**
      *  Get all disk images for this Virtual Machine
+     *  @param error_str Returns the error reason, if any
      *  @return 0 if success
      */
     int get_disk_images(string &error_str);
@@ -753,7 +680,6 @@ public:
      */
     int  generate_context(string &files);
 
-
     // ------------------------------------------------------------------------
     // Image repository related functions
     // ------------------------------------------------------------------------
@@ -763,8 +689,21 @@ public:
      *    @param  img_id ID of the image this disk will be saved to.
      *    @return 0 if success
      */
-    int  save_disk(int disk_id, int img_id);
+    int  save_disk(int disk_id, int img_id, string& error_str);
 
+    // ------------------------------------------------------------------------
+    // Authorization related functions
+    // ------------------------------------------------------------------------
+    /**
+     *  Sets an authorization request for a VirtualMachine template based on
+     *  the images and networks used
+     *    @param  uid for template owner
+     *    @param  ar the AuthRequest object
+     *    @param  tmpl the virtual machine template
+     */
+    static void set_auth_request(int uid, 
+                                 AuthRequest& ar, 
+                                 VirtualMachineTemplate *tmpl);
 private:
 
     // -------------------------------------------------------------------------
@@ -776,15 +715,6 @@ private:
     // Virtual Machine Attributes
     // *************************************************************************
 
-    // -------------------------------------------------------------------------
-    // Identification variables
-    // -------------------------------------------------------------------------
-
-    /**
-     *  User (owner) id
-     */
-    int         uid;
-
     // -------------------------------------------------------------------------
     // VM Scheduling & Managing Information
     // -------------------------------------------------------------------------
@@ -796,19 +726,6 @@ private:
     // -------------------------------------------------------------------------
     // Virtual Machine Description
     // -------------------------------------------------------------------------
-
-    /**
-     * Name of the VM
-     */
-    string                  name;
-
-    /**
-     *  The Virtual Machine template, holds the VM attributes.
-     */
-    VirtualMachineTemplate* vm_template;
-
-    // Dynamic state of the Virtual Machine
-
     /**
      *  The state of the virtual machine.
      */
@@ -854,11 +771,6 @@ private:
      */
     int         net_rx;
 
-    /**
-     *  Sequence number of the last history item.
-     */
-    int         last_seq;
-
     /**
      *  History record, for the current host
      */
@@ -869,6 +781,12 @@ private:
      */
     History *   previous_history;
 
+
+    /**
+     *  Complete set of history records for the VM
+     */
+    vector<History *> history_records;
+
     // -------------------------------------------------------------------------
     // Logging
     // -------------------------------------------------------------------------
@@ -887,14 +805,19 @@ private:
 
     /**
      *  Bootstraps the database table(s) associated to the VirtualMachine
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * db)
+    static int bootstrap(SqlDB * db)
     {
+        int rc;
+
         ostringstream oss_vm(VirtualMachine::db_bootstrap);
         ostringstream oss_hist(History::db_bootstrap);
 
-        db->exec(oss_vm);
-        db->exec(oss_hist);
+        rc =  db->exec(oss_vm);
+        rc += db->exec(oss_hist);
+
+        return rc;
     };
 
     /**
@@ -911,9 +834,10 @@ private:
      *  Execute an INSERT or REPLACE Sql query.
      *    @param db The SQL DB
      *    @param replace Execute an INSERT or a REPLACE
+     *    @param error_str Returns the error reason, if any
      *    @return 0 one success
-    */
-    int insert_replace(SqlDB *db, bool replace);
+     */
+    int insert_replace(SqlDB *db, bool replace, string& error_str);
 
     /**
      *  Updates the VM history record
@@ -945,7 +869,6 @@ private:
             return -1;
     };
 
-
     // -------------------------------------------------------------------------
     // Attribute Parser
     // -------------------------------------------------------------------------
@@ -958,16 +881,18 @@ private:
     /**
      *  Parse the "CONTEXT" attribute of the template by substituting
      *  $VARIABLE, $VARIABLE[ATTR] and $VARIABLE[ATTR, ATTR = VALUE]
+     *    @param error_str Returns the error reason, if any
      *    @return 0 on success
      */
-    int parse_context();
+    int parse_context(string& error_str);
 
     /**
      *  Parse the "REQUIREMENTS" attribute of the template by substituting
      *  $VARIABLE, $VARIABLE[ATTR] and $VARIABLE[ATTR, ATTR = VALUE]
+     *    @param error_str Returns the error reason, if any
      *    @return 0 on success
      */
-    int parse_requirements();
+    int parse_requirements(string& error_str);
 
     /**
      *  Parse the "GRAPHICS" attribute and generates a default PORT if not
@@ -975,13 +900,27 @@ private:
      */
     void parse_graphics();
 
+    /**
+     *  Function that renders the VM in XML format optinally including
+     *  extended information (all history records)
+     *  @param xml the resulting XML string
+     *  @param extended include additional info if true
+     *  @return a reference to the generated string
+     */
+    string& to_xml_extended(string& xml, bool extended) const;
+
 protected:
 
     //**************************************************************************
     // Constructor
     //**************************************************************************
 
-    VirtualMachine(int id=-1, VirtualMachineTemplate * _vm_template = 0);
+    VirtualMachine(int id, 
+                   int uid,
+                   int gid, 
+                   const string& uname,
+                   const string& gname,
+                   VirtualMachineTemplate * _vm_template);
 
     virtual ~VirtualMachine();
 
@@ -989,32 +928,10 @@ protected:
     // DataBase implementation
     // *************************************************************************
 
-    enum ColNames
-    {
-        OID             = 0,
-        UID             = 1,
-        NAME            = 2,
-        LAST_POLL       = 3,
-        STATE           = 4,
-        LCM_STATE       = 5,
-        STIME           = 6,
-        ETIME           = 7,
-        DEPLOY_ID       = 8,
-        MEMORY          = 9,
-        CPU             = 10,
-        NET_TX          = 11,
-        NET_RX          = 12,
-        LAST_SEQ        = 13,
-        TEMPLATE        = 14,
-        LIMIT           = 15
-    };
-
     static const char * table;
 
     static const char * db_names;
 
-    static const char * extended_db_names;
-
     static const char * db_bootstrap;
 
     /**
@@ -1029,48 +946,29 @@ protected:
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    virtual int insert(SqlDB * db, string& error_str);
+    int insert(SqlDB * db, string& error_str);
 
     /**
      *  Writes/updates the Virtual Machine data fields in the database.
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    virtual int update(SqlDB * db);
+    int update(SqlDB * db)
+    {
+        string error_str;
+        return insert_replace(db, true, error_str);
+    }
 
     /**
      * Deletes a VM from the database and all its associated information
      *   @param db pointer to the db
      *   @return -1
      */
-    virtual int drop(SqlDB * db)
+    int drop(SqlDB * db)
     {
         NebulaLog::log("ONE",Log::ERROR, "VM Drop not implemented!");
         return -1;
     }
-
-    /**
-     *  Dumps the contect of a set of VirtualMachine objects in the given stream
-     *  using XML format
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    static int dump(ostringstream& oss, int num, char ** values, char ** names);
-
-    /**
-     *  Dumps the contect of a set of VirtualMachine objects in the given stream
-     *  using XML format
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    static int dump_extended(   ostringstream& oss,
-                                int num, char ** values, char ** names);
 };
 
 #endif /*VIRTUAL_MACHINE_H_*/
