@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -34,25 +34,31 @@ public:
 
     VirtualMachinePool(SqlDB * db,
                        vector<const Attribute *> hook_mads,
-                       const string& hook_location);
+                       const string& hook_location,
+                       const string& remotes_location);
 
     ~VirtualMachinePool(){};
 
     /**
      *  Function to allocate a new VM object
      *    @param uid user id (the owner of the VM)
+     *    @param gid the id of the group this object is assigned to
      *    @param vm_template a VM Template object describing the VM
      *    @param oid the id assigned to the VM (output)
+     *    @param error_str Returns the error reason, if any
      *    @param on_hold flag to submit on hold
      *    @return oid on success, -1 error inserting in DB or -2 error parsing
      *  the template
      */
     int allocate (
-        int     uid,
-        VirtualMachineTemplate *vm_template,
-        int *   oid,
-        string& error_str,
-        bool    on_hold = false);
+        int                      uid,
+        int                      gid,
+        const string&            uname,
+        const string&            gname,
+        VirtualMachineTemplate * vm_template,
+        int *                    oid,
+        string&                  error_str,
+        bool                     on_hold = false);
 
     /**
      *  Function to get a VM from the pool, if the object is not in memory
@@ -117,10 +123,11 @@ public:
 
     /**
      *  Bootstraps the database table(s) associated to the VirtualMachine pool
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * _db)
+    static int bootstrap(SqlDB * _db)
     {
-        VirtualMachine::bootstrap(_db);
+        return VirtualMachine::bootstrap(_db);
     };
 
     /**
@@ -134,23 +141,9 @@ public:
      */
     int dump(ostringstream& oss, const string& where)
     {
-        return dump(oss, true, -1, where);
+        return PoolSQL::dump(oss, "VM_POOL", VirtualMachine::table, where);
     }
 
-    /**
-     *  Dumps the VM pool in XML format. A filter can be also added to the query
-     *  Also the hostname where the VirtualMachine is running is added to the
-     *  pool
-     *  @param oss the output stream to dump the pool contents
-     *  @param where filter for the objects, defaults to all
-     *  @param extended condition to include history and username data
-     *  @param state include only VMs in this state. -1 means any state,
-     *              except DONE
-     *
-     *  @return 0 on success
-     */
-    int dump(ostringstream& oss, bool extended, int state, const string& where);
-
 private:
     /**
      *  Factory method to produce VM objects
@@ -158,29 +151,8 @@ private:
      */
     PoolObjectSQL * create()
     {
-        return new VirtualMachine;
+        return new VirtualMachine(-1,-1,-1,"","",0);
     };
-
-    /**
-     *  Callback function to get output the vm pool in XML format
-     *  (VirtualMachinePool::dump)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int dump_cb(void * _oss, int num, char **values, char **names);
-
-    /**
-     *  Callback function to get output the vm pool in XML format
-     *  (VirtualMachinePool::dump)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int dump_extended_cb(void * _oss, int num, char **values, char **names);
-
 };
 
 #endif /*VIRTUAL_MACHINE_POOL_H_*/
