@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,6 +19,8 @@
 
 #include "Template.h"
 
+#include <string.h>
+
 using namespace std;
 
 /**
@@ -32,7 +34,28 @@ public:
 
     ~VirtualMachineTemplate(){};
 
+    VirtualMachineTemplate(VirtualMachineTemplate& vmt):Template(vmt){};
+
+    /**
+     *  Checks the template for RESTRICTED ATTRIBUTES
+     *    @param rs_attr the first restricted attribute found if any
+     *    @return true if a restricted attribute is found in the template
+     */
+    bool check(string& rs_attr);
+
 private:
+	/**
+	 *  Number of restricted attributes
+	 */
+	const static int    RS_ATTRS_LENGTH;
+
+	/**
+	 *  Restricted template attributes in the form 
+	 *  'SINGLE' or 'VECTOR/ATTR'. Restricted attributes are only
+	 *  allowed for ONE_ADMIN Group.
+	 */
+	const static string RESTRICTED_ATTRIBUTES[];
+    
     friend class VirtualMachine;
 };
 
