@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,8 +19,8 @@
 
 
 #include "PoolSQL.h"
-#include "VirtualNetworkTemplate.h"
 #include "Leases.h"
+#include "VirtualNetworkTemplate.h"
 
 #include <vector>
 #include <string>
@@ -35,8 +35,8 @@ using namespace std;
 /* -------------------------------------------------------------------------- */
 
 /**
- *  The Virtual Network class. It represents a Virtual Network at manages its leases.
- *  One lease is formed by one IP and one MAC address.
+ *  The Virtual Network class. It represents a Virtual Network at manages its 
+ *  leases. One lease is formed by one IP and one MAC address.
  *  MAC address are derived from IP addresses.
  */
 class VirtualNetwork : public PoolObjectSQL
@@ -59,52 +59,16 @@ public:
     // *************************************************************************
 
     /**
-     * Get the Vnet unique identifier VNID, that matches the OID of the object
-     *    @return VNID Image identifier
+     *  Factory method for virtual network templates
      */
-    int get_vnid() const
+    Template * get_new_template()
     {
-        return oid;
-    };
-
-    /**
-     * Gets the uid of the owner of the Virtual Network
-     * @return uid
-     **/
-    int get_uid()
-    {
-        return uid;
-    }
-
-    /**
-     *  Returns true if the Virtual Network is public
-     *     @return true if the Virtual Network is public
-     */
-    bool isPublic()
-    {
-        return (public_vnet == 1);
-    };
-
-    /**
-     *  Publish or unpublish a virtual network
-     *    @param pub true to publish the image
-     *    @return 0 on success
-     */
-    void publish(bool pub)
-    {
-        if (pub == true)
-        {
-            public_vnet = 1;
-        }
-        else
-        {
-            public_vnet = 0;
-        }
+        return new VirtualNetworkTemplate;
     }
 
     /**
      * Adds Leases to the virtual network (Only implemented for FIXED networks)
-     *  @param leases_template template in the form LEASES = [IP=XX, MAC=XX].
+     *  @param leases template in the form LEASES = [IP=XX, MAC=XX].
      *         MAC is optional. The template can only contain one LEASE 
      *         definition.
      *  @param error_msg If the action fails, this message contains the reason.
@@ -115,7 +79,7 @@ public:
     /**
      * Removes Leases from the virtual network; if they are not used.(Only 
      * implemented for FIXED networks)
-     *  @param leases_template template in the form LEASES = [IP=XX].
+     *  @param leases template in the form LEASES = [IP=XX].
      *         The template can only contain one LEASE definition.
      *  @param error_msg If the action fails, this message contains
      *         the reason.
@@ -123,6 +87,25 @@ public:
      */
     int remove_leases(VirtualNetworkTemplate* leases, string& error_msg);
 
+    /**
+     * Holds a Lease, marking it as used
+     *  @param leases template in the form LEASES = [IP=XX].
+     *          The template can only contain one LEASE definition.
+     *  @param error_msg If the action fails, this message contains the reason.
+     *  @return 0 on success
+     */
+    int hold_leases(VirtualNetworkTemplate * leases, string& error_msg);
+
+    /**
+     * Releases a Lease on hold
+     *  @param leases template in the form LEASES = [IP=XX].
+     *          The template can only contain one LEASE definition.
+     *  @param error_msg If the action fails, this message contains
+     *         the reason.
+     *  @return 0 on success
+     */
+    int free_leases(VirtualNetworkTemplate* leases, string& error_msg);
+
     /**
      *    Gets a new lease for a specific VM
      *    @param vid VM identifier
@@ -170,19 +153,6 @@ public:
         return leases->size;
     };
 
-    /**
-     *  Function to write a Virtual Network in an output stream
-     */
-    friend ostream& operator<<(ostream& os, VirtualNetwork& vn);
-
-    /**
-     * Function to print the VirtualNetwork object into a string in
-     * plain text
-     *  @param str the resulting string
-     *  @return a reference to the generated string
-     */
-    string& to_str(string& str) const;
-
     /**
      * Function to print the VirtualNetwork object into a string in
      * XML format
@@ -191,6 +161,14 @@ public:
      */
     string& to_xml(string& xml) const;
 
+    /**
+     * Function to print the VirtualNetwork object into a string in
+     * XML format. The extended XML includes the LEASES
+     *  @param xml the resulting XML string
+     *  @return a reference to the generated string
+     */
+    string& to_xml_extended(string& xml) const;
+
     /**
      * Modifies the given nic attribute adding the following attributes:
      *  * IP:  leased from network
@@ -202,63 +180,6 @@ public:
      */
     int nic_attribute(VectorAttribute * nic, int vid);
 
-    //------------------------------------------------------------------------
-    // Template
-    // ------------------------------------------------------------------------
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        string& name,
-        vector<const Attribute*>& values) const
-    {
-        return vn_template->get(name,values);
-    };
-
-    /**
-     *  Gets the values of a template attribute
-     *    @param name of the attribute
-     *    @param values of the attribute
-     *    @return the number of values
-     */
-    int get_template_attribute(
-        const char *name,
-        vector<const Attribute*>& values) const
-    {
-        string str=name;
-        return vn_template->get(str,values);
-    };
-
-    /**
-     *  Gets a string based VN attribute
-     *    @param name of the attribute
-     *    @param value of the attribute (a string), will be "" if not defined
-     */
-    void get_template_attribute(
-        const char *    name,
-        string&         value) const
-    {
-        string str=name;
-        vn_template->get(str,value);
-    }
-
-    /**
-     *  Gets a string based VN attribute
-     *    @param name of the attribute
-     *    @param value of the attribute (an int), will be 0 if not defined
-     */
-    void get_template_attribute(
-        const char *    name,
-        int&            value) const
-    {
-        string str=name;
-        vn_template->get(str,value);
-    }
-
 private:
 
     // -------------------------------------------------------------------------
@@ -270,19 +191,6 @@ private:
     // Virtual Network Private Attributes
     // *************************************************************************
 
-    // -------------------------------------------------------------------------
-    // Identification variables
-    // -------------------------------------------------------------------------
-    /**
-     *  Name of the Virtual Network
-     */
-    string  name;
-
-    /**
-     *  Owner of the Virtual Network
-     */
-    int     uid;
-
     // -------------------------------------------------------------------------
     // Binded physical attributes
     // -------------------------------------------------------------------------
@@ -292,6 +200,21 @@ private:
      */
     string  bridge;
 
+    /**
+     *  Name of the physical device the bridge should be attached to
+     */
+    string  phydev;
+    
+    /**
+     *  VLAN ID of the NIC
+     */
+    string  vlan_id;
+
+    /**
+     *  Whether or not to isolate this network with the vnm driver
+     */
+    int     vlan;
+
     // -------------------------------------------------------------------------
     // Virtual Network Description
     // -------------------------------------------------------------------------
@@ -300,21 +223,14 @@ private:
      */
     NetworkType type;
 
-    /**
-     *  Public scope of this Virtual Network
-     */
-    int         public_vnet;
-
     /**
      *  Pointer to leases class, can be fixed or ranged.
      *  Holds information on given (and, optionally, possible) leases
      */
     Leases *    leases;
 
-    /**
-     *  The Virtual Network template, holds the VNW attributes.
-     */
-    VirtualNetworkTemplate * vn_template;
+    unsigned int ip_start;
+    unsigned int ip_end;
 
     // *************************************************************************
     // DataBase implementation (Private)
@@ -324,45 +240,54 @@ private:
      *  Execute an INSERT or REPLACE Sql query.
      *    @param db The SQL DB
      *    @param replace Execute an INSERT or a REPLACE
+     *    @param error_str Returns the error reason, if any
      *    @return 0 on success
      */
-    int insert_replace(SqlDB *db, bool replace);
+    int insert_replace(SqlDB *db, bool replace, string& error_str);
 
     /**
      *  Bootstraps the database table(s) associated to the Virtual Network
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * db)
+    static int bootstrap(SqlDB * db)
     {
+        int rc;
+
         ostringstream oss_vnet(VirtualNetwork::db_bootstrap);
         ostringstream oss_lease(Leases::db_bootstrap);
 
-        db->exec(oss_vnet);
-        db->exec(oss_lease);
+        rc =  db->exec(oss_vnet);
+        rc += db->exec(oss_lease);
+
+        return rc;
     };
 
     /**
-     *  Callback function to unmarshall a VNW object (VirtualNetwork::select)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
+     * Function to print the VirtualNetwork object into a string in
+     * XML format
+     *  @param xml the resulting XML string
+     *  @param extended If true, leases are included
+     *  @return a reference to the generated string
      */
-    int select_cb(void * nil, int num, char **values, char **names);
+    string& to_xml_extended(string& xml, bool extended) const;
 
     /**
-     *  Function to drop VN entry in vn_pool
-     *    @return 0 on success
+     *  Rebuilds the object from an xml formatted string
+     *    @param xml_str The xml-formatted string
+     *
+     *    @return 0 on success, -1 otherwise
      */
-    int vn_drop(SqlDB * db);
-
-
-protected:
+    int from_xml(const string &xml_str);
 
     //**************************************************************************
     // Constructor
     //**************************************************************************
 
-    VirtualNetwork(VirtualNetworkTemplate * _vn_template = 0);
+    VirtualNetwork(int                      uid,
+                   int                      gid,
+                   const string&            _uname,
+                   const string&            _gname,
+                   VirtualNetworkTemplate * _vn_template = 0);
 
     ~VirtualNetwork();
 
@@ -370,24 +295,10 @@ protected:
     // DataBase implementation
     // *************************************************************************
 
-	enum ColNames
-    {
-        OID             = 0,
-        UID             = 1,
-        NAME            = 2,
-        TYPE            = 3,
-        BRIDGE          = 4,
-        PUBLIC          = 5,
-        TEMPLATE        = 6,
-        LIMIT           = 7
-    };
-
     static const char * table;
 
     static const char * db_names;
 
-    static const char * extended_db_names;
-
     static const char * db_bootstrap;
 
     /**
@@ -397,6 +308,23 @@ protected:
      */
     int select(SqlDB * db);
 
+    /**
+     *  Reads the Virtual Network (identified with its OID) from the database.
+     *    @param db pointer to the db
+     *    @param name of the network
+     *    @param uid of the owner 
+     * 
+     *    @return 0 on success
+     */
+    int select(SqlDB * db, const string& name, int uid);
+
+    /**
+     *  Reads the Virtual Network leases from the database.
+     *    @param db pointer to the db
+     *    @return 0 on success
+     */
+    int select_leases(SqlDB * db);
+
     /**
      *  Writes the Virtual Network and its associated template and leases in the database.
      *    @param db pointer to the db
@@ -409,7 +337,11 @@ protected:
      *    @param db pointer to the db
      *    @return 0 on success
      */
-    int update(SqlDB * db);
+    int update(SqlDB * db)
+    {
+        string error_str;
+        return insert_replace(db, true, error_str);
+    }
 
     /**
      * Deletes a VNW from the database and all its associated information:
@@ -418,27 +350,7 @@ protected:
      *   @param db pointer to the db
      *   @return 0 on success
      */
-    int drop(SqlDB * db)
-    {
-        int rc;
-
-        rc = leases->drop(db);
-
-        rc += vn_drop(db);
-
-        return rc;
-    }
-
-    /**
-     *  Dumps the contect of a VirtualNetwork object in the given stream using
-     *  XML format
-     *    @param oss the output stream
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-     static int dump(ostringstream& oss, int num, char **values, char **names);
+    int drop(SqlDB * db);
 };
 
 #endif /*VIRTUAL_NETWORK_H_*/
