@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -42,15 +42,20 @@ public:
     /**
      *  Function to allocate a new VNET object
      *    @param uid user identifier
+     *    @param gid the id of the group this object is assigned to
      *    @param vn_template a VirtualNetworkTemplate describing the VNET
      *    @param oid the id assigned to the VM (output)
+     *    @param error_str Returns the error reason, if any
      *    @return oid on success, -1 error
      */
     int allocate (
-        int     uid,
-        VirtualNetworkTemplate * vn_template,
-        int *   oid,
-        string& error_str);
+        int                         uid,
+        int                         gid,
+        const string&               uname,
+        const string&               gname,
+        VirtualNetworkTemplate *    vn_template,
+        int *                       oid,
+        string&                     error_str);
 
     /**
      *  Function to get a VN from the pool, if the object is not in memory
@@ -59,23 +64,24 @@ public:
      *    @param lock locks the VN mutex
      *    @return a pointer to the VN, 0 if the VN could not be loaded
      */
-    VirtualNetwork * get(
-        int     oid,
-        bool    lock)
+    VirtualNetwork * get(int oid, bool lock)
     {
         return static_cast<VirtualNetwork *>(PoolSQL::get(oid,lock));
     };
 
     /**
-     *  Function to get a VN from the pool using the network name
-     *  If the object is not in memory it is loaded from the DB
-     *    @param name VN unique name
-     *    @param lock locks the VN mutex
-     *    @return a pointer to the VN, 0 if the VN could not be loaded
+     *  Gets an object from the pool (if needed the object is loaded from the
+     *  database).
+     *   @param name of the object
+     *   @param uid id of owner
+     *   @param lock locks the object if true
+     *
+     *   @return a pointer to the object, 0 in case of failure
      */
-    VirtualNetwork * get(
-        const string&  name,
-        bool           lock);
+    VirtualNetwork * get(const string& name, int uid, bool lock)
+    {
+        return static_cast<VirtualNetwork *>(PoolSQL::get(name,uid,lock));
+    };
 
     //--------------------------------------------------------------------------
     // Virtual Network DB access functions
@@ -86,23 +92,26 @@ public:
      *  metadata
      *    @param nic the nic attribute to be generated
      *    @param vid of the VM requesting the lease
-     *    @return 0 on success, -1 error, -2 not using the pool
+     *    @return 0 on success, 
+     *            -1 error, 
+     *            -2 not using the pool
      */
-    int nic_attribute(VectorAttribute * nic, int vid);
+    int nic_attribute(VectorAttribute * nic, int uid, int vid);
 
     /**
      *  Generates an Authorization token for a NIC attribute
      *    @param nic the nic to be authorized
      *    @param ar the AuthRequest
      */
-    void authorize_nic(VectorAttribute * nic, AuthRequest * ar);
+    void authorize_nic(VectorAttribute * nic, int uid, AuthRequest * ar);
 
     /**
      *  Bootstraps the database table(s) associated to the VirtualNetwork pool
+     *    @return 0 on success
      */
-    static void bootstrap(SqlDB * _db)
+    static int bootstrap(SqlDB * _db)
     {
-        VirtualNetwork::bootstrap(_db);
+        return VirtualNetwork::bootstrap(_db);
     };
 
     /**
@@ -113,7 +122,10 @@ public:
      *
      *  @return 0 on success
      */
-    int dump(ostringstream& oss, const string& where);
+    int dump(ostringstream& oss, const string& where)
+    {
+        return PoolSQL::dump(oss, "VNET_POOL", VirtualNetwork::table,where);
+    }
 
     /**
      *  Get the mac prefix
@@ -150,28 +162,21 @@ private:
      */
     PoolObjectSQL * create()
     {
-        return new VirtualNetwork();
+        return new VirtualNetwork(-1,-1,"","",0);
     };
 
     /**
-     *  Callback function to get output the virtual network pool in XML format
-     *  (VirtualNetworkPool::dump)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
+     *  Function to get a VirtualNetwork by its name, as provided by a VM 
+     *  template
      */
-    int dump_cb(void * _oss, int num, char **values, char **names);
+    VirtualNetwork * get_nic_by_name(VectorAttribute * nic, 
+                                     const string&     name,
+                                     int               _uid);
 
     /**
-     *  Callback function to get the ID of a given virtual network
-     *  (VirtualNetworkPool::get)
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
+     *  Function to get a VirtualNetwork by its id, as provided by a VM template
      */
-    int get_cb(void * _oss, int num, char **values, char **names);
+    VirtualNetwork * get_nic_by_id(const string& id_s);
 };
 
 #endif /*VIRTUAL_NETWORK_POOL_H_*/
