@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -16,15 +16,10 @@
 
 #include "AuthManager.h"
 #include "NebulaLog.h"
-
+#include "SSLTools.h"
+#include "PoolObjectAuth.h"
 #include "Nebula.h"
 
-#include <openssl/sha.h>
-#include <openssl/hmac.h>
-#include <openssl/evp.h>
-#include <openssl/bio.h>
-#include <openssl/buffer.h>
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
@@ -35,62 +30,18 @@ const char * AuthManager::auth_driver_name = "auth_exe";
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-string * AuthRequest::base64_encode(const string& in)
-{
-    BIO *     bio_mem;
-    BIO *     bio_64;
-
-    char *    encoded_c;
-    long int  size;
-
-    bio_64  = BIO_new(BIO_f_base64());
-    bio_mem = BIO_new(BIO_s_mem());
-
-    BIO_push(bio_64, bio_mem);
-
-    BIO_set_flags(bio_64,BIO_FLAGS_BASE64_NO_NL);
-
-    BIO_write(bio_64, in.c_str(), in.length());
-
-    if (BIO_flush(bio_64) != 1)
-    {
-        return 0;
-    }
-
-    size = BIO_get_mem_data(bio_mem,&encoded_c);
-
-    string * encoded = new string(encoded_c,size);
-
-    BIO_free_all(bio_64);
-
-    return encoded;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void AuthRequest::add_auth(Object        ob,
-                           const string& ob_id,
-                           Operation     op,
-                           int           owner,
-                           bool          pub)
+void AuthRequest::add_auth(Operation             op,
+                           const PoolObjectAuth& ob_perms,
+                           string                ob_template)
 {
     ostringstream oss;
     bool          auth;
 
-    switch (ob)
-    {
-        case VM:       oss << "VM:" ; break;
-        case HOST:     oss << "HOST:" ; break;
-        case NET:      oss << "NET:" ; break;
-        case IMAGE:    oss << "IMAGE:" ; break;
-        case USER:     oss << "USER:" ; break;
-        case CLUSTER:  oss << "CLUSTER:" ; break;
-    }
+    oss << ob_perms.type_to_str() << ":";
 
-    if (op == CREATE) //encode the ob_id, it is a template
+    if ( !ob_template.empty() )
     {
-        string * encoded_id = base64_encode(ob_id);
+        string * encoded_id = SSLTools::base64_encode(ob_template);
 
         if (encoded_id != 0)
         {
@@ -104,83 +55,58 @@ void AuthRequest::add_auth(Object        ob,
     }
     else
     {
-        oss << ob_id << ":";
+        oss << ob_perms.oid << ":";
     }
 
-    switch (op)
-    {
-        case CREATE:
-            oss << "CREATE:" ;
-            break;
+    oss << operation_to_str(op) << ":";
 
-        case DELETE:
-            oss << "DELETE:" ;
-            break;
-
-        case USE:
-            oss << "USE:" ;
-            break;
-
-        case MANAGE:
-            oss << "MANAGE:" ;
-            break;
-            
-        case INFO:
-            oss << "INFO:" ;
-            break;
-    }
-
-    oss << owner << ":" << pub;
+    oss << ob_perms.uid << ":";
 
     // -------------------------------------------------------------------------
     // Authorize the request for self authorization
     // -------------------------------------------------------------------------
 
-    if ( uid == 0 )
+    // Default conditions that grants permission :
+    // User is oneadmin, or is in the oneadmin group
+    if ( uid == 0 || gid == GroupPool::ONEADMIN_ID )
     {
         auth = true;
     }
     else
     {
-        auth = false;
+        Nebula&     nd   = Nebula::instance();
+        AclManager* aclm = nd.get_aclm();
 
-        switch (op)
-        {
-            case CREATE:
-                if ( ob == VM || ob == NET || ob == IMAGE )
-                {
-                    auth = true;
-                }
-                break;
-
-            case DELETE:
-                auth = owner == uid;
-                break;
-
-            case USE:
-                if (ob == NET || ob == IMAGE)
-                {
-                    auth = (owner == uid) || pub;
-                }
-                else if (ob == HOST)
-                {
-                    auth = true;
-                }
-                break;
-
-            case MANAGE:
-                auth = owner == uid;
-                break;
-                
-            case INFO: // This is for completeness, as the only INFO existing 
-                       // is for UserPool, and just oneadmin can see it
-                break;
-        }
+        auth = aclm->authorize(uid, gid, ob_perms, op);
     }
 
+    oss << auth; // Store the ACL authorization result in the request
+
     self_authorize = self_authorize && auth;
 
     auths.push_back(oss.str());
+
+    if ( auth == false )
+    {
+        oss.str("");
+
+        oss << message;
+
+        if ( !message.empty() )
+        {
+            oss << "; ";
+        }
+
+        oss << "Not authorized to perform " << operation_to_str(op)
+            << " " << ob_perms.type_to_str();
+
+        if ( ob_perms.oid != -1 )
+        {
+            oss << " [" << ob_perms.oid << "]";
+        }
+
+        message = oss.str();
+    }
 }
 
 /* -------------------------------------------------------------------------- */
@@ -325,6 +251,7 @@ void AuthManager::authenticate_action(AuthRequest * ar)
 
     authm_md->authenticate(ar->id,
                            ar->uid,
+                           ar->driver,
                            ar->username,
                            ar->password,
                            ar->session);
@@ -352,7 +279,8 @@ void AuthManager::authorize_action(AuthRequest * ar)
 
     if (authm_md == 0)
     {
-        goto error_driver;
+        ar->message = "Could not find Authorization driver";
+        goto error;
     }
 
     // ------------------------------------------------------------------------
@@ -367,15 +295,23 @@ void AuthManager::authorize_action(AuthRequest * ar)
 
     auths = ar->get_auths();
 
-    authm_md->authorize(ar->id, ar->uid, auths);
+    if ( auths.empty() )
+    {
+        ar->message = "Empty authorization string";
+        goto error;
+    }
+
+    authm_md->authorize(ar->id, ar->uid, auths, ar->self_authorize);
 
     return;
 
-error_driver:
-    ar->result  = false;
-    ar->message = "Could not find Authorization driver";
+error:
+    ar->result = false;
     ar->notify();
+
+    return;
 }
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
@@ -387,12 +323,14 @@ void AuthManager::timer_action()
 
     lock();
 
-    for (it=auth_requests.begin();it!=auth_requests.end();it++)
+    it = auth_requests.begin();
+
+    while ( it !=auth_requests.end())
     {
-        if (the_time > it->second->time_out)
+        if ((it->second->time_out != 0) && (the_time > it->second->time_out))
         {
             AuthRequest * ar = it->second;
-            auth_requests.erase(it);
+            auth_requests.erase(it++);
 
             ar->result  = false;
             ar->timeout = true;
@@ -400,6 +338,10 @@ void AuthManager::timer_action()
 
             ar->notify();
         }
+        else
+        {
+            ++it;
+        }
     }
 
     unlock();
@@ -466,7 +408,16 @@ void AuthManager::notify_request(int auth_id,bool result,const string& message)
     }
 
     ar->result = result;
-    ar->message= message;
+
+    if ( message != "-" )
+    {
+        if ( !ar->message.empty() )
+        {
+            ar->message.append("; ");
+        }
+
+        ar->message.append(message);
+    }
 
     ar->notify();
 }
