@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,25 +25,30 @@
 /* Driver ASCII Protocol Implementation                                       */
 /* ************************************************************************** */
 
-void AuthManagerDriver::authorize(int oid, int uid, const string& reqs) const
+void AuthManagerDriver::authorize(int           oid, 
+                                  int           uid, 
+                                  const string& reqs, 
+                                  bool          acl) const
 {
     ostringstream os;
 
-    os << "AUTHORIZE " << oid << " " << uid << " " << reqs << endl;
+    os << "AUTHORIZE " << oid << " " << uid << " " << reqs << " " << acl <<endl;
 
     write(os);
 }
 
 void AuthManagerDriver::authenticate(int           oid,
-                      int           uid,
-                      const string& username,
-                      const string& password,
-                      const string& session) const
+                                     int           uid,
+                                     const string& driver,
+                                     const string& username,
+                                     const string& password,
+                                     const string& session) const
 {
     ostringstream os;
 
     os << "AUTHENTICATE " << oid << " "
                           << uid << " "
+                          << driver << " "
                           << username << " "
                           << password << " "
                           << session  << endl;
@@ -91,7 +96,7 @@ void AuthManagerDriver::protocol(
             {
                 is.clear();
                 getline(is,info);
-                NebulaLog::log("AuM",Log::INFO, info.c_str());
+                NebulaLog::log("AuM", log_type(result[0]), info.c_str());
             }
 
             return;
