@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,127 +14,51 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
-require 'Configuration'
 require 'OpenNebula'
-require 'pp'
 
 ##############################################################################
 # This class represents a generic Cloud Server using the OpenNebula Cloud
 # API (OCA). Any cloud implementation should derive from this class
 ##############################################################################
 class CloudServer
+    ##########################################################################
+    # Class Constants. Define the OpenNebula Error and HTTP codes mapping
+    ##########################################################################
+    HTTP_ERROR_CODE = {
+        OpenNebula::Error::EAUTHENTICATION => 401,
+        OpenNebula::Error::EAUTHORIZATION  => 403,
+        OpenNebula::Error::ENO_EXISTS      => 404,
+        OpenNebula::Error::EACTION         => 500,
+        OpenNebula::Error::EXML_RPC_API    => 500,
+        OpenNebula::Error::EINTERNAL       => 500,
+        OpenNebula::Error::ENOTDEFINED     => 500
+    }
 
     ##########################################################################
     # Public attributes
     ##########################################################################
     attr_reader :config
-    attr_reader :one_client
 
     # Initializes the Cloud server based on a config file
     # config_file:: _String_ for the server. MUST include the following
     # variables:
-    #   USER
-    #   PASSWORD
+    #   AUTH
     #   VM_TYPE
-    #   IMAGE_DIR
-    #   DATABASE
-    def initialize(config_file)
-
+    #   XMLRPC
+    def initialize(config)
         # --- Load the Cloud Server configuration file ---
-
-        @config = Configuration.new(config_file)
-
-        if @config[:vm_type] == nil
-            raise "No VM_TYPE defined."
-        end
-
-        @instance_types = Hash.new
-
-        if @config[:vm_type].kind_of?(Array)
-            @config[:vm_type].each {|type|
-                @instance_types[type['NAME']]=type
-            }
-        else
-            @instance_types[@config[:vm_type]['NAME']]=@config[:vm_type]
-        end
-
-        # --- Start an OpenNebula Session ---
-
-        @one_client = Client.new(nil,@config[:one_xmlrpc])
-        @user_pool  = UserPool.new(@one_client)
-
-        @img_repo = OpenNebula::ImageRepository.new
+        @config = config
     end
-
     #
     # Prints the configuration of the server
     #
-    def print_configuration
+    def self.print_configuration(config)
         puts "--------------------------------------"
         puts "         Server configuration         "
         puts "--------------------------------------"
-        pp @config
+        pp config
 
-        puts "--------------------------------------"
-        puts "      Registered Instance Types       "
-        puts "--------------------------------------"
-        pp @instance_types
-    end
-
-    ###########################################################################
-    # USER and OpenNebula Session Methods
-    ###########################################################################
-
-    # Generates an OpenNebula Session for the given user
-    # user:: _Hash_ the user information
-    # [return] an OpenNebula client session
-    def one_client_user(name, password)
-        client = Client.new("dummy:dummy")
-        client.one_auth = "#{name}:#{password}"
-
-        return client
-    end
-
-    # Gets the data associated with a user
-    # name:: _String_ the name of the user
-    # [return] _Hash_ with the user data
-    def get_user_password(name)
-        @user_pool.info
-        return @user_pool["USER[NAME=\"#{name}\"]/PASSWORD"]
-    end
-
-    ###########################################################################
-    # Repository Methods
-    ###########################################################################
-
-    # Adds a new image to the repository and deletes the temp_file
-    # uid:: _Integer_ owner of the image
-    # path:: _String_ path of the tmp file
-    # metadata:: Additional metadata for the file
-    # [return] _Image_ Newly created image object
-    def add_image(image, file=nil)
-        template = image.to_one_template
-        
-        if file
-            if file[:tempfile]
-                file_path = file[:tempfile].path
-                template << "\nPATH = #{file_path}"
-            else
-                error_msg = "Image not present, aborting."
-                error = OpenNebula::Error.new(error_msg)
-                return error
-            end
-        end
-
-        rc = @img_repo.create(image, template)
-
-        file[:tempfile].unlink if file
-
-        if OpenNebula.is_error?(rc)
-           return rc
-        end
-
-        return nil
+        STDOUT.flush
     end
 
     # Finds out if a port is available on ip
@@ -158,4 +82,3 @@ class CloudServer
       return false
     end
 end
-
