@@ -1,6 +1,6 @@
 #!/usr/bin/ruby
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -48,7 +48,8 @@ module EC2QueryClient
             elsif ENV["EC2_ACCESS_KEY"] and ENV["EC2_SECRET_KEY"]
                 ec2auth = [ENV["EC2_ACCESS_KEY"], ENV["EC2_SECRET_KEY"]]
             else
-                ec2auth=CloudClient::get_one_auth
+                ec2auth = CloudClient::get_one_auth
+                ec2auth[1] = Digest::SHA1.hexdigest(ec2auth[1])
             end
 
             if !ec2auth
@@ -56,7 +57,7 @@ module EC2QueryClient
             end
 
             @access_key_id     = ec2auth[0]
-            @access_key_secret = Digest::SHA1.hexdigest(ec2auth[1])
+            @access_key_secret = ec2auth[1]
 
             # Server location
 
@@ -75,7 +76,8 @@ module EC2QueryClient
                 :secret_access_key => @access_key_secret,
                 :server            => @uri.host,
                 :port              => @uri.port,
-                :use_ssl           => @uri.scheme == 'https')
+                :use_ssl           => @uri.scheme == 'https',
+                :path              => @uri.path)
         end
 
 
@@ -140,7 +142,7 @@ module EC2QueryClient
         def upload_image(file_name, curb=true)
             params = { "Action"           => "UploadImage",
                        "SignatureVersion" => "2",
-                       "SignatureMethod"  => 'HmacSHA1',
+                       "SignatureMethod"  => 'HmacSHA256',
                        "AWSAccessKeyId"   => @access_key_id,
                        "Version"          => API_VERSION,
                        "Timestamp"        => Time.now.getutc.iso8601 }
