@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,7 +15,6 @@
 #--------------------------------------------------------------------------- #
 
 require 'rubygems'
-require 'sinatra'
 require 'erb'
 require 'time'
 require 'AWS'
@@ -62,71 +61,27 @@ class EC2QueryServer < CloudServer
 
     ###########################################################################
 
-    def initialize(config_file,template,views)
-        super(config_file)
-        @config.add_configuration_value("TEMPLATE_LOCATION",template)
-        @config.add_configuration_value("VIEWS",views)
+    def initialize(client, config)
+        super(config)
 
-        if @config[:ssl_server]
-            @server_host=@config[:ssl_server]
-        else
-            @server_host=@config[:server]
-        end
-
-        @server_port=@config[:port]
-
-        print_configuration
+        @client = client
     end
 
-    ###########################################################################
-    # Authentication functions
-    ###########################################################################
-
-    # EC2 protocol authentication function
-    # params:: of the request
-    # [return] true if authenticated
-    def authenticate(params,env)
-        password = get_user_password(params['AWSAccessKeyId'])
-        return nil if !password
-
-        signature = case params['SignatureVersion']
-            when "1" then signature_version_1(params.clone, password)
-            when "2" then signature_version_2(params,
-                              password,
-                              env,
-                              true,
-                              false)
-        end
-
-        if params['Signature']==signature
-            return one_client_user(params['AWSAccessKeyId'], password)
-        else
-            if params['SignatureVersion']=="2"
-                signature = signature_version_2(params,
-                                  password,
-                                  env,
-                                  false,
-                                  false)
-                if params['Signature']==signature
-                    return one_client_user(params['AWSAccessKeyId'], password)
-                end
-            end
-        end
-
-        return nil
-    end
-
-
     ###########################################################################
     # Repository Interface
     ###########################################################################
 
-    def upload_image(params, one_client)
-        image = ImageEC2.new(Image.build_xml, one_client)
+    def upload_image(params)
+        image = ImageEC2.new(Image.build_xml, @client, params['file'])
 
-        rc = add_image(image, params['file'])
+        template = image.to_one_template
+        if OpenNebula.is_error?(template)
+            return OpenNebula::Error.new('Unsupported'), 400
+        end
+
+        rc = image.allocate(template)
         if OpenNebula.is_error?(rc)
-            return OpenNebula::Error.new('Unsupported'),400
+            return OpenNebula::Error.new('Unsupported'), 400
         end
 
         erb_version = params['Version']
@@ -135,11 +90,11 @@ class EC2QueryServer < CloudServer
         return response.result(binding), 200
     end
 
-    def register_image(params, one_client)
+    def register_image(params)
         # Get the Image ID
         tmp, img=params['ImageLocation'].split('-')
 
-        image = Image.new(Image.build_xml(img.to_i), one_client)
+        image = Image.new(Image.build_xml(img.to_i), @client)
 
         # Enable the new Image
         rc = image.info
@@ -155,12 +110,11 @@ class EC2QueryServer < CloudServer
         return response.result(binding), 200
     end
 
-    def describe_images(params, one_client)
-        user_flag=-1
-        impool = ImagePool.new(one_client, user_flag)
+    def describe_images(params)
+        user_flag = OpenNebula::Pool::INFO_ALL
+        impool = ImagePool.new(@client, user_flag)
         impool.info
 
-        erb_user_name = params['AWSAccessKeyId']
         erb_version = params['Version']
 
         response = ERB.new(File.read(@config[:views]+"/describe_images.erb"))
@@ -171,14 +125,14 @@ class EC2QueryServer < CloudServer
     # Instance Interface
     ###########################################################################
 
-    def run_instances(params, one_client)
+    def run_instances(params)
         # Get the instance type and path
         if params['InstanceType'] != nil
             instance_type_name = params['InstanceType']
-            instance_type      = @instance_types[instance_type_name]
+            instance_type      = @config[:instance_types][instance_type_name.to_sym]
 
             if instance_type != nil
-                path = @config[:template_location] + "/#{instance_type['TEMPLATE']}"
+                path = @config[:template_location] + "/#{instance_type[:template]}"
             end
         end
 
@@ -197,7 +151,7 @@ class EC2QueryServer < CloudServer
         template_text = template.result(binding)
 
         # Start the VM.
-        vm = VirtualMachine.new(VirtualMachine.build_xml, one_client)
+        vm = VirtualMachine.new(VirtualMachine.build_xml, @client)
 
         rc = vm.allocate(template_text)
         if OpenNebula::is_error?(rc)
@@ -215,26 +169,26 @@ class EC2QueryServer < CloudServer
         return response.result(binding), 200
     end
 
-    def describe_instances(params, one_client)
-        user_flag=-1
-        vmpool = VirtualMachinePool.new(one_client, user_flag)
+    def describe_instances(params)
+        user_flag = OpenNebula::Pool::INFO_ALL
+        vmpool = VirtualMachinePool.new(@client, user_flag)
         vmpool.info
 
         erb_version = params['Version']
         erb_user_name = params['AWSAccessKeyId']
-        
+
         response = ERB.new(File.read(@config[:views]+"/describe_instances.erb"))
         return response.result(binding), 200
     end
 
-    def terminate_instances(params, one_client)
+    def terminate_instances(params)
         # Get the VM ID
         vmid=params['InstanceId.1']
         vmid=params['InstanceId.01'] if !vmid
 
         tmp, vmid=vmid.split('-') if vmid[0]==?i
 
-        vm = VirtualMachine.new(VirtualMachine.build_xml(vmid),one_client)
+        vm = VirtualMachine.new(VirtualMachine.build_xml(vmid),@client)
         rc = vm.info
 
         return OpenNebula::Error.new('Unsupported'),400 if OpenNebula::is_error?(rc)
@@ -253,55 +207,6 @@ class EC2QueryServer < CloudServer
         return response.result(binding), 200
     end
 
-private
-
-    # Calculates signature version 1
-    def signature_version_1(params, secret_key, digest='sha1')
-        params.delete('Signature')
-        req_desc = params.sort {|x,y| x[0].downcase <=> y[0].downcase}.to_s
-
-        digest_generator = OpenSSL::Digest::Digest.new(digest)
-        digest = OpenSSL::HMAC.digest(digest_generator,
-                                      secret_key,
-                                      req_desc)
-        b64sig = Base64.b64encode(digest)
-        return b64sig.strip
-    end
-
-    # Calculates signature version 2
-    def signature_version_2(params, secret_key, env, includeport=true, urlencode=true)
-        signature_params = params.reject { |key,value|
-            key=='Signature' or key=='file' }
-
-        if includeport
-            server_str = @server_host + ':' + @server_port
-        else
-            server_str = @server_host
-        end
-
-        canonical_str = AWS.canonical_string(signature_params,
-                         server_str,
-                         env['REQUEST_METHOD'])
-
-        # Use the correct signature strength
-        sha_strength = case params['SignatureMethod']
-            when "HmacSHA1" then 'sha1'
-            when "HmacSHA256" then 'sha256'
-            else 'sha1'
-        end
-
-        digest = OpenSSL::Digest::Digest.new(sha_strength)
-        b64hmac =
-            Base64.encode64(
-                OpenSSL::HMAC.digest(digest, secret_key, canonical_str)).gsub("\n","")
-
-        if urlencode
-            return CGI::escape(b64hmac)
-        else
-            return b64hmac
-        end
-    end
-    
     ###########################################################################
     # Helper functions
     ###########################################################################
@@ -313,7 +218,7 @@ private
     end
 
     def render_launch_time(vm)
-        return "<launchTime>#{Time.at(vm[:stime].to_i).xmlschema}</launchTime>"
+        return "<launchTime>#{Time.at(vm["STIME"].to_i).xmlschema}</launchTime>"
     end
 end
 
