@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -23,19 +23,17 @@ class ImageOCCI < Image
         <STORAGE href="<%= base_url %>/storage/<%= self.id.to_s  %>">
             <ID><%= self.id.to_s %></ID>
             <NAME><%= self.name %></NAME>
-            <% if self['TEMPLATE/TYPE'] != nil %>
-            <TYPE><%= self['TEMPLATE/TYPE'] %></TYPE>
+            <% if self['TYPE'] != nil %>
+            <TYPE><%= self.type_str %></TYPE>
             <% end %>
             <% if self['TEMPLATE/DESCRIPTION'] != nil %>
             <DESCRIPTION><%= self['TEMPLATE/DESCRIPTION'] %></DESCRIPTION>
             <% end %>
-            <% if size != nil %>
-            <SIZE><%= size %></SIZE>
+            <SIZE><%= self['SIZE'] %></SIZE>
+            <% if self['FSTYPE'] != nil and  !self['FSTYPE'].empty? %>
+            <FSTYPE><%= self['FSTYPE'] %></FSTYPE>
             <% end %>
-            <% if fstype != nil %>
-            <FSTYPE><%= fstype %></FSTYPE>
-            <% end %>
-            <PUBLIC><%= self['PUBLIC'] == "0" ? "NO" : "YES"%></PUBLIC>
+            <PUBLIC><%= self.public? ? "YES" : "NO" %></PUBLIC>
             <PERSISTENT><%= self['PERSISTENT'] == "0" ? "NO" : "YES"%></PERSISTENT>
         </STORAGE>
     }
@@ -46,6 +44,9 @@ class ImageOCCI < Image
         <% if @image_info['DESCRIPTION'] != nil %>
         DESCRIPTION = "<%= @image_info['DESCRIPTION'] %>"
         <% end %>
+        <% if @image_file != nil %>
+        PATH = "<%= @image_file %>"
+        <% end %>
         <% if @image_info['PUBLIC'] != nil %>
         PUBLIC = "<%= @image_info['PUBLIC'] %>"
         <% end %>
@@ -64,9 +65,14 @@ class ImageOCCI < Image
     }.gsub(/^        /, '')
 
     # Class constructor
-    def initialize(xml, client, xml_info=nil)
+    def initialize(xml, client, xml_info=nil, file=nil)
         super(xml, client)
         @image_info = nil
+        @image_file = file
+
+        if file && file[:tempfile]
+            @image_file = file[:tempfile].path
+        end
 
         if xml_info != nil
             xmldoc      = XMLElement.build_xml(xml_info, 'STORAGE')
@@ -76,25 +82,18 @@ class ImageOCCI < Image
 
     # Creates the OCCI representation of an Image
     def to_occi(base_url)
-        size = nil
-
         begin
-            if self['SOURCE'] != nil and File.exists?(self['SOURCE'])
-                size = File.stat(self['SOURCE']).size
-                size = size / 1024
-            end
-
-            fstype = self['TEMPLATE/FSTYPE'] if self['TEMPLATE/FSTYPE']
+            occi_im = ERB.new(OCCI_IMAGE)
+            occi_im_text = occi_im.result(binding)
         rescue Exception => e
             error = OpenNebula::Error.new(e.message)
             return error
         end
 
-        occi = ERB.new(OCCI_IMAGE)
-        return occi.result(binding).gsub(/\n\s*/,'')
+        return occi_im_text.gsub(/\n\s*/,'')
     end
 
-    def to_one_template()
+    def to_one_template
         if @image_info == nil
             error_msg = "Missing STORAGE section in the XML body"
             error = OpenNebula::Error.new(error_msg)
