@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,12 +15,9 @@
 #--------------------------------------------------------------------------- #
 
 # Common cloud libs
-require 'rubygems'
-require 'sinatra'
 require 'CloudServer'
 
 # OCA
-require 'OpenNebula'
 include OpenNebula
 
 # OCCI libs
@@ -30,6 +27,8 @@ require 'VirtualNetworkOCCI'
 require 'VirtualNetworkPoolOCCI'
 require 'ImageOCCI'
 require 'ImagePoolOCCI'
+require 'UserOCCI'
+require 'UserPoolOCCI'
 
 require 'pp'
 
@@ -38,32 +37,26 @@ require 'pp'
 # The OCCI Server provides an OCCI implementation based on the
 # OpenNebula Engine
 ##############################################################################
-class OCCIServer < CloudServer
 
+COLLECTIONS = ["compute", "instance_type", "network", "storage"]
+
+# FLAG that will filter the elements retrieved from the Pools
+POOL_FILTER = Pool::INFO_ALL
+
+class OCCIServer < CloudServer
     # Server initializer
     # config_file:: _String_ path of the config file
     # template:: _String_ path to the location of the templates
-    def initialize(config_file,template)
-        super(config_file)
+    def initialize(client, config)
+        super(config)
 
-        @config.add_configuration_value("TEMPLATE_LOCATION",template)
-
-        if @config[:ssl_server]
-            @base_url=@config[:ssl_server]
+        if config[:ssl_server]
+            @base_url=config[:ssl_server]
         else
-            @base_url="http://#{@config[:server]}:#{@config[:port]}"
+            @base_url="http://#{config[:server]}:#{config[:port]}"
         end
 
-        print_configuration
-    end
-
-    # Retrieve a client with the user credentials
-    # requestenv:: _Hash_ Hash containing the environment of the request
-    # [return] _Client_ client with the user credentials
-    def get_client(requestenv)
-        auth =  Rack::Auth::Basic::Request.new(requestenv)
-
-        return one_client_user(auth.credentials[0], auth.credentials[1])
+        @client = client
     end
 
     # Prepare the OCCI XML Response
@@ -82,19 +75,52 @@ class OCCIServer < CloudServer
     ############################################################################
     ############################################################################
 
+    def get_collections(request)
+        xml_resp = "<COLLECTIONS>\n"
+
+        COLLECTIONS.each { |c|
+            xml_resp << "\t<#{c.upcase}_COLLECTION href=\"#{@base_url}/#{c}\">\n"
+        }
+
+        xml_resp << "</COLLECTIONS>"
+
+        return xml_resp, 200
+    end
+
+    def get_instance_types(request)
+        xml_resp = "<INSTANCE_TYPE_COLLECTION>\n"
+
+        @config[:instance_types].each { |k, v|
+            xml_resp << "\t<INSTANCE_TYPE href=\"#{@base_url}/instance_type/#{k.to_s}\">\n"
+            xml_resp << "\t\t<NAME>#{k.to_s}</NAME>\n"
+            v.each { |elem, value|
+                next if elem==:template
+                str = elem.to_s.upcase
+                xml_resp << "\t\t<#{str}>#{value}</#{str}>\n"
+            }
+            xml_resp << "\t</INSTANCE_TYPE>\n"
+        }
+
+        xml_resp << "</INSTANCE_TYPE_COLLECTION>"
+
+        return xml_resp, 200
+    end
+
+
     # Gets the pool representation of COMPUTES
     # request:: _Hash_ hash containing the data of the request
     # [return] _String_,_Integer_ Pool Representation or error, status code
     def get_computes(request)
         # --- Get User's VMs ---
-        user_flag = -1
         vmpool = VirtualMachinePoolOCCI.new(
-                        get_client(request.env),
-                        user_flag)
+                        @client,
+                        POOL_FILTER)
 
         # --- Prepare XML Response ---
         rc = vmpool.info
-        return rc, 404 if OpenNebula.is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return to_occi_xml(vmpool, 200)
     end
@@ -106,14 +132,15 @@ class OCCIServer < CloudServer
     # =>                          status code
     def get_networks(request)
         # --- Get User's VNETs ---
-        user_flag = -1
         network_pool = VirtualNetworkPoolOCCI.new(
-                            get_client(request.env),
-                            user_flag)
+                            @client,
+                            POOL_FILTER)
 
         # --- Prepare XML Response ---
         rc = network_pool.info
-        return rc, 404 if OpenNebula.is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return to_occi_xml(network_pool, 200)
     end
@@ -124,18 +151,36 @@ class OCCIServer < CloudServer
     #                             status code
     def get_storages(request)
         # --- Get User's Images ---
-        user_flag = -1
         image_pool = ImagePoolOCCI.new(
-                            get_client(request.env),
-                            user_flag)
+                            @client,
+                            POOL_FILTER)
 
         # --- Prepare XML Response ---
         rc = image_pool.info
-        return rc, 404 if OpenNebula.is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return to_occi_xml(image_pool, 200)
     end
 
+    # Gets the pool representation of USERs
+    # request:: _Hash_ hash containing the data of the request
+    # [return] _String_,_Integer_ User pool representation or error,
+    #                             status code
+    def get_users(request)
+        # --- Get Users Pool ---
+        user_pool = UserPoolOCCI.new(@client)
+
+        # --- Prepare XML Response ---
+        rc = user_pool.info
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
+
+        return to_occi_xml(user_pool, 200)
+    end
+
     ############################################################################
     ############################################################################
     #                      ENTITY RESOURCE METHODS
@@ -153,9 +198,9 @@ class OCCIServer < CloudServer
         # --- Create the new Instance ---
         vm = VirtualMachineOCCI.new(
                     VirtualMachine.build_xml,
-                    get_client(request.env),
+                    @client,
                     request.body.read,
-                    @instance_types,
+                    @config[:instance_types],
                     @config[:template_location])
 
         # --- Generate the template and Allocate the new Instance ---
@@ -163,7 +208,9 @@ class OCCIServer < CloudServer
         return template, 500 if OpenNebula.is_error?(template)
 
         rc = vm.allocate(template)
-        return rc, 500 if OpenNebula.is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         # --- Prepare XML Response ---
         vm.info
@@ -178,11 +225,13 @@ class OCCIServer < CloudServer
         # --- Get the VM ---
         vm = VirtualMachineOCCI.new(
                     VirtualMachine.build_xml(params[:id]),
-                    get_client(request.env))
+                    @client)
 
         # --- Prepare XML Response ---
         rc = vm.info
-        return rc, 404 if OpenNebula::is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return to_occi_xml(vm, 200)
     end
@@ -196,14 +245,13 @@ class OCCIServer < CloudServer
         # --- Get the VM ---
         vm = VirtualMachineOCCI.new(
                     VirtualMachine.build_xml(params[:id]),
-                    get_client(request.env))
-
-        rc = vm.info
-        return rc, 404 if OpenNebula::is_error?(rc)
+                    @client)
 
         # --- Finalize the VM ---
         result = vm.finalize
-        return result, 500 if OpenNebula::is_error?(result)
+        if OpenNebula.is_error?(result)
+            return result, CloudServer::HTTP_ERROR_CODE[result.errno]
+        end
 
         return "", 204
     end
@@ -213,77 +261,24 @@ class OCCIServer < CloudServer
     # [return] _String_,_Integer_ Update confirmation msg or error,
     #                             status code
     def put_compute(request, params)
-        
-        one_client = get_client(request.env)
-        
+        # --- Get the VM ---
         vm = VirtualMachineOCCI.new(
                     VirtualMachine.build_xml(params[:id]),
-                    one_client)
-        
+                    @client)
+
         rc = vm.info
-        return rc, 400 if OpenNebula.is_error?(rc)
-        
-        xmldoc  = XMLElement.build_xml(request.body, 'COMPUTE')
-        vm_info = XMLElement.new(xmldoc) if xmldoc != nil
-        
-        # Check the number of changes in the request
-        image_name = nil
-        image_type = nil
-        vm_info.each('DISK/SAVE_AS') { |disk|
-            if image_name
-                error_msg = "It is only allowed one save_as per request"
-                return OpenNebula::Error.new(error_msg), 400
-            end
-            image_name = disk.attr('.', 'name')
-            image_type = disk.attr('.', 'type')
-        }
-        state = vm_info['STATE']
-        
-        if image_name && state
-            error_msg = "It is not allowed to change the state and save_as" <<
-                        " a disk in the same request"
-            return OpenNebula::Error.new(error_msg), 400
-        elsif image_name
-            # Get the disk id
-            disk_id = vm_info.attr('DISK/SAVE_AS/..', 'id')
-            if disk_id.nil?
-                error_msg = "DISK id attribute not specified"
-                return OpenNebula::Error.new(error_msg), 400
-            end
-
-            disk_id = disk_id.to_i
-            if vm["TEMPLATE/DISK[DISK_ID=\"#{disk_id}\"]/SAVE_AS"]
-                error_msg = "The disk #{disk_id} is already" <<
-                            " suppossed to be saved"
-                return OpenNebula::Error.new(error_msg), 400
-            end
-            
-            # Create a new Image to save the disk
-            template = "NAME=\"#{image_name}\"\n"
-            if image_type
-                template << "TYPE=\"#{image_type}\"\n"
-            else
-                template << "TYPE=\"OS\"\n"
-            end
-
-            image = Image.new(Image.build_xml, one_client)
-
-            rc = image.allocate(template)
-            return rc, 400 if OpenNebula.is_error?(rc)
-            
-            rc = vm.save_as(disk_id, image.id)
-            if OpenNebula.is_error?(rc)
-                image.delete
-                return rc, 400
-            end 
-        elsif state
-            rc = vm.mk_action(state)
-            return rc, 400 if OpenNebula.is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
         end
 
-        # --- Prepare XML Response ---
-        vm.info
-        return to_occi_xml(vm, 202)
+        result, code = vm.update_from_xml(request.body)
+
+        if OpenNebula.is_error?(result)
+            return result, code
+        else
+            vm.info
+            return to_occi_xml(vm, code)
+        end
     end
 
     ############################################################################
@@ -297,16 +292,18 @@ class OCCIServer < CloudServer
         # --- Create the new Instance ---
         network = VirtualNetworkOCCI.new(
                         VirtualNetwork.build_xml,
-                        get_client(request.env),
+                        @client,
                         request.body,
-                        @config[:bridge])
+                        @config[:template_location])
 
         # --- Generate the template and Allocate the new Instance ---
         template = network.to_one_template
         return template, 500 if OpenNebula.is_error?(template)
 
         rc = network.allocate(template)
-        return rc, 500 if OpenNebula.is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         # --- Prepare XML Response ---
         network.info
@@ -318,14 +315,15 @@ class OCCIServer < CloudServer
     # [return] _String_,_Integer_ NETWORK occi representation or error,
     #                             status code
     def get_network(request, params)
-        # --- Get the VNET ---
         network = VirtualNetworkOCCI.new(
                         VirtualNetwork.build_xml(params[:id]),
-                        get_client(request.env))
+                        @client)
 
         # --- Prepare XML Response ---
         rc = network.info
-        return rc, 404 if OpenNebula::is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return to_occi_xml(network, 200)
     end
@@ -335,21 +333,19 @@ class OCCIServer < CloudServer
     # [return] _String_,_Integer_ Delete confirmation msg or error,
     #                             status code
     def delete_network(request, params)
-        # --- Get the VNET ---
         network = VirtualNetworkOCCI.new(
                         VirtualNetwork.build_xml(params[:id]),
-                        get_client(request.env))
-
-        rc = network.info
-        return rc, 404 if OpenNebula::is_error?(rc)
+                        @client)
 
         # --- Delete the VNET ---
         rc = network.delete
-        return rc, 500 if OpenNebula::is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return "", 204
     end
-    
+
     # Updates a NETWORK resource
     # request:: _Hash_ hash containing the data of the request
     # [return] _String_,_Integer_ Update confirmation msg or error,
@@ -360,17 +356,17 @@ class OCCIServer < CloudServer
 
         vnet = VirtualNetworkOCCI.new(
                     VirtualNetwork.build_xml(params[:id]),
-                    get_client(request.env))
-                    
-        rc = vnet.info
-        return rc, 400 if OpenNebula.is_error?(rc)
-        
+                    @client)
+
+        rc = nil
         if vnet_info['PUBLIC'] == 'YES'
             rc = vnet.publish
-            return rc, 400 if OpenNebula.is_error?(rc)
         elsif vnet_info['PUBLIC'] == 'NO'
             rc = vnet.unpublish
-            return rc, 400 if OpenNebula.is_error?(rc)
+        end
+
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
         end
 
         # --- Prepare XML Response ---
@@ -400,17 +396,21 @@ class OCCIServer < CloudServer
 
         image = ImageOCCI.new(
                         Image.build_xml,
-                        get_client(request.env),
-                        occixml)
+                        @client,
+                        occixml,
+                        request.params['file'])
 
-        rc = add_image(image, request.params['file'])
-        return rc, 500 if OpenNebula.is_error?(rc)
+        # --- Generate the template and Allocate the new Instance ---
+        template = image.to_one_template
+        return template, 500 if OpenNebula.is_error?(template)
 
-        # --- Enable the new Image ---
-        rc = image.enable
-        return rc, 500 if OpenNebula.is_error?(rc)
+        rc = image.allocate(template)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         # --- Prepare XML Response ---
+        image.info
         return to_occi_xml(image, 201)
     end
 
@@ -422,10 +422,12 @@ class OCCIServer < CloudServer
         # --- Get the Image ---
         image = ImageOCCI.new(
                         Image.build_xml(params[:id]),
-                        get_client(request.env))
+                        @client)
 
         rc = image.info
-        return rc, 404 if OpenNebula::is_error?(rc)
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         # --- Prepare XML Response ---
         return to_occi_xml(image, 200)
@@ -439,18 +441,17 @@ class OCCIServer < CloudServer
         # --- Get the Image ---
         image = ImageOCCI.new(
                         Image.build_xml(params[:id]),
-                        get_client(request.env))
-
-        rc = image.info
-        return rc, 404 if OpenNebula::is_error?(rc)
+                        @client)
 
         # --- Delete the Image ---
-        rc = @img_repo.delete(image)
-        return rc, 500 if OpenNebula::is_error?(rc)
+        rc = image.delete
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
 
         return "", 204
     end
-    
+
     # Updates a STORAGE resource
     # request:: _Hash_ hash containing the data of the request
     # [return] _String_,_Integer_ Update confirmation msg or error,
@@ -461,30 +462,47 @@ class OCCIServer < CloudServer
 
         image = ImageOCCI.new(
                     Image.build_xml(params[:id]),
-                    get_client(request.env))
-                    
-        rc = image.info
-        return rc, 400 if OpenNebula.is_error?(rc)
-        
+                    @client)
+
+        rc = nil
         if image_info['PERSISTENT'] && image_info['PUBLIC']
             error_msg = "It is not allowed more than one change per request"
             return OpenNebula::Error.new(error_msg), 400
         elsif image_info['PERSISTENT'] == 'YES'
             rc = image.persistent
-            return rc, 400 if OpenNebula.is_error?(rc)
         elsif image_info['PERSISTENT'] == 'NO'
             rc = image.nonpersistent
-            return rc, 400 if OpenNebula.is_error?(rc)
         elsif image_info['PUBLIC'] == 'YES'
             rc = image.publish
-            return rc, 400 if OpenNebula.is_error?(rc)
         elsif image_info['PUBLIC'] == 'NO'
             rc = image.unpublish
-            return rc, 400 if OpenNebula.is_error?(rc)
+        end
+
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
         end
 
         # --- Prepare XML Response ---
         image.info
         return to_occi_xml(image, 202)
     end
+
+    # Get the representation of a USER
+    # request:: _Hash_ hash containing the data of the request
+    # [return] _String_,_Integer_ USER representation or error,
+    #                             status code
+    def get_user(request, params)
+        # --- Get the USER ---
+        user = UserOCCI.new(
+                    User.build_xml(params[:id]),
+                    @client)
+
+        # --- Prepare XML Response ---
+        rc = user.info
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
+
+        return to_occi_xml(user, 200)
+    end
 end
