@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,6 +15,7 @@
 #--------------------------------------------------------------------------- #
 
 require 'OpenNebula'
+require 'ipaddr'
 
 include OpenNebula
 
@@ -26,35 +27,23 @@ class VirtualNetworkOCCI < VirtualNetwork
             <% if self['TEMPLATE/DESCRIPTION'] != nil %>
             <DESCRIPTION><%= self['TEMPLATE/DESCRIPTION'] %></DESCRIPTION>
             <% end %>
-            <ADDRESS><%= self['TEMPLATE/NETWORK_ADDRESS'] %></ADDRESS>
-            <% if self['TEMPLATE/NETWORK_SIZE'] %>
-            <SIZE><%= self['TEMPLATE/NETWORK_SIZE'] %></SIZE>
+            <% if network_address != nil %>
+            <ADDRESS><%= network_address %></ADDRESS>
             <% end %>
-            <PUBLIC><%= self['PUBLIC'] == "0" ? "NO" : "YES"%></PUBLIC>
+            <% if network_size != nil %>
+            <SIZE><%= network_size %></SIZE>
+            <% end %>
+            <USED_LEASES><%= self['TOTAL_LEASES'] %></USED_LEASES>
+            <PUBLIC><%= self.public? ? "YES" : "NO" %></PUBLIC>
         </NETWORK>
     }
 
-    ONE_NETWORK = %q{
-        NAME            = "<%= @vnet_info['NAME'] %>"
-        TYPE            = RANGED
-        <% if @vnet_info['DESCRIPTION'] != nil %>
-        DESCRIPTION     = "<%= @vnet_info['DESCRIPTION'] %>"
-        <% end %>
-        <% if @vnet_info['PUBLIC'] != nil %>
-        PUBLIC     = "<%= @vnet_info['PUBLIC'] %>"
-        <% end %>
-        <% if @bridge %>
-        BRIDGE          = <%= @bridge %>
-        <% end %>
-        NETWORK_ADDRESS = <%= @vnet_info['ADDRESS'] %>
-        NETWORK_SIZE    = <%= @vnet_info['SIZE']%>
-    }.gsub(/^        /, '')
-
     # Class constructor
-    def initialize(xml, client, xml_info=nil, bridge=nil)
+    #
+    def initialize(xml, client, xml_info=nil, base=nil)
         super(xml, client)
-        @bridge    = bridge
         @vnet_info = nil
+        @common_template = base + '/network.erb' if base
 
         if xml_info != nil
             xmldoc     = XMLElement.build_xml(xml_info, 'NETWORK')
@@ -64,6 +53,24 @@ class VirtualNetworkOCCI < VirtualNetwork
 
     # Creates the OCCI representation of a Virtual Network
     def to_occi(base_url)
+        network_address = nil
+        network_size    = nil
+
+        if self['RANGE/IP_START']
+            network_address = self['RANGE/IP_START']
+
+            ip_start = IPAddr.new(network_address, Socket::AF_INET)
+            ip_end = IPAddr.new(self['RANGE/IP_END'], Socket::AF_INET)
+
+            network_size = ip_end.to_i - ip_start.to_i
+        end
+
+        if self['PERMISSIONS/GROUP_U'] == "1" || self['PERMISSIONS/OTHER_U'] == "1"
+            pub = "YES"
+        else
+            pub = "NO"
+        end
+
         begin
             occi = ERB.new(OCCI_NETWORK)
             occi_text = occi.result(binding)
@@ -78,11 +85,16 @@ class VirtualNetworkOCCI < VirtualNetwork
     def to_one_template()
         if @vnet_info == nil
             error_msg = "Missing NETWORK section in the XML body"
-            error = OpenNebula::Error.new(error_msg)
+            return OpenNebula::Error.new(error_msg), 400
+        end
+
+        begin
+            template = ERB.new(File.read(@common_template)).result(binding)
+        rescue Exception => e
+            error = OpenNebula::Error.new(e.message)
             return error
         end
 
-        one = ERB.new(ONE_NETWORK)
-        return one.result(binding)
+        return template
     end
 end
