@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -75,8 +75,20 @@ string * VectorAttribute::to_xml() const
 
 	for (it=attribute_value.begin();it!=attribute_value.end();it++)
 	{
-		oss << "<" << it->first << "><![CDATA[" << it->second
-			<< "]]></"<< it->first << ">";
+        if ( it->first.empty() )
+        {
+            continue;
+        }
+
+        if ( it->second.empty() )
+        {
+            oss << "<" << it->first << "/>";
+        }
+        else
+        {
+	    	oss << "<" << it->first << "><![CDATA[" << it->second
+			    << "]]></"<< it->first << ">";
+        }
 	}
 
 	oss << "</"<< name() << ">";
@@ -133,9 +145,16 @@ void VectorAttribute::unmarshall(const string& sattr, const char * _sep)
     	{
     		continue;
     	}
-
-        attribute_value.insert(make_pair(tmp.substr(0,mpos),
-                                         tmp.substr(mpos+1)));
+        
+        if ( mpos + 1 == tmp.size() )
+        {
+            attribute_value.insert(make_pair(tmp.substr(0,mpos),""));
+        }
+        else
+        {
+            attribute_value.insert(make_pair(tmp.substr(0,mpos),
+                                             tmp.substr(mpos+1)));
+        } 
     }
 }
 /* -------------------------------------------------------------------------- */
