@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -539,6 +539,50 @@ error:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+int DispatchManager::reboot(int vid)
+{
+    VirtualMachine *    vm;
+    ostringstream       oss;
+
+    vm = vmpool->get(vid,true);
+
+    if ( vm == 0 )
+    {
+        return -1;
+    }
+
+    oss << "Rebooting VM " << vid;
+    NebulaLog::log("DiM",Log::DEBUG,oss);
+
+    if (vm->get_state()     == VirtualMachine::ACTIVE &&
+        vm->get_lcm_state() == VirtualMachine::RUNNING )
+    {
+        Nebula&             nd  = Nebula::instance();
+        LifeCycleManager *  lcm = nd.get_lcm();
+
+        lcm->trigger(LifeCycleManager::REBOOT,vid);
+    }
+    else
+    {
+        goto error;
+    }
+
+    vm->unlock();
+
+    return 0;
+
+error:
+    oss.str("");
+    oss << "Could not reboot VM " << vid << ", wrong state.";
+    NebulaLog::log("DiM",Log::ERROR,oss);
+
+    vm->unlock();
+
+    return -2;
+}
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int DispatchManager::finalize(
     int vid)
 {
@@ -565,6 +609,7 @@ int DispatchManager::finalize(
     switch (state)
     {
         case VirtualMachine::SUSPENDED:
+        case VirtualMachine::FAILED:
             tm->trigger(TransferManager::EPILOG_DELETE,vid);
 
         case VirtualMachine::INIT:
@@ -573,7 +618,6 @@ int DispatchManager::finalize(
         case VirtualMachine::STOPPED:
             vm->set_exit_time(time(0));
 
-        case VirtualMachine::FAILED:
             vm->set_state(VirtualMachine::LCM_INIT);
             vm->set_state(VirtualMachine::DONE);
             vmpool->update(vm);
@@ -608,6 +652,7 @@ int DispatchManager::resubmit(int vid)
 
     Nebula&             nd  = Nebula::instance();
     LifeCycleManager *  lcm = nd.get_lcm();
+    TransferManager *   tm  = nd.get_tm();
 
     vm = vmpool->get(vid,true);
 
@@ -628,9 +673,10 @@ int DispatchManager::resubmit(int vid)
         case VirtualMachine::PENDING:
         break;
 
+        case VirtualMachine::FAILED: //Cleanup VM host files
+            tm->trigger(TransferManager::EPILOG_DELETE,vid);
         case VirtualMachine::HOLD: // Move the VM to PENDING in any of these
         case VirtualMachine::STOPPED:
-        case VirtualMachine::FAILED:
             vm->set_state(VirtualMachine::LCM_INIT);
             vm->set_state(VirtualMachine::PENDING);
             vmpool->update(vm);
