@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,6 +25,7 @@ using namespace std;
 
 const string im_mad  = "im_mad";
 const string vmm_mad = "vmm_mad";
+const string vnm_mad = "vnm_mad";
 const string tm_mad  = "tm_mad";
 
 const string names[] = {"Host one", "Second host"};
@@ -32,8 +33,8 @@ const string names[] = {"Host one", "Second host"};
 const string xmls[] =
 {
     "<HOST><ID>0</ID><NAME>Host one</NAME><STATE>0</STATE>"
-    "<IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD>"
-    "<LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><HOST_SHARE><HID>0</HID>"
+    "<IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD>"
+    "<LAST_MON_TIME>0</LAST_MON_TIME><HOST_SHARE>"
     "<DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE>"
     "<MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU>"
     "<FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU>"
@@ -41,8 +42,8 @@ const string xmls[] =
     "<RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST>",
 
     "<HOST><ID>1</ID><NAME>Second host</NAME><STATE>0</STATE>"
-    "<IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD>"
-    "<LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><HOST_SHARE><HID>1</HID>"
+    "<IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD>"
+    "<LAST_MON_TIME>0</LAST_MON_TIME><HOST_SHARE>"
     "<DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE>"
     "<MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU>"
     "<FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU>"
@@ -53,82 +54,76 @@ const string xmls[] =
 // This xml dump result has the LAST_MON_TIMEs modified to 0000000000
 const string xml_dump =
     "<HOST_POOL><HOST><ID>0</ID><NAME>a</NAME><STATE>0</STATE><IM_MAD>im_mad</I"
-    "M_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0"
-    "</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHARE><HID>0</HID><DISK_USAGE>0</DISK_USAGE><MEM"
+    "M_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0"
+    "</LAST_MON_TIME><HOST_SHARE><DISK_USAGE>0</DISK_USAGE><MEM"
     "_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM"
     ">0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_"
     "MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><U"
-    "SED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST>"
+    "SED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST>"
     "<ID>1</ID><NAME>a name</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MA"
-    "D>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_M"
-    "ON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHARE><HID>1</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</ME"
+    "D>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_M"
+    "ON_TIME><HOST_SHARE><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</ME"
     "M_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM>"
     "<MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CP"
     "U>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</U"
-    "SED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST><ID>2</ID><N"
+    "SED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST><ID>2</ID><N"
     "AME>a_name</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</V"
-    "M_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOS"
-    "T_SHARE><HID>2</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU"
+    "M_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><HOS"
+    "T_SHARE><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU"
     "_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</"
     "MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CP"
     "U><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUN"
-    "NING_VMS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST><ID>3</ID><NAME>another "
+    "NING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST><ID>3</ID><NAME>another "
     "name</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD>"
-    "<TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHAR"
-    "E><HID>3</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE"
+    "<VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><HOST_SHAR"
+    "E><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE"
     ">0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CP"
     "U><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USE"
     "D_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_V"
-    "MS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST><ID>4</ID><NAME>host</NAME><ST"
-    "ATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad"
-    "</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHARE><HID>4</HID>"
+    "MS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST><ID>4</ID><NAME>host</NAME><ST"
+    "ATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad"
+    "</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><HOST_SHARE>"
     "<DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE>"
     "<MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0"
     "</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED"
     "_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_"
-    "VMS></HOST_SHARE></HOST></HOST_POOL>";
+    "VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST></HOST_POOL>";
 
 const string xml_dump_like_a =
     "<HOST_POOL><HOST><ID>0</ID><NAME>a</NAME><STATE>0</STATE><IM_MAD>im_mad</I"
-    "M_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0"
-    "</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHARE><HID>0</HID><DISK_USAGE>0</DISK_USAGE><MEM"
+    "M_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0"
+    "</LAST_MON_TIME><HOST_SHARE><DISK_USAGE>0</DISK_USAGE><MEM"
     "_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM"
     ">0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_"
     "MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><U"
-    "SED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST>"
+    "SED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST>"
     "<ID>1</ID><NAME>a name</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MA"
-    "D>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_M"
-    "ON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHARE><HID>1</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</ME"
+    "D>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_M"
+    "ON_TIME><HOST_SHARE><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</ME"
     "M_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM>"
     "<MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CP"
     "U>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</U"
-    "SED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST><ID>2</ID><N"
+    "SED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST><ID>2</ID><N"
     "AME>a_name</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</V"
-    "M_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOS"
-    "T_SHARE><HID>2</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU"
+    "M_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><HOS"
+    "T_SHARE><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU"
     "_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</"
     "MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CP"
     "U><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUN"
-    "NING_VMS>0</RUNNING_VMS></HOST_SHARE></HOST><HOST><ID>3</ID><NAME>another "
+    "NING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST><HOST><ID>3</ID><NAME>another "
     "name</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD>"
-    "<TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><TEMPLATE></TEMPLATE><HOST_SHAR"
-    "E><HID>3</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE"
+    "<VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><HOST_SHAR"
+    "E><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE"
     ">0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CP"
     "U><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USE"
     "D_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_V"
-    "MS>0</RUNNING_VMS></HOST_SHARE></HOST></HOST_POOL>";
+    "MS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST></HOST_POOL>";
 
 const string host0_updated =
-    "<HOST><ID>0</ID><NAME>Host one</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>default</CLUSTER><HOST_SHARE><HID>0</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE><ATT_A><![CDATA[VALUE_A]]></ATT_A><ATT_B><![CDATA[VALUE_B]]></ATT_B></TEMPLATE></HOST>";
-
-const string cluster_default =
-    "<CLUSTER><ID>0</ID><NAME>default</NAME></CLUSTER>";
-
-const string cluster_xml_dump =
-    "<CLUSTER_POOL><CLUSTER><ID>0</ID><NAME>default</NAME></CLUSTER><CLUSTER><ID>1</ID><NAME>cluster_a</NAME></CLUSTER><CLUSTER><ID>3</ID><NAME>cluster_c</NAME></CLUSTER><CLUSTER><ID>4</ID><NAME>cluster_d</NAME></CLUSTER></CLUSTER_POOL>";
+    "<HOST><ID>0</ID><NAME>Host one</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><HOST_SHARE><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE><ATT_A><![CDATA[VALUE_A]]></ATT_A><ATT_B><![CDATA[VALUE_B]]></ATT_B></TEMPLATE></HOST>";
 
 const string host_0_cluster =
-    "<HOST><ID>0</ID><NAME>Host one</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>cluster_a</CLUSTER><HOST_SHARE><HID>0</HID><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST>";
+    "<HOST><ID>0</ID><NAME>Host one</NAME><STATE>0</STATE><IM_MAD>im_mad</IM_MAD><VM_MAD>vmm_mad</VM_MAD><VN_MAD>vnm_mad</VN_MAD><TM_MAD>tm_mad</TM_MAD><LAST_MON_TIME>0</LAST_MON_TIME><CLUSTER>cluster_a</CLUSTER><HOST_SHARE><DISK_USAGE>0</DISK_USAGE><MEM_USAGE>0</MEM_USAGE><CPU_USAGE>0</CPU_USAGE><MAX_DISK>0</MAX_DISK><MAX_MEM>0</MAX_MEM><MAX_CPU>0</MAX_CPU><FREE_DISK>0</FREE_DISK><FREE_MEM>0</FREE_MEM><FREE_CPU>0</FREE_CPU><USED_DISK>0</USED_DISK><USED_MEM>0</USED_MEM><USED_CPU>0</USED_CPU><RUNNING_VMS>0</RUNNING_VMS></HOST_SHARE><TEMPLATE></TEMPLATE></HOST>";
 /* ************************************************************************* */
 /* ************************************************************************* */
 
@@ -143,15 +138,10 @@ class HostPoolTest : public PoolTest
     CPPUNIT_TEST (dump_where);
     CPPUNIT_TEST (discover);
     CPPUNIT_TEST (duplicates);
-
-    CPPUNIT_TEST (cluster_init);
-    CPPUNIT_TEST (cluster_allocate);
-    CPPUNIT_TEST (cluster_drop);
-    CPPUNIT_TEST (cluster_id);
-    CPPUNIT_TEST (cluster_dump);
-    CPPUNIT_TEST (set_cluster);
-    CPPUNIT_TEST (remove_cluster);
     CPPUNIT_TEST (update_info);
+    CPPUNIT_TEST (name_index);
+
+//    CPPUNIT_TEST (scale_test);
 
     CPPUNIT_TEST_SUITE_END ();
 
@@ -166,7 +156,7 @@ protected:
     {
         vector<const Attribute *> hook;
 
-        return new HostPool(db,hook,"./");
+        return new HostPool(db,hook,"./", "./");
     };
 
     int allocate(int index)
@@ -174,7 +164,7 @@ protected:
         int    oid;
         string err;
         return ((HostPool*)pool)->allocate(&oid, names[index], im_mad,
-                                           vmm_mad, tm_mad, err);
+                                           vmm_mad, vnm_mad, tm_mad, err);
     };
 
     void check(int index, PoolObjectSQL* obj)
@@ -184,12 +174,21 @@ protected:
         CPPUNIT_ASSERT( obj != 0 );
 
         string xml_str = "";
-        string name = host->get_hostname();
+        string name = host->get_name();
 
         CPPUNIT_ASSERT( name == names[index] );
 
         // Get the xml
         host->to_xml(xml_str);
+
+//  A little help for debugging
+//*
+        if( xml_str != xmls[index] )
+        {
+            cout << endl << xml_str << endl << "========"
+                 << endl << xmls[index] << endl;
+        }
+//*/
         CPPUNIT_ASSERT( xml_str == xmls[index]);
     };
 
@@ -245,16 +244,16 @@ public:
 
         // If we try to allocate two hosts with the same name and drivers,
         // should fail
-        rc = hp->allocate(&oid_0, names[0], im_mad, vmm_mad, tm_mad, err);
+        rc = hp->allocate(&oid_0, names[0], im_mad, vmm_mad, vnm_mad, tm_mad, err);
         CPPUNIT_ASSERT( oid_0 == 0 );
         CPPUNIT_ASSERT( rc    == oid_0 );
 
-        rc = hp->allocate(&oid_1, names[0], im_mad, vmm_mad, tm_mad, err);
+        rc = hp->allocate(&oid_1, names[0], im_mad, vmm_mad, vnm_mad, tm_mad, err);
         CPPUNIT_ASSERT( oid_1 == -1 );
         CPPUNIT_ASSERT( rc    == oid_1 );
 
         // the hostname can not be repeated if the drivers change
-        rc = hp->allocate(&oid_1, names[0], im_mad, vmm_mad, tm_mad_2, err);
+        rc = hp->allocate(&oid_1, names[0], im_mad, vmm_mad, vnm_mad, tm_mad_2, err);
         CPPUNIT_ASSERT( oid_1 == -1 );
         CPPUNIT_ASSERT( rc    == oid_1 );
 
@@ -275,7 +274,7 @@ public:
         for(int i=0; i<5; i++)
         {
             ((HostPool*)pool)->allocate(&oid, names[i],
-                                        im_mad, vmm_mad, tm_mad, err);
+                                        im_mad, vmm_mad, vnm_mad, tm_mad, err);
         }
 
         ostringstream oss;
@@ -285,6 +284,15 @@ public:
 
         string result = oss.str();
 
+//  A little help for debugging
+//*
+        if( result != xml_dump )
+        {
+            cout << endl << result << endl << "========"
+                 << endl << xml_dump << endl;
+        }
+//*/
+
         CPPUNIT_ASSERT( result == xml_dump );
     }
 
@@ -299,17 +307,26 @@ public:
         for(int i=0; i<5; i++)
         {
             ((HostPool*)pool)->allocate(&oid, names[i],
-                                        im_mad, vmm_mad, tm_mad, err);
+                                        im_mad, vmm_mad, vnm_mad, tm_mad, err);
         }
 
 
         ostringstream oss;
-        rc = ((HostPool*)pool)->dump(oss, "host_name LIKE 'a%'");
+        rc = ((HostPool*)pool)->dump(oss, "name LIKE 'a%'");
         CPPUNIT_ASSERT(rc == 0);
 
 
         string result = oss.str();
 
+//  A little help for debugging
+//*
+        if( result != xml_dump_like_a )
+        {
+            cout << endl << result << endl << "========"
+                 << endl << xml_dump_like_a << endl;
+        }
+//*/
+
         CPPUNIT_ASSERT( result == xml_dump_like_a );
     }
 
@@ -331,7 +348,7 @@ public:
         {
             oss << "host" << i;
 
-            hp->allocate(&oid, oss.str().c_str(), im_mad, vmm_mad, tm_mad, err);
+            hp->allocate(&oid, oss.str().c_str(), im_mad, vmm_mad, vnm_mad, tm_mad, err);
             CPPUNIT_ASSERT(oid == i);
 
             if (i >=8 )
@@ -360,201 +377,87 @@ public:
         }
     }
 
-    /* ********************************************************************* */
     /* ********************************************************************* */
 
-    void cluster_init()
+    void scale_test()
     {
+        time_t the_time, the_time2;
+        int oid,i,j;
+
+        ostringstream oss,ossdump;
+        string        err;
+        Host * host;
+
+        string monitor = "ARCH=x86_64 MODELNAME=\"Intel(R) Core(TM)2 Duo CPU P9300 @ 2.26GHz\" HYPERVISOR=kvm TOTALCPU=200 CPUSPEED=800 TOTALMEMORY=4005416 USEDMEMORY=2351928 FREEMEMORY=2826904 FREECPU=188.4 USEDCPU=11.599999999999994 NETRX=0 NETTX=0 HOSTNAME=pc-ruben";
+
+        cout << endl << "Allocate Test" << endl;
+
+        tearDown();
+
+        for (i=1000; i<30000 ; i = i + 5000)
+        {
+            setUp();
+            
+            HostPool * hp = static_cast<HostPool *>(pool);
+
+            the_time = time(0);
+
+            for (j=0,oss.str(""); j<i ; j=j+1,oss.str(""))
+            {
+                oss << "host" << j;
+
+                hp->allocate(&oid, oss.str().c_str(),im_mad,vmm_mad,vnm_mad,tm_mad,err);
+            }
+
+            the_time2 = time(0) - the_time;
+
+            hp->clean();
+
+            the_time = time(0);
+
+            hp->dump(ossdump, "");
+
+            cout <<"\t"<<i<<"\t"<<the_time2<<"\t"<<time(0)-the_time<< endl;
+
+            tearDown();
+        } 
+
+        cout << endl << "Read Test" << endl;
+
+        setUp();
+
+
+        // Allocate a HostPool
+        setUp();
+
         HostPool * hp = static_cast<HostPool *>(pool);
 
-        CPPUNIT_ASSERT( hp->info_cluster(0) == cluster_default );
-    }
+        for (i=10000,oss.str(""); i<30000 ; i++,oss.str(""))
+        {
+            oss << "host" << i;
+            hp->allocate(&oid,oss.str().c_str(),im_mad,vmm_mad,vnm_mad,tm_mad,err);
 
-    /* ********************************************************************* */
+            host = hp->get(oid, false);
 
-    void cluster_allocate()
-    {
-        HostPool * hp = static_cast<HostPool *>(pool);
-        int clid, rc;
-        string err;
+            host->update_info(monitor);
+            hp->update(host);
+        }
+           
+       //Load test 
+        for (i=0; i<25000; i=i+5000)
+        {
+            hp->clean();
 
-        rc = hp->allocate_cluster(&clid, "new_cluster", err);
-        CPPUNIT_ASSERT( rc == clid );
-        CPPUNIT_ASSERT( clid == 1 );
+            the_time = time(0);
 
-        CPPUNIT_ASSERT( hp->info_cluster(clid) ==
-                "<CLUSTER><ID>1</ID><NAME>new_cluster</NAME></CLUSTER>");
+            for (j=0; j<i ; j++)
+            {
+                host = hp->get(j,true);
+                host->unlock();
+            }
 
-        // Try to allocate using the same name
-        rc = hp->allocate_cluster(&clid, "new_cluster", err);
-        CPPUNIT_ASSERT( rc == clid );
-        CPPUNIT_ASSERT( clid == -1 );
-    }
-
-    /* ********************************************************************* */
-
-    void cluster_drop()
-    {
-        HostPool * hp = static_cast<HostPool *>(pool);
-        int clid, rc;
-        string err;
-
-        // Drop a non-existing cluster
-        rc = hp->drop_cluster(20);
-        CPPUNIT_ASSERT( rc == -1 );
-
-        // Allocate a cluster and drop it
-        rc = hp->allocate_cluster(&clid, "new_cluster", err);
-        CPPUNIT_ASSERT( clid == 1);
-
-        rc = hp->drop_cluster(clid);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        // Try to drop the default cluster, should fail
-        rc = hp->drop_cluster(0);
-        CPPUNIT_ASSERT( rc == -1 );
-    }
-
-    /* ********************************************************************* */
-
-    void cluster_id()
-    {
-        HostPool * hp = static_cast<HostPool *>(pool);
-        int clid, rc;
-        ostringstream oss;
-        string err;
-
-        // Allocate some clusters
-        rc = hp->allocate_cluster(&clid, "cluster_a", err);
-        CPPUNIT_ASSERT( rc == 1 );
-
-        rc = hp->allocate_cluster(&clid, "cluster_b", err);
-        CPPUNIT_ASSERT( rc == 2 );
-
-        rc = hp->allocate_cluster(&clid, "cluster_c", err);
-        CPPUNIT_ASSERT( rc == 3 );
-
-        rc = hp->allocate_cluster(&clid, "cluster_d", err);
-        CPPUNIT_ASSERT( rc == 4 );
-
-        // Drop id 2
-        rc = hp->drop_cluster(2);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        // Next one should use id 5, because the biggest id is 4
-        rc = hp->allocate_cluster(&clid, "cluster_e", err);
-        CPPUNIT_ASSERT( rc == 5 );
-
-        // Drop id 5
-        rc = hp->drop_cluster(5);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        // Next one should use id 5, because the biggest id is 4 again
-        rc = hp->allocate_cluster(&clid, "cluster_f", err);
-        CPPUNIT_ASSERT( rc == 5 );
-
-    }
-
-    /* ********************************************************************* */
-
-    void cluster_dump()
-    {
-        HostPool * hp = static_cast<HostPool *>(pool);
-        int clid, rc;
-        ostringstream oss;
-        string err;
-
-        // Allocate some clusters
-        rc = hp->allocate_cluster(&clid, "cluster_a", err);
-        CPPUNIT_ASSERT( rc == 1 );
-
-        rc = hp->allocate_cluster(&clid, "cluster_b", err);
-        CPPUNIT_ASSERT( rc == 2 );
-
-        rc = hp->allocate_cluster(&clid, "cluster_c", err);
-        CPPUNIT_ASSERT( rc == 3 );
-
-        rc = hp->allocate_cluster(&clid, "cluster_d", err);
-        CPPUNIT_ASSERT( rc == 4 );
-
-
-        // Drop one of them
-        rc = hp->drop_cluster(2);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        // dump the pool
-        rc = hp->dump_cluster(oss);
-        CPPUNIT_ASSERT( oss.str() == cluster_xml_dump );
-    }
-
-    /* ********************************************************************* */
-
-    void set_cluster()
-    {
-        HostPool *  hp = static_cast<HostPool *>(pool);
-        Host*       host;
-        int         clid, rc, oid;
-        string      xml_str, err;
-
-        // Allocate a host
-        oid = allocate(0);
-        CPPUNIT_ASSERT(oid >= 0);
-
-        host = hp->get(0, false);
-        CPPUNIT_ASSERT(host != 0);
-
-        rc = hp->allocate_cluster(&clid, "cluster_a", err);
-        CPPUNIT_ASSERT( rc == 1 );
-
-        rc = hp->set_cluster(host, clid);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        host->to_xml(xml_str);
-        CPPUNIT_ASSERT( xml_str == host_0_cluster);
-
-
-        // Try to set a non-existing cluster
-        rc = hp->set_cluster(host, 20);
-        CPPUNIT_ASSERT( rc == -1 );
-
-        CPPUNIT_ASSERT( xml_str == host_0_cluster);
-
-    }
-
-    /* ********************************************************************* */
-
-    void remove_cluster()
-    {
-        HostPool *  hp = static_cast<HostPool *>(pool);
-        Host*       host;
-        int         clid, rc, oid;
-        string      xml_str, err;
-
-        // Allocate a host
-        oid = allocate(0);
-        CPPUNIT_ASSERT(oid >= 0);
-
-        host = hp->get(0, false);
-        CPPUNIT_ASSERT(host != 0);
-
-        rc = hp->allocate_cluster(&clid, "cluster_a", err);
-        CPPUNIT_ASSERT( rc == 1 );
-
-        // Set host 0 to cluster 1
-        rc = hp->set_cluster(host, clid);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        // Check
-        host->to_xml(xml_str);
-        CPPUNIT_ASSERT( xml_str == host_0_cluster);
-
-        // Remove the cluster
-        rc = hp->set_default_cluster(host);
-        CPPUNIT_ASSERT( rc == 0 );
-
-        // The host should have been moved to the default cluster
-        host->to_xml(xml_str);
-        check(0, host);
+            cout << "\t" << i << "\t" << time(0) - the_time << endl;
+        }
     }
 
     /* ********************************************************************* */
@@ -590,6 +493,70 @@ public:
         CPPUNIT_ASSERT( host != 0 );
         CPPUNIT_ASSERT( host->to_xml(str) == host0_updated );
     }
+
+    /* ********************************************************************* */
+
+    void name_index()
+    {
+        HostPool *  hp = static_cast<HostPool *>(pool);
+        Host        *host_oid, *host_name;
+        int         oid_0;
+        int         uid_0;
+        string      name_0;
+
+        oid_0 = allocate(0);
+
+        CPPUNIT_ASSERT(oid_0 != -1);
+
+        // ---------------------------------
+        // Get by oid
+        host_oid = hp->get(oid_0, true);
+        CPPUNIT_ASSERT(host_oid != 0);
+
+        name_0 = host_oid->get_name();
+        uid_0  = host_oid->get_uid();
+
+        host_oid->unlock();
+
+        // Get by name and check it is the same object
+        host_name = hp->get(name_0, true);
+        CPPUNIT_ASSERT(host_name != 0);
+        host_name->unlock();
+
+        CPPUNIT_ASSERT(host_oid == host_name);
+
+        // ---------------------------------
+        // Clean the cache, forcing the pool to read the objects from the DB
+        hp->clean();
+
+        // Get by oid
+        host_oid = hp->get(oid_0, true);
+        CPPUNIT_ASSERT(host_oid != 0);
+        host_oid->unlock();
+
+        // Get by name and check it is the same object
+        host_name = hp->get(name_0, true);
+        CPPUNIT_ASSERT(host_name != 0);
+        host_name->unlock();
+
+        CPPUNIT_ASSERT(host_oid == host_name);
+
+        // ---------------------------------
+        // Clean the cache, forcing the pool to read the objects from the DB
+        hp->clean();
+
+        // Get by name
+        host_name = hp->get(name_0, true);
+        CPPUNIT_ASSERT(host_name != 0);
+        host_name->unlock();
+
+        // Get by oid and check it is the same object
+        host_oid = hp->get(oid_0, true);
+        CPPUNIT_ASSERT(host_oid != 0);
+        host_oid->unlock();
+
+        CPPUNIT_ASSERT(host_oid == host_name);
+    }
 };
 
 
