@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,68 +20,36 @@
 
 #include "ImagePool.h"
 #include "AuthManager.h"
+#include "Nebula.h"
+#include "PoolObjectAuth.h"
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
-string ImagePool::_source_prefix;
 string ImagePool::_default_type;
 string ImagePool::_default_dev_prefix;
 
-int ImagePool::init_cb(void *nil, int num, char **values, char **names)
-{
-    if ( num == 0 || values == 0 || values[0] == 0 )
-    {
-        return -1;
-    }
-
-    image_names.insert(make_pair(values[1],atoi(values[0])));
-
-    return 0;
-}
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-
-ImagePool::ImagePool(   SqlDB * db,
-                        const string&   __source_prefix,
-                        const string&   __default_type,
-                        const string&   __default_dev_prefix):
-
+ImagePool::ImagePool(SqlDB *       db,
+                     const string& __default_type,
+                     const string& __default_dev_prefix):
                         PoolSQL(db,Image::table)
 {
-    ostringstream   sql;
-    int             rc;
+    ostringstream sql;
 
     // Init static defaults
-    _source_prefix       = __source_prefix;
-    _default_type        = __default_type;
-    _default_dev_prefix  = __default_dev_prefix;
+    _default_type       = __default_type;
+    _default_dev_prefix = __default_dev_prefix;
 
     // Set default type
     if (_default_type != "OS"       &&
         _default_type != "CDROM"    &&
         _default_type != "DATABLOCK" )
     {
-        NebulaLog::log("IMG", Log::ERROR,
-                 "Bad default for image type, setting OS");
+        NebulaLog::log("IMG", Log::ERROR, "Bad default for type, setting OS");
         _default_type = "OS";
     }
-
-    // Read from the DB the existing images, and build the ID:Name map
-    set_callback(static_cast<Callbackable::Callback>(&ImagePool::init_cb));
-
-    sql  << "SELECT oid, name FROM " <<  Image::table;
-
-    rc = db->exec(sql, this);
-
-    unset_callback();
-
-    if ( rc != 0 )
-    {
-        NebulaLog::log("IMG", Log::ERROR,
-                 "Could not load the existing images from the DB.");
-    }
 }
 
 /* -------------------------------------------------------------------------- */
@@ -89,134 +57,205 @@ ImagePool::ImagePool(   SqlDB * db,
 
 int ImagePool::allocate (
         int            uid,
+        int            gid,
+        const string&  uname,
+        const string&  gname,
         ImageTemplate* img_template,
         int *          oid,
         string&        error_str)
 {
-    Image * img;
-    string  name;
+    Image *         img;
+    Image *         img_aux = 0;
+    string          name;
+    ostringstream   oss;
 
-    // ---------------------------------------------------------------------
-    // Build a new Image object
-    // ---------------------------------------------------------------------
-    img = new Image(uid,img_template);
+    img = new Image(uid, gid, uname, gname, img_template);
 
+    // Check name
     img->get_template_attribute("NAME", name);
 
-    // ---------------------------------------------------------------------
-    // Insert the Object in the pool
-    // ---------------------------------------------------------------------
-    *oid = PoolSQL::allocate(img, error_str);
+    if ( name.empty() )
+    {
+        goto error_name;
+    }
+
+    if ( name.length() > 128 )
+    {
+        goto error_name_length;
+    }
+
+    // Check for duplicates
+    img_aux = get(name,uid,false);
+
+    if( img_aux != 0 )
+    {
+        goto error_duplicated;
+    }
 
     // ---------------------------------------------------------------------
-    // Add the image name to the map of image_names
+    // Insert the Object in the pool & Register the image in the repository
     // ---------------------------------------------------------------------
+    *oid = PoolSQL::allocate(img, error_str);
+    
     if ( *oid != -1 )
     {
-        image_names.insert(make_pair(name, *oid));
+        Nebula&        nd     = Nebula::instance();
+        ImageManager * imagem = nd.get_imagem();
+
+        if ( imagem->register_image(*oid) == -1 )
+        {
+            error_str = "Failed to copy image to repository. "
+                        "Image left in ERROR state.";
+            return -1;
+        }
     }
 
+    return *oid;
+
+error_name:
+    oss << "NAME cannot be empty.";
+
+    goto error_common;
+
+error_name_length:
+    oss << "NAME is too long; max length is 128 chars.";
+    goto error_common;
+
+error_duplicated:
+    oss << "NAME is already taken by IMAGE "
+        << img_aux->get_oid() << ".";
+
+error_common:
+    delete img;
+
+    *oid = -1;
+    error_str = oss.str();
+
     return *oid;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int ImagePool::dump_cb(void * _oss, int num, char **values, char **names)
+static int get_disk_uid(VectorAttribute *  disk, int _uid)
 {
-    ostringstream * oss;
+    istringstream  is;
 
-    oss = static_cast<ostringstream *>(_oss);
+    string uid_s ;
+    string uname;
+    int    uid;
 
-    return Image::dump(*oss, num, values, names);
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-int ImagePool::dump(ostringstream& oss, const string& where)
-{
-    int             rc;
-    ostringstream   cmd;
-
-    oss << "<IMAGE_POOL>";
-
-    set_callback(static_cast<Callbackable::Callback>(&ImagePool::dump_cb),
-                  static_cast<void *>(&oss));
-
-    cmd << "SELECT "<< Image::extended_db_names << ", user_pool.user_name FROM "
-        << Image::table
-        << " LEFT OUTER JOIN (SELECT oid, user_name FROM user_pool) "
-        << "AS user_pool ON " << Image::table << ".uid = user_pool.oid";
-
-    if ( !where.empty() )
+    if (!(uid_s = disk->vector_value("IMAGE_UID")).empty())
     {
-        cmd << " WHERE " << where;
+        is.str(uid_s);
+        is >> uid;
+
+        if( is.fail() )
+        {
+            return -1;
+        }
+    }
+    else if (!(uname = disk->vector_value("IMAGE_UNAME")).empty())
+    {
+        User *     user;
+        Nebula&    nd    = Nebula::instance();
+        UserPool * upool = nd.get_upool();
+        
+        user = upool->get(uname,true);
+        
+        if ( user == 0 )
+        {
+            return -1;
+        }
+
+        uid = user->get_oid();
+
+        user->unlock();
+    }
+    else
+    {
+        uid = _uid;        
     }
 
-    rc = db->exec(cmd, this);
+    return uid;
+}
+        
+/* -------------------------------------------------------------------------- */
 
-    oss << "</IMAGE_POOL>";
+static int get_disk_id(const string& id_s)
+{
+    istringstream  is;
+    int            id;
 
-    unset_callback();
+    is.str(id_s);
+    is >> id;
 
-    return rc;
+    if( is.fail() )
+    {
+        return -1;
+    }
+
+    return id;
 }
 
-/* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 int ImagePool::disk_attribute(VectorAttribute *  disk,
                               int                disk_id,
                               int *              index,
-                              Image::ImageType * img_type)
+                              Image::ImageType * img_type,
+                              int                uid,
+                              int&               image_id)
 {
     string  source;
     Image * img = 0;
-    int     rc;
+    int     rc  = 0;
 
     ostringstream oss;
 
-    source = disk->vector_value("IMAGE");
+    Nebula&        nd     = Nebula::instance();
+    ImageManager * imagem = nd.get_imagem();
 
-    if (source.empty())
+    if (!(source = disk->vector_value("IMAGE")).empty())
     {
-        istringstream   is;
-        int             image_id;
-
-        source = disk->vector_value("IMAGE_ID");
-
-        if (!source.empty())
+        int uiid = get_disk_uid(disk,uid);
+       
+        if ( uiid == -1)
         {
-            is.str(source);
-            is >> image_id;
-
-            if( !is.fail() )
-            {
-                img = get(image_id,true);
-
-                if (img == 0)
-                {
-                    return -1;
-                }
-            }
+            return -1; 
         }
-    }
-    else
-    {
-        img = get(source,true);
 
-        if (img == 0)
+        img = imagem->acquire_image(source, uiid);
+
+        if ( img == 0 )
         {
             return -1;
         }
     }
-
-    if (img == 0)
+    else if (!(source = disk->vector_value("IMAGE_ID")).empty())
     {
-        string type = disk->vector_value("TYPE");
+        int iid = get_disk_id(source);
 
-        transform(type.begin(), type.end(), type.begin(), (int(*)(int))toupper);
+        if ( iid == -1)
+        {
+            return -1; 
+        }
+
+        img = imagem->acquire_image(iid);
+
+        if ( img == 0 )
+        {
+            return -1;
+        }
+    }
+    else //Not using the image repository
+    {
+        string type;
+
+        rc   = -2;
+        type = disk->vector_value("TYPE");
+
+        transform(type.begin(),type.end(),type.begin(),(int(*)(int))toupper);
 
         if( type == "SWAP" )
         {
@@ -231,17 +270,15 @@ int ImagePool::disk_attribute(VectorAttribute *  disk,
                 disk->replace("TARGET", dev_prefix);
             }
         }
-
-        rc = -2;
     }
-    else
-    {
-        rc = img->disk_attribute(disk, index, img_type);
 
-        if ( rc == 0 )
-        {
-            update(img);
-        }
+    if ( img != 0 )
+    {
+        img->disk_attribute(disk, index, img_type);
+
+        image_id = img->get_oid();
+        
+        update(img);
 
         img->unlock();
     }
@@ -255,36 +292,34 @@ int ImagePool::disk_attribute(VectorAttribute *  disk,
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void ImagePool::authorize_disk(VectorAttribute * disk, AuthRequest * ar)
+void ImagePool::authorize_disk(VectorAttribute * disk,int uid, AuthRequest * ar)
 {
-    string  source;
-    Image * img = 0;
+    string          source;
+    Image *         img = 0;
+    
+    PoolObjectAuth  perm;
 
-    source = disk->vector_value("IMAGE");
-
-    if (source.empty())
+    if (!(source = disk->vector_value("IMAGE")).empty())
     {
-        istringstream   is;
-        int             image_id;
-
-        source = disk->vector_value("IMAGE_ID");
-
-        if (source.empty())
+        int uiid = get_disk_uid(disk,uid);
+       
+        if ( uiid == -1)
         {
-            return;
+            return; 
         }
 
-        is.str(source);
-        is >> image_id;
-
-        if( !is.fail() )
-        {
-            img = get(image_id,true);
-        }
+        img = get(source , uiid, true);
     }
-    else
+    else if (!(source = disk->vector_value("IMAGE_ID")).empty())
     {
-        img = get(source,true);
+        int iid = get_disk_id(source);
+
+        if ( iid == -1)
+        {
+            return; 
+        }
+
+        img = get(iid, true);
     }
 
     if (img == 0)
@@ -292,11 +327,9 @@ void ImagePool::authorize_disk(VectorAttribute * disk, AuthRequest * ar)
         return;
     }
 
-    ar->add_auth(AuthRequest::IMAGE,
-                 img->get_iid(),
-                 AuthRequest::USE,
-                 img->get_uid(),
-                 img->isPublic());
+    img->get_permissions(perm);
 
     img->unlock();
+
+    ar->add_auth(AuthRequest::USE, perm);
 }
