@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -22,6 +22,10 @@
 using namespace std;
 
 const int uids[] = {123, 261, 123};
+const int gids[] = {150, 164, 175};
+
+const char* unames[] = {"one", "two", "three"};
+const char* gnames[] = {"oneadmin", "oneadmin", "users"};
 
 const string names[] = {"VM one", "Second VM", "VM 3"};
 
@@ -43,6 +47,7 @@ const string templates[] =
 static int     hid = 123;
 static string  hostname    = "test_hostname";
 static string  vmm_mad     = "vmm_mad";
+static string  vnm_mad     = "vnm_mad";
 static string  tm_mad      = "tm_mad";
 static string  vmdir       = "vmdir";
 
@@ -185,8 +190,17 @@ private:
 
         if( rc == 0 )
         {
-            return vmpool->allocate(uids[index], vm_template, &oid,
-                                                err, false);
+            string uname = unames[index];
+            string gname = gnames[index];
+
+            return vmpool->allocate(uids[index], 
+                                    gids[index], 
+                                    uname,
+                                    gname,
+                                    vm_template, 
+                                    &oid,
+                                    err, 
+                                    false);
         }
         else
         {
@@ -212,7 +226,7 @@ private:
 
         vm->lock();
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
 
@@ -476,7 +490,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -560,7 +574,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -581,7 +595,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -602,7 +616,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -624,7 +638,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -648,7 +662,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -672,7 +686,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
 
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
@@ -701,7 +715,7 @@ public:
 
         dm->cancel(vm->get_oid());
 
-        wait_assert(vm, VirtualMachine::DONE);
+        wait_assert(vm, VirtualMachine::ACTIVE, VirtualMachine::EPILOG );
     }
 
 /* -------------------------------------------------------------------------- */
@@ -736,7 +750,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
 
@@ -759,7 +773,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
 
@@ -782,7 +796,7 @@ public:
     {
         vm = allocate_running(0);
 
-        vm->add_history(hid,hostname,vmdir,vmm_mad,tm_mad);
+        vm->add_history(hid,hostname,vmdir,vmm_mad,vnm_mad,tm_mad);
         rc = vmpool->update_history(vm);
         CPPUNIT_ASSERT( rc == 0 );
 
