@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- */
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 # not use this file except in compliance with the License. You may obtain    */
 # a copy of the License at                                                   */
@@ -16,7 +16,6 @@
 require 'thread'
 
 =begin rdoc
-Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
 
 This class provides support to handle actions. Class methods, or actions, can be
 registered in the action manager. The manager will wait for actions to be
@@ -177,40 +176,6 @@ class ActionManager
     
 private
     
-    def read_configuration
-        one_config=nil
-        
-        if ENV['ONE_LOCATION']
-            one_config=ENV['ONE_LOCATION']+'/var/config'
-        else
-            one_config='/var/lib/one/config'
-        end
-        
-        config=Hash.new
-        cfg=''
-        
-        begin
-            open(one_config) do |file|
-                cfg=file.read
-            end
-            
-            cfg.split(/\n/).each do |line|
-                m=line.match(/^([^=]+)=(.*)$/)
-                
-                if m
-                    name=m[1].strip.upcase
-                    value=m[2].strip
-                    config[name]=value
-                end
-            end
-        rescue Exception => e
-            STDERR.puts "Error reading config: #{e.inspect}"
-            STDERR.flush
-        end
-        
-        config
-    end
-    
     def delete_running_action(action_id)
         @action_running.delete(action_id)
     end
