@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -43,6 +43,7 @@ void Nebula::start()
     sigset_t        mask;
     int             signal;
     char            hn[80];
+    string          scripts_remote_dir;
 
     if ( gethostname(hn,79) != 0 )
     {
@@ -55,7 +56,7 @@ void Nebula::start()
     // Configuration
     // -----------------------------------------------------------
 
-    nebula_configuration = new NebulaTemplate(etc_location, var_location);
+    nebula_configuration = new OpenNebulaTemplate(etc_location, var_location);
 
     rc = nebula_configuration->load_configuration();
 
@@ -73,6 +74,9 @@ void Nebula::start()
         config_file.close();
     }
 
+    nebula_configuration->get("SCRIPTS_REMOTE_DIR", scripts_remote_dir);
+    hook_location = scripts_remote_dir + "/hooks/";
+
     // -----------------------------------------------------------
     // Log system
     // -----------------------------------------------------------
@@ -101,9 +105,17 @@ void Nebula::start()
                                    log_fname.c_str(),
                                    ios_base::trunc);
 
-        NebulaLog::log("ONE",Log::INFO,"Init OpenNebula Log system");
+        os << "Starting " << version() << endl;
+        os << "----------------------------------------\n";
+        os << "     OpenNebula Configuration File      \n";
+        os << "----------------------------------------\n";
+        os << *nebula_configuration;
+        os << "----------------------------------------";
 
-        os << "Log Level: " << clevel << " [0=ERROR,1=WARNING,2=INFO,3=DEBUG]";
+        NebulaLog::log("ONE",Log::INFO,os);
+
+        os.str("");
+        os << "Log level:" << clevel << " [0=ERROR,1=WARNING,2=INFO,3=DEBUG]";
 
         NebulaLog::log("ONE",Log::INFO,os);
     }
@@ -112,18 +124,6 @@ void Nebula::start()
         throw;
     }
 
-
-    NebulaLog::log("ONE",Log::INFO,"----------------------------------------");
-    NebulaLog::log("ONE",Log::INFO,"     OpenNebula Configuration File      ");
-    NebulaLog::log("ONE",Log::INFO,"----------------------------------------");
-
-    os.str("");
-    os << "\n----------------------------------\n";
-    os << *nebula_configuration;
-    os << "----------------------------------";
-
-    NebulaLog::log("ONE",Log::INFO,os);
-
     // -----------------------------------------------------------
     // Initialize the XML library
     // -----------------------------------------------------------
@@ -132,7 +132,6 @@ void Nebula::start()
     // -----------------------------------------------------------
     // Pools
     // -----------------------------------------------------------
-
     try
     {
         vector<const Attribute *> dbs;
@@ -208,7 +207,7 @@ void Nebula::start()
         {
             ostringstream   oss;
 
-            db = new MySqlDB(server,port,user,passwd,0);
+            db = new MySqlDB(server,port,user,passwd,db_name);
 
             oss << "CREATE DATABASE IF NOT EXISTS " << db_name;
             rc = db->exec(oss);
@@ -227,13 +226,40 @@ void Nebula::start()
             }
         }
 
-        NebulaLog::log("ONE",Log::INFO,"Bootstraping OpenNebula database.");
+        NebulaLog::log("ONE",Log::INFO,"Checking database version.");
+        rc = check_db_version();
 
-        VirtualMachinePool::bootstrap(db);
-        HostPool::bootstrap(db);
-        VirtualNetworkPool::bootstrap(db);
-        UserPool::bootstrap(db);
-        ImagePool::bootstrap(db);
+        if( rc == -1 )
+        {
+            throw runtime_error("Database version mismatch.");
+        }
+
+        if( rc == -2 )
+        {
+            rc = 0;
+
+            NebulaLog::log("ONE",Log::INFO,"Bootstrapping OpenNebula database.");
+
+            rc += VirtualMachinePool::bootstrap(db);
+            rc += HostPool::bootstrap(db);
+            rc += VirtualNetworkPool::bootstrap(db);
+            rc += GroupPool::bootstrap(db);
+            rc += UserPool::bootstrap(db);
+            rc += ImagePool::bootstrap(db);
+            rc += VMTemplatePool::bootstrap(db);
+            rc += AclManager::bootstrap(db);
+
+            // Create the versioning table only if bootstrap went well
+            if ( rc == 0 )
+            {
+                rc += bootstrap();
+            }
+
+            if ( rc != 0 )
+            {
+                throw runtime_error("Error bootstrapping database.");
+            }
+        }
     }
     catch (exception&)
     {
@@ -244,9 +270,9 @@ void Nebula::start()
     {
         string  mac_prefix;
         int     size;
-        string  repository_path;
         string  default_image_type;
         string  default_device_prefix;
+        time_t  expiration_time;
 
         vector<const Attribute *> vm_hooks;
         vector<const Attribute *> host_hooks;
@@ -254,25 +280,30 @@ void Nebula::start()
         nebula_configuration->get("VM_HOOK", vm_hooks);
         nebula_configuration->get("HOST_HOOK", host_hooks);
 
-        vmpool = new VirtualMachinePool(db, vm_hooks, hook_location);
-        hpool  = new HostPool(db, host_hooks, hook_location);
+        vmpool = new VirtualMachinePool(db, 
+                                        vm_hooks, 
+                                        hook_location, 
+                                        remotes_location);
+        hpool  = new HostPool(db, host_hooks, hook_location, remotes_location);
 
         nebula_configuration->get("MAC_PREFIX", mac_prefix);
         nebula_configuration->get("NETWORK_SIZE", size);
 
         vnpool = new VirtualNetworkPool(db,mac_prefix,size);
+        gpool  = new GroupPool(db);
 
-        upool  = new UserPool(db);
+        nebula_configuration->get("SESSION_EXPIRATION_TIME", expiration_time);
+        upool  = new UserPool(db, expiration_time);
 
-        nebula_configuration->get("IMAGE_REPOSITORY_PATH", repository_path);
         nebula_configuration->get("DEFAULT_IMAGE_TYPE", default_image_type);
         nebula_configuration->get("DEFAULT_DEVICE_PREFIX",
                                   default_device_prefix);
 
         ipool  = new ImagePool(db,
-                               repository_path,
                                default_image_type,
                                default_device_prefix);
+
+        tpool  = new VMTemplatePool(db);
     }
     catch (exception&)
     {
@@ -318,9 +349,12 @@ void Nebula::start()
     {
         time_t                    poll_period;
         vector<const Attribute *> vmm_mads;
+        int                       vm_limit;
 
         nebula_configuration->get("VM_POLLING_INTERVAL", poll_period);
 
+        nebula_configuration->get("VM_PER_INTERVAL", vm_limit);
+
         nebula_configuration->get("VM_MAD", vmm_mads);
 
         vmm = new VirtualMachineManager(
@@ -328,6 +362,7 @@ void Nebula::start()
             hpool,
             timer_period,
             poll_period,
+            vm_limit,
             vmm_mads);
     }
     catch (bad_alloc&)
@@ -364,14 +399,18 @@ void Nebula::start()
     {
         vector<const Attribute *>   im_mads;
         time_t                      monitor_period;
+        int                         host_limit;
 
         nebula_configuration->get("HOST_MONITORING_INTERVAL", monitor_period);
 
+        nebula_configuration->get("HOST_PER_INTERVAL", host_limit);
+
         nebula_configuration->get("IM_MAD", im_mads);
 
         im = new InformationManager(hpool,
                                     timer_period,
                                     monitor_period,
+                                    host_limit,
                                     remotes_location,
                                     im_mads);
     }
@@ -425,35 +464,6 @@ void Nebula::start()
        throw runtime_error("Could not start the Dispatch Manager");
     }
 
-    // ---- Request Manager ----
-    try
-    {
-        int             rm_port = 0;
-
-        nebula_configuration->get("PORT", rm_port);
-
-        rm = new RequestManager(
-            vmpool,
-            hpool,
-            vnpool,
-            upool,
-            ipool,
-            rm_port,
-            log_location + "one_xmlrpc.log");
-    }
-    catch (bad_alloc&)
-    {
-        NebulaLog::log("ONE", Log::ERROR, "Error starting RM");
-        throw;
-    }
-
-    rc = rm->start();
-
-    if ( rc != 0 )
-    {
-       throw runtime_error("Could not start the Request Manager");
-    }
-
     // ---- Hook Manager ----
     try
     {
@@ -507,6 +517,66 @@ void Nebula::start()
         }
     }
 
+    // ---- ACL Manager ----
+    try
+    {
+        aclm = new AclManager(db);
+    }
+    catch (bad_alloc&)
+    {
+        throw;
+    }
+
+    rc = aclm->start();
+
+    if ( rc != 0 )
+    {
+       throw runtime_error("Could not start the ACL Manager");
+    }
+
+    // ---- Image Manager ----
+    try
+    {
+        vector<const Attribute *> image_mads;
+
+        nebula_configuration->get("IMAGE_MAD", image_mads);
+
+        imagem = new ImageManager(ipool,image_mads);
+    }
+    catch (bad_alloc&)
+    {
+        throw;
+    }
+
+    rc = imagem->start();
+
+    if ( rc != 0 )
+    {
+       throw runtime_error("Could not start the Image Manager");
+    }
+
+    // ---- Request Manager ----
+    try
+    {
+        int             rm_port = 0;
+
+        nebula_configuration->get("PORT", rm_port);
+
+        rm = new RequestManager(rm_port, log_location + "one_xmlrpc.log");
+    }
+    catch (bad_alloc&)
+    {
+        NebulaLog::log("ONE", Log::ERROR, "Error starting RM");
+        throw;
+    }
+
+    rc = rm->start();
+
+    if ( rc != 0 )
+    {
+       throw runtime_error("Could not start the Request Manager");
+    }
+
     // -----------------------------------------------------------
     // Load mads
     // -----------------------------------------------------------
@@ -518,6 +588,7 @@ void Nebula::start()
     im->load_mads(0);
     tm->load_mads(0);
     hm->load_mads(0);
+    imagem->load_mads(0);
 
     if ( authm != 0 )
     {
@@ -548,6 +619,7 @@ void Nebula::start()
     im->finalize();
     rm->finalize();
     hm->finalize();
+    imagem->finalize();
 
     //sleep to wait drivers???
 
@@ -559,9 +631,107 @@ void Nebula::start()
     pthread_join(im->get_thread_id(),0);
     pthread_join(rm->get_thread_id(),0);
     pthread_join(hm->get_thread_id(),0);
+    pthread_join(imagem->get_thread_id(),0);
 
     //XML Library
     xmlCleanupParser();
 
     NebulaLog::log("ONE", Log::INFO, "All modules finalized, exiting.\n");
 }
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int Nebula::bootstrap()
+{
+    int             rc;
+    ostringstream   oss;
+
+    oss <<  "CREATE TABLE pool_control (tablename VARCHAR(32) PRIMARY KEY, "
+            "last_oid BIGINT UNSIGNED)";
+
+    rc = db->exec(oss);
+
+    oss.str("");
+    oss <<  "CREATE TABLE db_versioning (oid INTEGER PRIMARY KEY, "
+            "version VARCHAR(256), timestamp INTEGER, comment VARCHAR(256))";
+
+    rc += db->exec(oss);
+
+    oss.str("");
+    oss << "INSERT INTO db_versioning (oid, version, timestamp, comment) "
+        << "VALUES (0, '" << db_version() << "', " << time(0)
+        << ", '" << version() << " daemon bootstrap')";
+
+    rc += db->exec(oss);
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int Nebula::check_db_version()
+{
+    int             rc;
+    ostringstream   oss;
+
+
+    string loaded_db_version = "";
+
+    // Try to read latest version
+    set_callback( static_cast<Callbackable::Callback>(&Nebula::select_cb),
+                  static_cast<void *>(&loaded_db_version) );
+
+    oss << "SELECT version FROM db_versioning "
+        << "WHERE oid=(SELECT MAX(oid) FROM db_versioning)";
+
+    db->exec(oss, this);
+
+    oss.str("");
+    unset_callback();
+
+    if( loaded_db_version == "" )
+    {
+        // Table user_pool is present for all OpenNebula versions, and it
+        // always contains at least the oneadmin user.
+        oss << "SELECT MAX(oid) FROM user_pool";
+        rc = db->exec(oss);
+
+        oss.str("");
+
+        if( rc != 0 )   // Database needs bootstrap
+        {
+            return -2;
+        }
+    }
+
+    if( db_version() != loaded_db_version )
+    {
+        oss << "Database version mismatch. "
+            << "Installed " << version() << " uses DB version '" << db_version()
+            << "', and existing DB version is '"
+            << loaded_db_version << "'.";
+
+        NebulaLog::log("ONE",Log::ERROR,oss);
+        return -1;
+    }
+
+    return 0;
+}
+
+int Nebula::select_cb(void *_loaded_db_version, int num, char **values,
+                      char **names)
+{
+    istringstream   iss;
+    string *        loaded_db_version;
+
+    loaded_db_version = static_cast<string *>(_loaded_db_version);
+
+    if ( (values[0]) && (num == 1) )
+    {
+        *loaded_db_version = values[0];
+    }
+
+    return 0;
+};
