@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -15,39 +15,83 @@
 /* -------------------------------------------------------------------------- */
 
 #include "NebulaTemplate.h"
-#include "Nebula.h"
 
 using namespace std;
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-const char * NebulaTemplate::conf_name="oned.conf";
+int NebulaTemplate::load_configuration()
+{
+    char * error = 0;
+    int    rc;
 
+    string      aname;
+    Attribute * attr;
+
+    map<string, Attribute *>::iterator  iter, j;
+
+    set_conf_default();
+
+    rc = parse(conf_file.c_str(), &error);
+
+    if ( rc != 0 && error != 0)
+    {
+        cout << "\nError while parsing configuration file:\n" << error << endl;
+
+        free(error);
+
+        return -1;
+    }
+
+    for(iter=conf_default.begin();iter!=conf_default.end();)
+    {
+        aname = iter->first;
+        attr  = iter->second;
+
+        j = attributes.find(aname);
+
+        if ( j == attributes.end() )
+        {
+            attributes.insert(make_pair(aname,attr));
+            iter++;
+        }
+        else
+        {
+            delete iter->second;
+            conf_default.erase(iter++);
+        }
+    }
+
+    return 0;
+}
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+const char * OpenNebulaTemplate::conf_name="oned.conf";
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-NebulaTemplate::NebulaTemplate(string& etc_location, string& var_location)
+void OpenNebulaTemplate::set_conf_default()
 {
-    ostringstream       os;
     SingleAttribute *   attribute;
     VectorAttribute *   vattribute;
     string              value;
 
-    conf_file = etc_location + conf_name;
-
     // MANAGER_TIMER
     value = "15";
 
     attribute = new SingleAttribute("MANAGER_TIMER",value);
     conf_default.insert(make_pair(attribute->name(),attribute));
-
 /*
 #*******************************************************************************
 # Daemon configuration attributes
 #-------------------------------------------------------------------------------
 #  HOST_MONITORING_INTERVAL
+#  HOST_PER_INTERVAL
 #  VM_POLLING_INTERVAL
+#  VM_PER_INTERVAL
 #  VM_DIR
 #  PORT
 #  DB
@@ -61,12 +105,24 @@ NebulaTemplate::NebulaTemplate(string& etc_location, string& var_location)
     attribute = new SingleAttribute("HOST_MONITORING_INTERVAL",value);
     conf_default.insert(make_pair(attribute->name(),attribute));
 
+    // HOST_PER_INTERVAL
+    value = "15";
+
+    attribute = new SingleAttribute("HOST_PER_INTERVAL",value);
+    conf_default.insert(make_pair(attribute->name(),attribute));
+
     // POLL_INTERVAL
     value = "600";
 
     attribute = new SingleAttribute("VM_POLLING_INTERVAL",value);
     conf_default.insert(make_pair(attribute->name(),attribute));
 
+    // VM_PER_INTERVAL
+    value = "5";
+
+    attribute = new SingleAttribute("VM_PER_INTERVAL",value);
+    conf_default.insert(make_pair(attribute->name(),attribute));
+
     //VM_DIR
     attribute = new SingleAttribute("VM_DIR",var_location);
     conf_default.insert(make_pair(attribute->name(),attribute));
@@ -126,17 +182,10 @@ NebulaTemplate::NebulaTemplate(string& etc_location, string& var_location)
 #*******************************************************************************
 # Image Repository Configuration
 #*******************************************************************************
-#  IMAGE_REPOSITORY_PATH
 #  DEFAULT_IMAGE_TYPE
 #  DEFAULT_DEVICE_PREFIX
 #*******************************************************************************
 */
-    //IMAGE_REPOSITORY_PATH
-    value = var_location + "/images";
-
-    attribute = new SingleAttribute("IMAGE_REPOSITORY_PATH",value);
-    conf_default.insert(make_pair(attribute->name(),attribute));
-
     //DEFAULT_IMAGE_TYPE
     value = "OS";
 
@@ -148,51 +197,25 @@ NebulaTemplate::NebulaTemplate(string& etc_location, string& var_location)
 
     attribute = new SingleAttribute("DEFAULT_DEVICE_PREFIX",value);
     conf_default.insert(make_pair(attribute->name(),attribute));
+/*
+
+#*******************************************************************************
+# Auth Manager Configuration
+#*******************************************************************************
+# SESSION_EXPIRATION_TIME
+# ENABLE_OTHER_PERMISSIONS
+#*******************************************************************************
+*/
+    // SESSION_EXPIRATION_TIME
+    value = "0";
+
+    attribute = new SingleAttribute("SESSION_EXPIRATION_TIME",value);
+    conf_default.insert(make_pair(attribute->name(),attribute));
+
+    // ENABLE_OTHER_PERMISSIONS
+    value = "YES";
+
+    attribute = new SingleAttribute("ENABLE_OTHER_PERMISSIONS",value);
+    conf_default.insert(make_pair(attribute->name(),attribute));
 }
 
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-
-int NebulaTemplate::load_configuration()
-{
-    char *                              error = 0;
-    map<string, Attribute *>::iterator  iter, j;
-    int                                 rc;
-
-    string      aname;
-    Attribute * attr;
-
-    rc = parse(conf_file.c_str(), &error);
-
-    if ( rc != 0 && error != 0)
-    {
-
-        cout << "\nError while parsing configuration file:\n" << error << endl;
-
-        free(error);
-
-        return -1;
-    }
-
-    for(iter=conf_default.begin();iter!=conf_default.end();)
-    {
-        aname = iter->first;
-        attr  = iter->second;
-
-        j = attributes.find(aname);
-
-        if ( j == attributes.end() )
-        {
-            attributes.insert(make_pair(aname,attr));
-            iter++;
-        }
-        else
-        {
-            delete iter->second;
-            conf_default.erase(iter++);
-        }
-    }
-
-    return 0;
-}
