@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@ public class Host extends PoolElement{
     private static final String INFO            = METHOD_PREFIX + "info";
     private static final String DELETE          = METHOD_PREFIX + "delete";
     private static final String ENABLE          = METHOD_PREFIX + "enable";
+    private static final String UPDATE          = METHOD_PREFIX + "update";
     
     private static final String[] HOST_STATES = 
         {"INIT", "MONITORING", "MONITORED", "ERROR", "DISABLED"};
@@ -71,6 +72,9 @@ public class Host extends PoolElement{
      * @param vmm The name of the virtual machine manager mad name
      * (vmm_mad_name), this values are taken from the oned.conf with the
      * tag name VM_MAD (name)
+     * @param vnm The name of the virtual network manager mad name
+     * (vnm_mad_name), this values are taken from the oned.conf with the
+     * tag name VN_MAD (name)
      * @param tm The transfer manager mad name to be used with this host 
      * @return If successful the message contains the associated
      * id generated for this host
@@ -79,9 +83,10 @@ public class Host extends PoolElement{
                                        String hostname,
                                        String im,
                                        String vmm,
+                                       String vnm,
                                        String tm)
     {
-        return client.call(ALLOCATE, hostname, im, vmm, tm);
+        return client.call(ALLOCATE, hostname, im, vmm, vnm, tm);
     }
 
     /**
@@ -122,7 +127,19 @@ public class Host extends PoolElement{
     {
         return client.call(ENABLE, id, enable);
     }
-    
+
+    /**
+     * Replaces the template contents.
+     * 
+     * @param client XML-RPC Client.
+     * @param id The image id of the target host we want to modify.
+     * @param new_template New template contents
+     * @return If successful the message contains the host id.
+     */
+    public static OneResponse update(Client client, int id, String new_template)
+    {
+        return client.call(UPDATE, id, new_template);
+    }
 
     // =================================
     // Instanced object XML-RPC methods
@@ -181,6 +198,17 @@ public class Host extends PoolElement{
         return enable(false);
     }
 
+    /**
+     * Replaces the template contents.
+     * 
+     * @param new_template New template contents
+     * @return If successful the message contains the host id.
+     */
+    public OneResponse update(String new_template)
+    {
+        return update(client, id, new_template);
+    }
+
     // =================================
     // Helpers
     // =================================
@@ -227,14 +255,4 @@ public class Host extends PoolElement{
     {
         return state() != 4;
     }
-
-    /**
-     * Returns the name of the cluster this host is assigned to.
-     * 
-     * @return The name of the cluster this host is assigned to.
-     */
-    public String getCluster()
-    {
-        return xpath("CLUSTER");
-    }
 }
