@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,9 +32,10 @@ public class Image extends PoolElement
     private static final String INFO     = METHOD_PREFIX + "info";
     private static final String DELETE   = METHOD_PREFIX + "delete";
     private static final String UPDATE   = METHOD_PREFIX + "update";
-    private static final String RMATTR   = METHOD_PREFIX + "rmattr";
     private static final String ENABLE   = METHOD_PREFIX + "enable";
-    private static final String PUBLISH  = METHOD_PREFIX + "publish";
+    private static final String CHOWN    = METHOD_PREFIX + "chown";
+    private static final String CHMOD    = METHOD_PREFIX + "chmod";
+    private static final String CHTYPE   = METHOD_PREFIX + "chtype";
 
     private static final String[] IMAGE_STATES =
         {"INIT", "READY", "USED", "DISABLED"};
@@ -110,31 +111,16 @@ public class Image extends PoolElement
     }
 
     /**
-     * Modifies an image attribute.
+     * Replaces the template contents.
      * 
      * @param client XML-RPC Client.
      * @param id The image id of the target image we want to modify.
-     * @param att_name The name of the attribute to update.
-     * @param att_val The new value for the attribute.
+     * @param new_template New template contents
      * @return If successful the message contains the image id.
      */
-    public static OneResponse update(Client client, int id,
-                                     String att_name, String att_val)
+    public static OneResponse update(Client client, int id, String new_template)
     {
-        return client.call(UPDATE, id, att_name, att_val);
-    }
-
-    /**
-     * Removes an image attribute.
-     * 
-     * @param client XML-RPC Client.
-     * @param id The image id of the target image we want to modify.
-     * @param att_name The name of the attribute to remove.
-     * @return If successful the message contains the image id.
-     */
-    public static OneResponse rmattr(Client client, int id, String att_name)
-    {
-        return client.call(RMATTR, id, att_name);
+        return client.call(UPDATE, id, new_template);
     }
 
     /**
@@ -160,9 +146,90 @@ public class Image extends PoolElement
      */
     public static OneResponse publish(Client client, int id, boolean publish)
     {
-        return client.call(PUBLISH, id, publish);
+        int group_u = publish ? 1 : 0;
+
+        return chmod(client, id, -1, -1, -1, group_u, -1, -1, -1, -1, -1);
     }
 
+    /**
+     * Changes the owner/group
+     * 
+     * @param client XML-RPC Client.
+     * @param id The image id of the target image we want to modify.
+     * @param uid The new owner user ID. Set it to -1 to leave the current one.
+     * @param gid The new group ID. Set it to -1 to leave the current one.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chown(Client client, int id, int uid, int gid)
+    {
+        return client.call(CHOWN, id, uid, gid);
+    }
+
+    /**
+     * Changes the Image permissions
+     * 
+     * @param client XML-RPC Client.
+     * @param id The image id of the target image we want to modify.
+     * @param owner_u 1 to allow, 0 deny, -1 do not change
+     * @param owner_m 1 to allow, 0 deny, -1 do not change
+     * @param owner_a 1 to allow, 0 deny, -1 do not change
+     * @param group_u 1 to allow, 0 deny, -1 do not change
+     * @param group_m 1 to allow, 0 deny, -1 do not change
+     * @param group_a 1 to allow, 0 deny, -1 do not change
+     * @param other_u 1 to allow, 0 deny, -1 do not change
+     * @param other_m 1 to allow, 0 deny, -1 do not change
+     * @param other_a 1 to allow, 0 deny, -1 do not change
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chmod(Client client, int id,
+                                    int owner_u, int owner_m, int owner_a,
+                                    int group_u, int group_m, int group_a,
+                                    int other_u, int other_m, int other_a)
+    {
+        return chmod(client, CHMOD, id,
+                owner_u, owner_m, owner_a,
+                group_u, group_m, group_a,
+                other_u, other_m, other_a);
+    }
+
+    /**
+     * Changes the permissions
+     * 
+     * @param client XML-RPC Client.
+     * @param id The id of the target object.
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chmod(Client client, int id, String octet)
+    {
+        return chmod(client, CHMOD, id, octet);
+    }
+
+    /**
+     * Changes the permissions
+     * 
+     * @param client XML-RPC Client.
+     * @param id The id of the target object.
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chmod(Client client, int id, int octet)
+    {
+        return chmod(client, CHMOD, id, octet);
+    }
+
+    /**
+     * Changes the Image type
+     * 
+     * @param client XML-RPC Client.
+     * @param id The image id of the target image we want to modify.
+     * @param type The new Image type
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chtype(Client client, int id, String type)
+    {
+        return client.call(CHTYPE, id, type);
+    }
 
     // =================================
     // Instanced object XML-RPC methods
@@ -192,26 +259,14 @@ public class Image extends PoolElement
     }
 
     /**
-     * Modifies an image attribute.
+     * Replaces the template contents.
      * 
-     * @param att_name The name of the attribute to update.
-     * @param att_val The new value for the attribute.
+     * @param new_template New template contents
      * @return If successful the message contains the image id.
      */
-    public OneResponse update(String att_name, String att_val)
+    public OneResponse update(String new_template)
     {
-        return update(client, id, att_name, att_val); 
-    }
-
-    /**
-     * Removes an image attribute.
-     * 
-     * @param att_name The name of the attribute to remove.
-     * @return If successful the message contains the image id.
-     */
-    public OneResponse rmattr(String att_name)
-    {
-        return rmattr(client, id, att_name);
+        return update(client, id, new_template);
     }
 
     /**
@@ -276,6 +331,97 @@ public class Image extends PoolElement
         return publish(false);
     }
 
+    /**
+     * Changes the owner/group
+     * 
+     * @param uid The new owner user ID. Set it to -1 to leave the current one.
+     * @param gid The new group ID. Set it to -1 to leave the current one.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chown(int uid, int gid)
+    {
+        return chown(client, id, uid, gid);
+    }
+
+    /**
+     * Changes the owner
+     * 
+     * @param uid The new owner user ID.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chown(int uid)
+    {
+        return chown(uid, -1);
+    }
+
+    /**
+     * Changes the group
+     * 
+     * @param gid The new group ID.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chgrp(int gid)
+    {
+        return chown(-1, gid);
+    }
+
+    /**
+     * Changes the Image permissions
+     * 
+     * @param owner_u 1 to allow, 0 deny, -1 do not change
+     * @param owner_m 1 to allow, 0 deny, -1 do not change
+     * @param owner_a 1 to allow, 0 deny, -1 do not change
+     * @param group_u 1 to allow, 0 deny, -1 do not change
+     * @param group_m 1 to allow, 0 deny, -1 do not change
+     * @param group_a 1 to allow, 0 deny, -1 do not change
+     * @param other_u 1 to allow, 0 deny, -1 do not change
+     * @param other_m 1 to allow, 0 deny, -1 do not change
+     * @param other_a 1 to allow, 0 deny, -1 do not change
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chmod(int owner_u, int owner_m, int owner_a,
+                             int group_u, int group_m, int group_a,
+                             int other_u, int other_m, int other_a)
+    {
+        return chmod(client, id,
+                    owner_u, owner_m, owner_a,
+                    group_u, group_m, group_a,
+                    other_u, other_m, other_a);
+    }
+
+    /**
+     * Changes the permissions
+     *
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chmod(String octet)
+    {
+        return chmod(client, id, octet);
+    }
+
+    /**
+     * Changes the permissions
+     *
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chmod(int octet)
+    {
+        return chmod(client, id, octet);
+    }
+
+    /**
+     * Changes the Image type
+     * 
+     * @param type The new Image type
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chtype(String type)
+    {
+        return chtype(client, id, type);
+    }
+
     // =================================
     // Helpers
     // =================================
@@ -348,15 +494,4 @@ public class Image extends PoolElement
     {
         return state() != 3;
     }
-
-    /**
-     * Returns true if the image is public.
-     * 
-     * @return True if the image is public.
-     */
-    public boolean isPublic()
-    {
-        String isPub = xpath("PUBLIC"); 
-        return isPub != null && isPub.equals("1");
-    }
 }
