@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ package org.opennebula.client.vm;
 import org.opennebula.client.Client;
 import org.opennebula.client.OneResponse;
 import org.opennebula.client.PoolElement;
+import org.opennebula.client.template.Template;
 import org.w3c.dom.Node;
 
 /**
@@ -34,6 +35,8 @@ public class VirtualMachine extends PoolElement{
     private static final String ACTION   = METHOD_PREFIX + "action";
     private static final String MIGRATE  = METHOD_PREFIX + "migrate";
     private static final String SAVEDISK = METHOD_PREFIX + "savedisk";
+    private static final String CHOWN    = METHOD_PREFIX + "chown";
+    private static final String CHMOD    = METHOD_PREFIX + "chmod";
 
     private static final String[] VM_STATES =
     {
@@ -134,6 +137,56 @@ public class VirtualMachine extends PoolElement{
         return client.call(ALLOCATE, description);
     }
 
+    /**
+     * Allocates a new VM in OpenNebula from a registered Template.
+     * 
+     * @param client XML-RPC Client.
+     * @param templateId The source Template's ID
+     * @param newName Name for the new VM, replaces the Template's one.
+     * Can be null.
+     * @return If successful the message contains the associated
+     * id generated for this VM.
+     */
+    public static OneResponse allocateFromTemplate(Client client,
+            int templateId, String newName)
+    {
+        String template = "TEMPLATE_ID = " + templateId;
+        if( newName != null )
+        {
+            template += "\nNAME = " + newName;
+        }
+
+        return allocate(client, template);
+    }
+
+    /**
+     * Allocates a new VM in OpenNebula from a registered Template.
+     * 
+     * @param client XML-RPC Client.
+     * @param templateId The source Template's ID
+     * @return If successful the message contains the associated
+     * id generated for this VM.
+     */
+    public static OneResponse allocateFromTemplate(Client client,
+            int templateId)
+    {
+        return allocateFromTemplate(client, templateId, null);
+    }
+
+    /**
+     * Allocates a new VM in OpenNebula from a registered Template.
+     * 
+     * @param client XML-RPC Client.
+     * @param template The source Template.
+     * @return If successful the message contains the associated
+     * id generated for this VM.
+     */
+    public static OneResponse allocateFromTemplate(Client client,
+            Template template)
+    {
+        return allocateFromTemplate(client, template.id());
+    }
+
     /**
      * Retrieves the information of the given VM.
      *
@@ -147,6 +200,72 @@ public class VirtualMachine extends PoolElement{
         return client.call(INFO, id);
     }
 
+    /**
+     * Changes the owner/group
+     * 
+     * @param client XML-RPC Client.
+     * @param id The virtual machine id (vid) of the target instance.
+     * @param uid The new owner user ID. Set it to -1 to leave the current one.
+     * @param gid The new group ID. Set it to -1 to leave the current one.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chown(Client client, int id, int uid, int gid)
+    {
+        return client.call(CHOWN, id, uid, gid);
+    }
+
+    /**
+     * Changes the VM permissions
+     * 
+     * @param client XML-RPC Client.
+     * @param id The VM id of the target VM.
+     * @param owner_u 1 to allow, 0 deny, -1 do not change
+     * @param owner_m 1 to allow, 0 deny, -1 do not change
+     * @param owner_a 1 to allow, 0 deny, -1 do not change
+     * @param group_u 1 to allow, 0 deny, -1 do not change
+     * @param group_m 1 to allow, 0 deny, -1 do not change
+     * @param group_a 1 to allow, 0 deny, -1 do not change
+     * @param other_u 1 to allow, 0 deny, -1 do not change
+     * @param other_m 1 to allow, 0 deny, -1 do not change
+     * @param other_a 1 to allow, 0 deny, -1 do not change
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chmod(Client client, int id,
+                                    int owner_u, int owner_m, int owner_a,
+                                    int group_u, int group_m, int group_a,
+                                    int other_u, int other_m, int other_a)
+    {
+        return chmod(client, CHMOD, id,
+                owner_u, owner_m, owner_a,
+                group_u, group_m, group_a,
+                other_u, other_m, other_a);
+    }
+
+    /**
+     * Changes the permissions
+     * 
+     * @param client XML-RPC Client.
+     * @param id The id of the target object.
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chmod(Client client, int id, String octet)
+    {
+        return chmod(client, CHMOD, id, octet);
+    }
+
+    /**
+     * Changes the permissions
+     * 
+     * @param client XML-RPC Client.
+     * @param id The id of the target object.
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse chmod(Client client, int id, int octet)
+    {
+        return chmod(client, CHMOD, id, octet);
+    }
 
     // =================================
     // Instanced object XML-RPC methods
@@ -183,6 +302,7 @@ public class VirtualMachine extends PoolElement{
      * It is recommended to use the helper methods instead:
      * <ul>
      * <li>{@link VirtualMachine#shutdown()}</li>
+     * <li>{@link VirtualMachine#reboot()}</li>
      * <li>{@link VirtualMachine#cancel()}</li>
      * <li>{@link VirtualMachine#hold()}</li>
      * <li>{@link VirtualMachine#release()}</li>
@@ -194,7 +314,7 @@ public class VirtualMachine extends PoolElement{
      * </ul>
      *
      * @param action The action name to be performed, can be:<br/>
-     * "shutdown", "hold", "release", "stop", "cancel", "suspend",
+     * "shutdown", "reboot", "hold", "release", "stop", "cancel", "suspend",
      * "resume", "restart", "finalize".
      * @return If an error occurs the error message contains the reason.
      */
@@ -222,12 +342,108 @@ public class VirtualMachine extends PoolElement{
      * VirtualMachine shutdowns.
      *
      * @param diskId ID of the disk to be saved.
-     * @param imageId ID of the image where the disk will be saved.
+     * @param imageName Name of the new Image that will be created.
      * @return If an error occurs the error message contains the reason.
      */
-    public OneResponse savedisk(int diskId, int imageId)
+    public OneResponse savedisk(int diskId, String imageName)
     {
-        return client.call(SAVEDISK, id ,diskId, imageId);
+        return savedisk(diskId, imageName, "");
+    }
+
+    /**
+     * Sets the specified vm's disk to be saved in a new image when the
+     * VirtualMachine shutdowns.
+     *
+     * @param diskId ID of the disk to be saved.
+     * @param imageName Name of the new Image that will be created.
+     * @param imageType Type of the new image. Set to empty string to use
+     * the default type
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse savedisk(int diskId, String imageName, String imageType)
+    {
+        return client.call(SAVEDISK, id ,diskId, imageName, imageType);
+    }
+
+    /**
+     * Changes the owner/group
+     * 
+     * @param uid The new owner user ID. Set it to -1 to leave the current one.
+     * @param gid The new group ID. Set it to -1 to leave the current one.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chown(int uid, int gid)
+    {
+        return chown(client, id, uid, gid);
+    }
+
+    /**
+     * Changes the owner
+     * 
+     * @param uid The new owner user ID.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chown(int uid)
+    {
+        return chown(uid, -1);
+    }
+
+    /**
+     * Changes the group
+     * 
+     * @param gid The new group ID.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chgrp(int gid)
+    {
+        return chown(-1, gid);
+    }
+
+
+    /**
+     * Changes the VM permissions
+     * 
+     * @param owner_u 1 to allow, 0 deny, -1 do not change
+     * @param owner_m 1 to allow, 0 deny, -1 do not change
+     * @param owner_a 1 to allow, 0 deny, -1 do not change
+     * @param group_u 1 to allow, 0 deny, -1 do not change
+     * @param group_m 1 to allow, 0 deny, -1 do not change
+     * @param group_a 1 to allow, 0 deny, -1 do not change
+     * @param other_u 1 to allow, 0 deny, -1 do not change
+     * @param other_m 1 to allow, 0 deny, -1 do not change
+     * @param other_a 1 to allow, 0 deny, -1 do not change
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chmod(int owner_u, int owner_m, int owner_a,
+                             int group_u, int group_m, int group_a,
+                             int other_u, int other_m, int other_a)
+    {
+        return chmod(client, id,
+                    owner_u, owner_m, owner_a,
+                    group_u, group_m, group_a,
+                    other_u, other_m, other_a);
+    }
+
+    /**
+     * Changes the permissions
+     *
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chmod(String octet)
+    {
+        return chmod(client, id, octet);
+    }
+
+    /**
+     * Changes the permissions
+     *
+     * @param octet Permissions octed , e.g. 640
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse chmod(int octet)
+    {
+        return chmod(client, id, octet);
     }
 
     // =================================
@@ -243,6 +459,15 @@ public class VirtualMachine extends PoolElement{
         return action("shutdown");
     }
 
+    /**
+     * Reboots a running VM.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse reboot()
+    {
+        return action("reboot");
+    }
+
     /**
      * Cancels the running VM.
      * @return If an error occurs the error message contains the reason.
