@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -63,7 +63,8 @@ public class HostTest
     @Before
     public void setUp() throws Exception
     {
-        res = Host.allocate(client, name, "im_dummy", "vmm_dummy", "tm_dummy");
+        res = Host.allocate(client, name, "im_dummy", "vmm_dummy", "vnm_dummy",
+            "tm_dummy");
 
         int hid = !res.isError() ? Integer.parseInt(res.getMessage()) : -1;
         host    = new Host(hid, client);
@@ -83,7 +84,8 @@ public class HostTest
     {
         String name = "allocate_test";
 
-        res = Host.allocate(client, name, "im_dummy", "vmm_dummy", "tm_dummy");
+        res = Host.allocate(client, name, "im_dummy", "vmm_dummy", "vmm_dummy",
+            "tm_dummy");
         assertTrue( !res.isError() );
 //        assertTrue( res.getMessage().equals("0") );
 
@@ -99,7 +101,7 @@ public class HostTest
     }
 
     @Test
-    public void update()
+    public void info()
     {
         res = host.info();
         assertTrue( !res.isError() );
@@ -152,6 +154,30 @@ public class HostTest
 
         assertTrue( !found );
     }
+
+    @Test
+    public void update()
+    {
+        res = host.info();
+        assertTrue( !res.isError() );
+
+        assertTrue( host.xpath("TEMPLATE/ATT1").equals( "" ) );
+        assertTrue( host.xpath("TEMPLATE/ATT2").equals( "" ) );
+
+        String new_template =  "ATT2 = NEW_VAL\n" +
+                        "ATT3 = VAL3";
+
+        res = host.update(new_template);
+        assertTrue( !res.isError() );
+
+
+        res = host.info();
+        assertTrue( !res.isError() );
+        assertTrue( host.xpath("TEMPLATE/ATT1").equals( "" ) );
+        assertTrue( host.xpath("TEMPLATE/ATT2").equals( "NEW_VAL" ) );
+        assertTrue( host.xpath("TEMPLATE/ATT3").equals( "VAL3" ) );
+    }
+
 /*
     @Test
     public void attributes()
