@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,7 +23,6 @@ import org.junit.Test;
 import org.opennebula.client.Client;
 import org.opennebula.client.OneResponse;
 import org.opennebula.client.host.Host;
-import org.opennebula.client.image.Image;
 import org.opennebula.client.vm.VirtualMachine;
 import org.opennebula.client.vm.VirtualMachinePool;
 
@@ -81,11 +80,11 @@ public class VirtualMachineTest
 
 
         res = Host.allocate(client, "host_A",
-                            "im_dummy", "vmm_dummy", "tm_dummy");
+                            "im_dummy", "vmm_dummy", "vmm_dummy", "tm_dummy");
         hid_A = Integer.parseInt( res.getMessage() );
 
         res = Host.allocate(client, "host_B",
-                            "im_dummy", "vmm_dummy", "tm_dummy");
+                            "im_dummy", "vmm_dummy", "vmm_dummy", "tm_dummy");
         hid_B = Integer.parseInt( res.getMessage() );
     }
 
@@ -128,7 +127,8 @@ public class VirtualMachineTest
     @Test
     public void allocate()
     {
-        vmPool.info();
+        res = vmPool.info();
+        assertTrue( !res.isError() );
 
         boolean found = false;
         for(VirtualMachine vm : vmPool)
@@ -309,21 +309,14 @@ public class VirtualMachineTest
 
         vm = new VirtualMachine(vmid, client);
 
-        String imgTemplate =
-            "NAME = \"image\"\n" +
-            "ATT1 = \"val1\"";
-
-        res = Image.allocate(client, imgTemplate);
-
-        int imgid = res.isError() ? -1 : Integer.parseInt(res.getMessage());
-
-        res = vm.savedisk(0, imgid);
+        res = vm.savedisk(0, "New_image");
         assertTrue( !res.isError() );
+        assertTrue( res.getMessage().equals("0") );
 
         res = vm.info();
         assertTrue( !res.isError() );
 
-        assertTrue( vm.xpath("TEMPLATE/DISK/SAVE_AS").equals(Integer.toString(imgid)) );
+        assertTrue( vm.xpath("TEMPLATE/DISK/SAVE_AS").equals(("0")) );
 
     }
 }
