@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -117,7 +117,7 @@ public class VirtualNetworkTest
     }
 
     @Test
-    public void update()
+    public void info()
     {
         res = vnet.info();
         assertTrue( !res.isError() );
@@ -136,8 +136,6 @@ public class VirtualNetworkTest
 //        assertTrue( vnet.xpath("ID").equals("0") );
         assertTrue( vnet.xpath("NAME").equals(name) );
         assertTrue( vnet.xpath("BRIDGE").equals("vbr0") );
-        assertTrue( vnet.xpath("TEMPLATE/NETWORK_ADDRESS").equals("192.168.0.0") );
-        assertTrue( vnet.xpath("TEMPLATE/TYPE").equals("RANGED") );
     }
 
     @Test
@@ -149,7 +147,8 @@ public class VirtualNetworkTest
         res = vnet.info();
         assertTrue( res.isError() );
     }
-
+// TODO
+/*
     @Test
     public void publish()
     {
@@ -171,7 +170,7 @@ public class VirtualNetworkTest
         res = vnet.info();
         assertTrue( !vnet.isPublic() );
     }
-
+*/
     @Test
     public void addLeases()
     {
@@ -210,4 +209,68 @@ public class VirtualNetworkTest
 
         fixed_vnet.delete();
     }
+
+    @Test
+    public void holdFixed()
+    {
+        res = VirtualNetwork.allocate(client, fixed_template);
+        assertTrue( !res.isError() );
+
+        VirtualNetwork fixed_vnet =
+            new VirtualNetwork(Integer.parseInt(res.getMessage()), client);
+
+        res = fixed_vnet.hold("130.10.0.1");
+        assertTrue( !res.isError() );
+
+        res = fixed_vnet.hold("130.10.0.5");
+        assertTrue( res.isError() );
+
+        res = fixed_vnet.release("130.10.0.1");
+        assertTrue( !res.isError() );
+
+        res = fixed_vnet.release("130.10.0.1");
+        assertTrue( res.isError() );
+
+        res = fixed_vnet.release("130.10.0.5");
+        assertTrue( res.isError() );
+
+        fixed_vnet.delete();
+    }
+
+    @Test
+    public void holdRanged()
+    {
+        res = vnet.hold("192.168.0.10");
+        assertTrue( !res.isError() );
+
+        res = vnet.hold("192.168.100.1");
+        assertTrue( res.isError() );
+
+        res = vnet.release("192.168.0.10");
+        assertTrue( !res.isError() );
+
+        res = vnet.release("192.168.0.10");
+        assertTrue( res.isError() );
+
+        res = vnet.release("192.168.100.1");
+        assertTrue( res.isError() );
+
+        vnet.delete();
+    }
+
+    @Test
+    public void update()
+    {
+        String new_template =  "ATT2 = NEW_VAL\n" +
+                "ATT3 = VAL3";
+
+        res = vnet.update(new_template);
+        assertTrue( !res.isError() );
+
+        res = vnet.info();
+        assertTrue( !res.isError() );
+        assertTrue( vnet.xpath("TEMPLATE/ATT1").equals( "" ) );
+        assertTrue( vnet.xpath("TEMPLATE/ATT2").equals( "NEW_VAL" ) );
+        assertTrue( vnet.xpath("TEMPLATE/ATT3").equals( "VAL3" ) );
+    }
 }
