@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,6 +14,7 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+
 begin # require 'rubygems'
     require 'rubygems'
 rescue Exception
@@ -31,28 +32,40 @@ require 'OpenNebula/VirtualNetwork'
 require 'OpenNebula/VirtualNetworkPool'
 require 'OpenNebula/Image'
 require 'OpenNebula/ImagePool'
-require 'OpenNebula/ImageRepository'
 require 'OpenNebula/User'
 require 'OpenNebula/UserPool'
 require 'OpenNebula/Host'
 require 'OpenNebula/HostPool'
-require 'OpenNebula/Cluster'
-require 'OpenNebula/ClusterPool'
+require 'OpenNebula/Template'
+require 'OpenNebula/TemplatePool'
+require 'OpenNebula/Group'
+require 'OpenNebula/GroupPool'
+require 'OpenNebula/Acl'
+require 'OpenNebula/AclPool'
 
 module OpenNebula
 
-    # -------------------------------------------------------------------------
     # The Error Class represents a generic error in the OpenNebula
     # library. It contains a readable representation of the error.
     # Any function in the OpenNebula module will return an Error
     # object in case of error.
-    # -------------------------------------------------------------------------
     class Error
-        attr_reader :message
+        ESUCCESS        = 0x0000
+        EAUTHENTICATION = 0x0100
+        EAUTHORIZATION  = 0x0200
+        ENO_EXISTS      = 0x0400
+        EACTION         = 0x0800
+        EXML_RPC_API    = 0x1000
+        EINTERNAL       = 0x2000
+        ENOTDEFINED     = 0x1111
 
-        # +message+ a description of the error
-        def initialize(message=nil)
-            @message=message
+        attr_reader :message, :errno
+
+        # +message+ Description of the error
+        # +errno+   OpenNebula code error
+        def initialize(message=nil, errno=0x1111)
+            @message = message
+            @errno   = errno
         end
 
         def to_str()
@@ -60,18 +73,14 @@ module OpenNebula
         end
     end
 
-    # -------------------------------------------------------------------------
     # Returns true if the object returned by a method of the OpenNebula
     # library is an Error
-    # -------------------------------------------------------------------------
     def self.is_error?(value)
         value.class==OpenNebula::Error
     end
 
-    # -------------------------------------------------------------------------
     # The client class, represents the connection with the core and handles the
     # xml-rpc calls.
-    # -------------------------------------------------------------------------
     class Client
         attr_accessor :one_auth
 
@@ -84,39 +93,26 @@ module OpenNebula
 
         def initialize(secret=nil, endpoint=nil)
             if secret
-                one_secret = secret
+                @one_auth = secret
             elsif ENV["ONE_AUTH"] and !ENV["ONE_AUTH"].empty? and File.file?(ENV["ONE_AUTH"])
-                one_secret=File.read(ENV["ONE_AUTH"])
+                @one_auth = File.read(ENV["ONE_AUTH"])
             elsif File.file?(ENV["HOME"]+"/.one/one_auth")
-                one_secret=File.read(ENV["HOME"]+"/.one/one_auth")
+                @one_auth = File.read(ENV["HOME"]+"/.one/one_auth")
             else
                 raise "ONE_AUTH file not present"
             end
 
-            if !one_secret.match(".+:.+")
-                raise "Authorization file malformed"
-            end
-
-
-            one_secret=~/^(.+?):(.+)$/
-            user=$1
-            password=$2
-
-            if password.match(/^plain:/)
-                @one_auth = "#{user}:#{password.split(':').last}"
-            else
-                @one_auth = "#{user}:#{Digest::SHA1.hexdigest(password)}"
-            end
+            @one_auth.rstrip!
 
             if endpoint
-                @one_endpoint=endpoint
+                @one_endpoint = endpoint
             elsif ENV["ONE_XMLRPC"]
-                @one_endpoint=ENV["ONE_XMLRPC"]
+                @one_endpoint = ENV["ONE_XMLRPC"]
             else
-                @one_endpoint="http://localhost:2633/RPC2"
+                @one_endpoint = "http://localhost:2633/RPC2"
             end
 
-            @server=XMLRPC::Client.new2(@one_endpoint)
+            @server = XMLRPC::Client.new2(@one_endpoint)
         end
 
         def call(action, *args)
@@ -129,7 +125,7 @@ module OpenNebula
                 response = @server.call_async("one."+action, @one_auth, *args)
 
                 if response[0] == false
-                    Error.new(response[1])
+                    Error.new(response[1], response[2])
                 else
                     response[1] #response[1..-1]
                 end
