@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,32 +14,38 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+
 require 'OpenNebula/Pool'
 require 'fileutils'
 
 module OpenNebula
     class Image < PoolElement
-        # ---------------------------------------------------------------------
+        #######################################################################
         # Constants and Class Methods
-        # ---------------------------------------------------------------------
+        #######################################################################
+
+
         IMAGE_METHODS = {
             :info        => "image.info",
             :allocate    => "image.allocate",
             :update      => "image.update",
-            :rmattr      => "image.rmattr",
             :enable      => "image.enable",
-            :publish     => "image.publish",
             :persistent  => "image.persistent",
-            :delete      => "image.delete"
+            :delete      => "image.delete",
+            :chown       => "image.chown",
+            :chmod       => "image.chmod",
+            :chtype      => "image.chtype"
         }
 
-        IMAGE_STATES=%w{INIT READY USED DISABLED}
+        IMAGE_STATES=%w{INIT READY USED DISABLED LOCKED ERROR}
 
         SHORT_IMAGE_STATES={
             "INIT"      => "init",
             "READY"     => "rdy",
             "USED"      => "used",
-            "DISABLED"  => "disa"
+            "DISABLED"  => "disa",
+            "LOCKED"    => "lock",
+            "ERROR"     => "err"
         }
 
         IMAGE_TYPES=%w{OS CDROM DATABLOCK}
@@ -90,20 +96,11 @@ module OpenNebula
             super(IMAGE_METHODS[:allocate],description)
         end
 
-        # Modifies an image attribute
+        # Replaces the template contents
         #
-        # +name+ Name of the attribute to be changed
-        #
-        # +value+ New value for the attribute
-        def update(name, value)
-            super(IMAGE_METHODS[:update], name, value)
-        end
-
-        # Deletes an Image attribute
-        #
-        # +name+ Name of the attribute to be deleted
-        def remove_attr(name)
-            do_rm_attr(name)
+        # +new_template+ New template contents
+        def update(new_template)
+            super(IMAGE_METHODS[:update], new_template)
         end
 
         # Enables an Image
@@ -125,7 +122,7 @@ module OpenNebula
         def unpublish
             set_publish(false)
         end
-        
+
         # Makes the Image persistent
         def persistent
             set_persistent(true)
@@ -140,7 +137,47 @@ module OpenNebula
         def delete()
             super(IMAGE_METHODS[:delete])
         end
-    
+
+        # Changes the owner/group
+        # uid:: _Integer_ the new owner id. Set to -1 to leave the current one
+        # gid:: _Integer_ the new group id. Set to -1 to leave the current one
+        # [return] nil in case of success or an Error object
+        def chown(uid, gid)
+            super(IMAGE_METHODS[:chown], uid, gid)
+        end
+
+        # Changes the Image permissions.
+        #
+        # @param octet [String] Permissions octed , e.g. 640
+        # @return [nil, OpenNebula::Error] nil in case of success, Error
+        #   otherwise
+        def chmod_octet(octet)
+            super(IMAGE_METHODS[:chmod], octet)
+        end
+
+        # Changes the Image permissions.
+        # Each [Integer] argument must be 1 to allow, 0 deny, -1 do not change
+        #
+        # @return [nil, OpenNebula::Error] nil in case of success, Error
+        #   otherwise
+        def chmod(owner_u, owner_m, owner_a, group_u, group_m, group_a, other_u,
+                other_m, other_a)
+            super(IMAGE_METHODS[:chmod], owner_u, owner_m, owner_a, group_u,
+                group_m, group_a, other_u, other_m, other_a)
+        end
+
+        # Changes the Image type
+        # @param type [String] new Image type
+        # @return [nil, OpenNebula::Error] nil in case of success, Error
+        #   otherwise
+        def chtype(type)
+            return Error.new('ID not defined') if !@pe_id
+
+            rc = @client.call(IMAGE_METHODS[:chtype], @pe_id, type)
+            rc = nil if !OpenNebula.is_error?(rc)
+
+            return rc
+        end
 
         #######################################################################
         # Helpers to get Image information
@@ -176,6 +213,20 @@ module OpenNebula
             SHORT_IMAGE_TYPES[type_str]
         end
 
+        # Returns the group identifier
+        # [return] _Integer_ the element's group ID
+        def gid
+            self['GID'].to_i
+        end
+
+        def public?
+            if self['PERMISSIONS/GROUP_U'] == "1" || self['PERMISSIONS/OTHER_U'] == "1"
+                true
+            else
+                false
+            end
+        end
+
     private
 
         def set_enabled(enabled)
@@ -188,12 +239,9 @@ module OpenNebula
         end
 
         def set_publish(published)
-            return Error.new('ID not defined') if !@pe_id
+            group_u = published ? 1 : 0
 
-            rc = @client.call(IMAGE_METHODS[:publish], @pe_id, published)
-            rc = nil if !OpenNebula.is_error?(rc)
-
-            return rc
+            chmod(-1, -1, -1, group_u, -1, -1, -1, -1, -1)
         end
         
         def set_persistent(persistence)
@@ -204,15 +252,5 @@ module OpenNebula
 
             return rc
         end
-
-        def do_rm_attr(name)
-            return Error.new('ID not defined') if !@pe_id
-
-            rc = @client.call(IMAGE_METHODS[:rmattr], @pe_id, name)
-            rc = nil if !OpenNebula.is_error?(rc)
-
-            return rc
-        end
-
     end
 end
