@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,15 +14,15 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+
 module OpenNebula
+
     # The Pool class represents a generic OpenNebula Pool in XML format
     # and provides the basic functionality to handle the Pool elements
-    
     class Pool < XMLPool
         include Enumerable
 
     protected
-
         #pool:: _String_ XML name of the root element
         #element:: _String_ XML name of the Pool elements
         #client::  _Client_ represents a XML-RPC connection
@@ -33,7 +33,6 @@ module OpenNebula
             @element_name = element.upcase
 
             @client = client
-            @hash   = nil
         end
 
         # Default Factory Method for the Pools. The factory method returns an
@@ -49,12 +48,35 @@ module OpenNebula
         # Common XML-RPC Methods for all the Pool Types
         #######################################################################
 
+        #Gets the pool without any filter. Host, Group and User Pools
+        # xml_method:: _String_ the name of the XML-RPC method
+        def info(xml_method)
+            return xmlrpc_info(xml_method)
+        end
+
+        def info_all(xml_method)
+            return xmlrpc_info(xml_method,INFO_ALL,-1,-1)
+        end
+
+        def info_mine(xml_method)
+            return xmlrpc_info(xml_method,INFO_MINE,-1,-1)
+        end
+
+        def info_group(xml_method)
+            return xmlrpc_info(xml_method,INFO_GROUP,-1,-1)
+        end
+
+        def info_filter(xml_method, who, start_id, end_id)
+            return xmlrpc_info(xml_method,who, start_id, end_id)
+        end
+
+    private
         # Calls to the corresponding info method to retreive the pool
         # representation in XML format
         # xml_method:: _String_ the name of the XML-RPC method
         # args:: _Array_ with additional arguments for the info call
         # [return] nil in case of success or an Error object
-        def info(xml_method,*args)
+        def xmlrpc_info(xml_method,*args)
             rc = @client.call(xml_method,*args)
 
             if !OpenNebula.is_error?(rc)
@@ -66,6 +88,10 @@ module OpenNebula
         end
 
     public
+        # Constants for info queries (include/RequestManagerPoolInfoFilter.h)
+        INFO_GROUP = -1
+        INFO_ALL   = -2
+        INFO_MINE  = -3
 
         # Iterates over every PoolElement in the Pool and calls the block with a
         # a PoolElement obtained calling the factory method
@@ -93,7 +119,6 @@ module OpenNebula
         def initialize(node, client)
             @xml    = node
             @client = client
-            @hash   = nil
 
             if self['ID']
                 @pe_id = self['ID'].to_i
@@ -148,13 +173,12 @@ module OpenNebula
         # Calls to the corresponding update method to modify
         # the object's template
         # xml_method:: _String_ the name of the XML-RPC method
-        # name:: _String_ the name of the property to be modified
-        # value:: _String_ the new value of the property to be modified
+        # new_template:: _String_ the new template contents
         # [return] nil in case of success or an Error object
-        def update(xml_method, name, value)
+        def update(xml_method, new_template)
             return Error.new('ID not defined') if !@pe_id
 
-            rc = @client.call(xml_method,@pe_id, name, value)
+            rc = @client.call(xml_method,@pe_id, new_template)
             rc = nil if !OpenNebula.is_error?(rc)
 
             return rc
@@ -173,6 +197,62 @@ module OpenNebula
             return rc
         end
 
+        # Calls to the corresponding chown method to modify
+        # the object's owner and group
+        # xml_method:: _String_ the name of the XML-RPC method
+        # uid:: _Integer_ the new owner id. Set to -1 to leave the current one
+        # gid:: _Integer_ the new group id. Set to -1 to leave the current one
+        # [return] nil in case of success or an Error object
+        def chown(xml_method, uid, gid)
+            return Error.new('ID not defined') if !@pe_id
+
+            rc = @client.call(xml_method,@pe_id, uid, gid)
+            rc = nil if !OpenNebula.is_error?(rc)
+
+            return rc
+        end
+
+        # Calls to the corresponding chmod method to modify
+        # the object's permission bits
+        #
+        # @param xml_method [String] the name of the XML-RPC method
+        # @param octet [String] Permissions octed , e.g. 640
+        # @return [nil, OpenNebula::Error] nil in case of success, Error
+        #   otherwise
+        def chmod_octet(xml_method, octet)
+            owner_u = octet[0..0].to_i & 4 != 0 ? 1 : 0
+            owner_m = octet[0..0].to_i & 2 != 0 ? 1 : 0
+            owner_a = octet[0..0].to_i & 1 != 0 ? 1 : 0
+            group_u = octet[1..1].to_i & 4 != 0 ? 1 : 0
+            group_m = octet[1..1].to_i & 2 != 0 ? 1 : 0
+            group_a = octet[1..1].to_i & 1 != 0 ? 1 : 0
+            other_u = octet[2..2].to_i & 4 != 0 ? 1 : 0
+            other_m = octet[2..2].to_i & 2 != 0 ? 1 : 0
+            other_a = octet[2..2].to_i & 1 != 0 ? 1 : 0
+
+            chmod(owner_u, owner_m, owner_a, group_u, group_m, group_a, other_u,
+                other_m, other_a)
+        end
+
+        # Calls to the corresponding chmod method to modify
+        # the object's permission bits
+        # Each [Integer] parameter must be 1 to allow, 0 deny, -1 do not change
+        #
+        # @param xml_method [String] the name of the XML-RPC method
+        # @return [nil, OpenNebula::Error] nil in case of success, Error
+        #   otherwise
+        def chmod(xml_method, owner_u, owner_m, owner_a, group_u, group_m, group_a, other_u,
+                other_m, other_a)
+            return Error.new('ID not defined') if !@pe_id
+
+            rc = @client.call(xml_method, @pe_id, owner_u, owner_m,
+                            owner_a, group_u, group_m, group_a, other_u,
+                            other_m, other_a)
+            rc = nil if !OpenNebula.is_error?(rc)
+
+            return rc
+        end
+
     public
 
         # Creates new element specifying its id
