@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,6 +14,7 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
+
 module OpenNebula
 
     begin
@@ -30,11 +31,9 @@ module OpenNebula
         REXML_FORMATTERS=false
     end
 
-    ###########################################################################
     # The XMLElement class provides an abstraction of the underlying
     # XML parser engine. It provides XML-related methods for the Pool and
     # PoolElement classes
-    ###########################################################################
     class XMLElement
 
         # xml:: _opaque xml object_ an xml object as returned by build_xml
@@ -101,22 +100,28 @@ module OpenNebula
                 element.text
             end
         end
-        
-        def retrieve_elements(filter)
-            ids_array = Array.new            
-            if NOKOGIRI
-                elements=@xml.xpath(filter.to_s)
 
-                if elements.size == 0
-                    return nil
-                end
-                
-                elements.each{ |e| ids_array << e.text }
+        # Gets an array of text from elemenets extracted
+        # using  the XPATH  expression passed as filter
+        def retrieve_elements(filter)
+            elements_array = Array.new
+
+            if NOKOGIRI
+                @xml.xpath(filter.to_s).each { |pelem|
+                    elements_array << pelem.text if pelem.text
+                 }
             else
-                @xml.each(filter.to_s) { |e|  ids_array << e.text }
+                @xml.elements.each(filter.to_s) { |pelem|
+                    elements_array << pelem.text if pelem.text
+                }
             end
-            
-            return ids_array
+
+            if elements_array.size == 0
+                return nil
+            else
+                return elements_array
+            end
+
         end
 
         # Gets an attribute from an elemenT
@@ -158,6 +163,18 @@ module OpenNebula
             end
         end
 
+        def each_xpath(xpath_str,&block)
+            if NOKOGIRI
+                @xml.xpath(xpath_str).each { |pelem|
+                    block.call pelem.text
+                }
+            else
+                @xml.elements.each(xpath_str) { |pelem|
+                    block.call pelem.text
+                }
+            end
+        end
+
         def name
             @xml.name
         end
@@ -184,7 +201,7 @@ module OpenNebula
             template_like_str('TEMPLATE', indent)
         end
 
-        def template_like_str(root_element, indent=true)
+        def template_like_str(root_element, indent=true, xpath_exp=nil)
             if NOKOGIRI
                 xml_template=@xml.xpath(root_element).to_s
                 rexml=REXML::Document.new(xml_template).root
@@ -200,7 +217,7 @@ module OpenNebula
                 ind_tab=' '
             end
 
-            str=rexml.collect {|n|
+            str=rexml.elements.collect(xpath_exp) {|n|
                 if n.class==REXML::Element
                     str_line=""
                     if n.has_elements?
@@ -208,14 +225,15 @@ module OpenNebula
 
                         str_line << n.collect {|n2|
                             if n2 && n2.class==REXML::Element
-                                str = ind_tab + n2.name + "="
-                                str += n2.text if n2.text
+                                str = ""
+                                str << ind_tab << n2.name << '='
+                                str << attr_to_str(n2.text) if n2.text
                                 str
                             end
-                        }.compact.join(","+ind_enter)
+                        }.compact.join(','+ind_enter)
                         str_line<<" ]"
                     else
-                        str_line<<n.name << "=" << n.text.to_s
+                        str_line << n.name << '=' << attr_to_str(n.text.to_s)
                     end
                     str_line
                 end
@@ -278,12 +296,21 @@ module OpenNebula
 
             hash
         end
+
+    private
+        def attr_to_str(attr)
+            attr.gsub!('"',"\\\"")
+
+            if attr.match(/[=,' ']/)
+                return '"' + attr + '"'
+            end
+
+            return attr
+        end
     end
 
-    ###########################################################################
     # The XMLUtilsPool module provides an abstraction of the underlying
     # XML parser engine. It provides XML-related methods for the Pools
-    ###########################################################################
     class XMLPool < XMLElement
 
         def initialize(xml=nil)
