@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -18,6 +18,22 @@
 #include "NebulaLog.h"
 #include <cerrno>
 
+#include "RequestManagerPoolInfoFilter.h"
+#include "RequestManagerInfo.h"
+#include "RequestManagerDelete.h"
+#include "RequestManagerAllocate.h"
+#include "RequestManagerUpdateTemplate.h"
+#include "RequestManagerChown.h"
+#include "RequestManagerChmod.h"
+
+#include "RequestManagerVirtualNetwork.h"
+#include "RequestManagerVirtualMachine.h"
+#include "RequestManagerVMTemplate.h"
+#include "RequestManagerHost.h"
+#include "RequestManagerImage.h"
+#include "RequestManagerUser.h"
+#include "RequestManagerAcl.h"
+
 #include <sys/signal.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -213,187 +229,176 @@ void RequestManager::do_action(
         
 void RequestManager::register_xml_methods()
 {
-    xmlrpc_c::methodPtr vm_allocate(new 
-        RequestManager::VirtualMachineAllocate(vmpool, vnpool, ipool, upool));
-        
-    xmlrpc_c::methodPtr vm_deploy(new 
-        RequestManager::VirtualMachineDeploy(vmpool,hpool,upool));
-        
-    xmlrpc_c::methodPtr vm_migrate(new 
-        RequestManager::VirtualMachineMigrate(vmpool,hpool,upool));
-        
-    xmlrpc_c::methodPtr vm_action(new 
-        RequestManager::VirtualMachineAction(vmpool,upool));
+    // User Methods
+    xmlrpc_c::methodPtr user_change_password(new UserChangePassword());
+    xmlrpc_c::methodPtr user_change_auth(new UserChangeAuth());
 
-    xmlrpc_c::methodPtr vm_savedisk(new
-        RequestManager::VirtualMachineSaveDisk(vmpool,upool,ipool));
+    // VMTemplate Methods
+    xmlrpc_c::methodPtr template_instantiate(new VMTemplateInstantiate());
 
-    xmlrpc_c::methodPtr vm_info(new 
-        RequestManager::VirtualMachineInfo(vmpool,upool));
+    // VirtualMachine Methods
+    xmlrpc_c::methodPtr vm_deploy(new VirtualMachineDeploy());
+    xmlrpc_c::methodPtr vm_migrate(new VirtualMachineMigrate());
+    xmlrpc_c::methodPtr vm_action(new VirtualMachineAction()); 
+    xmlrpc_c::methodPtr vm_savedisk(new VirtualMachineSaveDisk());
 
-    xmlrpc_c::methodPtr vm_pool_info(new
-        RequestManager::VirtualMachinePoolInfo(vmpool,upool));
-        
-    xmlrpc_c::methodPtr host_allocate(new 
-        RequestManager::HostAllocate(hpool,upool));
-        
-    xmlrpc_c::methodPtr host_info(new 
-        RequestManager::HostInfo(hpool, upool));
+    // VirtualNetwork Methods
+    xmlrpc_c::methodPtr vn_addleases(new VirtualNetworkAddLeases());
+    xmlrpc_c::methodPtr vn_rmleases(new VirtualNetworkRemoveLeases());
+    xmlrpc_c::methodPtr vn_hold(new VirtualNetworkHold());
+    xmlrpc_c::methodPtr vn_release(new VirtualNetworkRelease());
 
-    xmlrpc_c::methodPtr hostpool_info(new 
-        RequestManager::HostPoolInfo(hpool,upool));
-        
-    xmlrpc_c::methodPtr host_delete(new 
-        RequestManager::HostDelete(hpool,upool));
+    // Update Template Methods
+    xmlrpc_c::methodPtr image_update(new ImageUpdateTemplate());
+    xmlrpc_c::methodPtr template_update(new TemplateUpdateTemplate());
+    xmlrpc_c::methodPtr host_update(new HostUpdateTemplate());
+    xmlrpc_c::methodPtr vn_update(new VirtualNetworkUpdateTemplate());
+    xmlrpc_c::methodPtr user_update(new UserUpdateTemplate());
 
-    xmlrpc_c::methodPtr host_enable(new 
-        RequestManager::HostEnable(hpool,upool));
+    // Allocate Methods
+    xmlrpc_c::methodPtr vm_allocate(new VirtualMachineAllocate());
+    xmlrpc_c::methodPtr image_allocate(new ImageAllocate());
+    xmlrpc_c::methodPtr vn_allocate(new VirtualNetworkAllocate());
+    xmlrpc_c::methodPtr group_allocate(new GroupAllocate());
+    xmlrpc_c::methodPtr template_allocate(new TemplateAllocate());
+    xmlrpc_c::methodPtr host_allocate(new HostAllocate());
+    xmlrpc_c::methodPtr user_allocate(new  UserAllocate());
 
-    xmlrpc_c::methodPtr cluster_allocate(new 
-        RequestManager::ClusterAllocate(hpool,upool));
+    // Delete Methods
+    xmlrpc_c::methodPtr host_delete(new HostDelete());
+    xmlrpc_c::methodPtr template_delete(new TemplateDelete());
+    xmlrpc_c::methodPtr group_delete(new GroupDelete());
+    xmlrpc_c::methodPtr vn_delete(new VirtualNetworkDelete());
+    xmlrpc_c::methodPtr user_delete(new UserDelete());
+    xmlrpc_c::methodPtr image_delete(new ImageDelete());
 
-    xmlrpc_c::methodPtr cluster_info(new 
-        RequestManager::ClusterInfo(hpool,upool));
+    // Info Methods
+    xmlrpc_c::methodPtr vm_info(new VirtualMachineInfo());
+    xmlrpc_c::methodPtr host_info(new HostInfo());
+    xmlrpc_c::methodPtr template_info(new TemplateInfo());
+    xmlrpc_c::methodPtr group_info(new GroupInfo());
+    xmlrpc_c::methodPtr vn_info(new VirtualNetworkInfo());
+    xmlrpc_c::methodPtr user_info(new UserInfo());
+    xmlrpc_c::methodPtr image_info(new ImageInfo());
 
-    xmlrpc_c::methodPtr cluster_delete(new 
-        RequestManager::ClusterDelete(hpool,upool));
+    // PoolInfo Methods 
+    xmlrpc_c::methodPtr hostpool_info(new HostPoolInfo());
+    xmlrpc_c::methodPtr grouppool_info(new GroupPoolInfo());
+    xmlrpc_c::methodPtr userpool_info(new UserPoolInfo());
 
-    xmlrpc_c::methodPtr cluster_add(new 
-        RequestManager::ClusterAdd(hpool,upool));
+    // PoolInfo Methods with Filtering
+    xmlrpc_c::methodPtr vm_pool_info(new VirtualMachinePoolInfo());
+    xmlrpc_c::methodPtr template_pool_info(new TemplatePoolInfo());
+    xmlrpc_c::methodPtr vnpool_info(new VirtualNetworkPoolInfo());
+    xmlrpc_c::methodPtr imagepool_info(new ImagePoolInfo());
 
-    xmlrpc_c::methodPtr cluster_remove(new 
-        RequestManager::ClusterRemove(hpool,upool));
+    // Host Methods
+    xmlrpc_c::methodPtr host_enable(new HostEnable());
 
-    xmlrpc_c::methodPtr clusterpool_info(new 
-        RequestManager::ClusterPoolInfo(hpool,upool));
+    // Image Methods
+    xmlrpc_c::methodPtr image_persistent(new ImagePersistent());
+    xmlrpc_c::methodPtr image_enable(new ImageEnable());
+    xmlrpc_c::methodPtr image_chtype(new ImageChangeType());
 
-    xmlrpc_c::methodPtr vn_allocate(new 
-        RequestManager::VirtualNetworkAllocate(vnpool,upool));
-        
-    xmlrpc_c::methodPtr vn_info(new 
-        RequestManager::VirtualNetworkInfo(vnpool,upool));
-        
-    xmlrpc_c::methodPtr vnpool_info(new 
-        RequestManager::VirtualNetworkPoolInfo(vnpool,upool));
-        
-    xmlrpc_c::methodPtr vn_publish(new    
-        RequestManager::VirtualNetworkPublish(vnpool, upool));
+    // Chown Methods
+    xmlrpc_c::methodPtr vm_chown(new VirtualMachineChown());
+    xmlrpc_c::methodPtr template_chown(new TemplateChown());
+    xmlrpc_c::methodPtr vn_chown(new VirtualNetworkChown());
+    xmlrpc_c::methodPtr image_chown(new ImageChown());
+    xmlrpc_c::methodPtr user_chown(new UserChown());
 
-    xmlrpc_c::methodPtr vn_delete(new 
-        RequestManager::VirtualNetworkDelete(vnpool, upool));
+    // Chmod Methods
+    xmlrpc_c::methodPtr vm_chmod(new VirtualMachineChmod());
+    xmlrpc_c::methodPtr template_chmod(new TemplateChmod());
+    xmlrpc_c::methodPtr vn_chmod(new VirtualNetworkChmod());
+    xmlrpc_c::methodPtr image_chmod(new ImageChmod());
 
-    xmlrpc_c::methodPtr vn_addleases(new
-        RequestManager::VirtualNetworkAddLeases(vnpool, upool));
-
-    xmlrpc_c::methodPtr vn_rmleases(new
-        RequestManager::VirtualNetworkRemoveLeases(vnpool, upool));
-
-    xmlrpc_c::methodPtr user_allocate(new    
-        RequestManager::UserAllocate(upool));
-
-    xmlrpc_c::methodPtr user_delete(new    
-        RequestManager::UserDelete(upool));
-
-    xmlrpc_c::methodPtr user_info(new    
-        RequestManager::UserInfo(upool));
-
-    xmlrpc_c::methodPtr user_change_password(new
-        RequestManager::UserChangePassword(upool));
-
-    xmlrpc_c::methodPtr userpool_info(new    
-        RequestManager::UserPoolInfo(upool));
-        
-    xmlrpc_c::methodPtr image_allocate(new    
-        RequestManager::ImageAllocate(ipool, upool));
-        
-    xmlrpc_c::methodPtr image_delete(new    
-        RequestManager::ImageDelete(ipool, upool));
-        
-    xmlrpc_c::methodPtr image_info(new    
-        RequestManager::ImageInfo(ipool, upool));
-        
-    xmlrpc_c::methodPtr image_update(new    
-        RequestManager::ImageUpdate(ipool, upool));
-    
-    xmlrpc_c::methodPtr image_rm_attribute(new    
-        RequestManager::ImageRemoveAttribute(ipool, upool));
-        
-    xmlrpc_c::methodPtr image_publish(new    
-        RequestManager::ImagePublish(ipool, upool));
-        
-    xmlrpc_c::methodPtr image_persistent(new    
-        RequestManager::ImagePersistent(ipool, upool));
-        
-    xmlrpc_c::methodPtr image_enable(new    
-        RequestManager::ImageEnable(ipool, upool));
-        
-    xmlrpc_c::methodPtr imagepool_info(new    
-        RequestManager::ImagePoolInfo(ipool, upool));
+    // ACL Methods
+    xmlrpc_c::methodPtr acl_addrule(new AclAddRule());
+    xmlrpc_c::methodPtr acl_delrule(new AclDelRule());
+    xmlrpc_c::methodPtr acl_info(new AclInfo());
 
     /* VM related methods  */    
-        
-    RequestManagerRegistry.addMethod("one.vm.allocate", vm_allocate);
     RequestManagerRegistry.addMethod("one.vm.deploy", vm_deploy);
     RequestManagerRegistry.addMethod("one.vm.action", vm_action);
     RequestManagerRegistry.addMethod("one.vm.migrate", vm_migrate);
-    RequestManagerRegistry.addMethod("one.vm.info", vm_info);
     RequestManagerRegistry.addMethod("one.vm.savedisk", vm_savedisk);
+    RequestManagerRegistry.addMethod("one.vm.allocate", vm_allocate);
+    RequestManagerRegistry.addMethod("one.vm.info", vm_info);
+    RequestManagerRegistry.addMethod("one.vm.chown", vm_chown);
+    RequestManagerRegistry.addMethod("one.vm.chmod", vm_chmod);
 
     RequestManagerRegistry.addMethod("one.vmpool.info", vm_pool_info);
-     
+
+    /* VM Template related methods*/
+    RequestManagerRegistry.addMethod("one.template.update", template_update);
+    RequestManagerRegistry.addMethod("one.template.instantiate",template_instantiate);
+    RequestManagerRegistry.addMethod("one.template.allocate",template_allocate);
+    RequestManagerRegistry.addMethod("one.template.delete", template_delete);
+    RequestManagerRegistry.addMethod("one.template.info", template_info);
+    RequestManagerRegistry.addMethod("one.template.chown", template_chown);
+    RequestManagerRegistry.addMethod("one.template.chmod", template_chmod);
+
+    RequestManagerRegistry.addMethod("one.templatepool.info",template_pool_info);
+
     /* Host related methods*/
-     
-    RequestManagerRegistry.addMethod("one.host.allocate", host_allocate);   
-    RequestManagerRegistry.addMethod("one.host.info", host_info);
-    RequestManagerRegistry.addMethod("one.host.delete", host_delete);
     RequestManagerRegistry.addMethod("one.host.enable", host_enable);
-    
+    RequestManagerRegistry.addMethod("one.host.update", host_update);
+    RequestManagerRegistry.addMethod("one.host.allocate", host_allocate);   
+    RequestManagerRegistry.addMethod("one.host.delete", host_delete);
+    RequestManagerRegistry.addMethod("one.host.info", host_info);
+
     RequestManagerRegistry.addMethod("one.hostpool.info", hostpool_info); 
 
-    /* Cluster related methods */
+    /* Group related methods */
+    RequestManagerRegistry.addMethod("one.group.allocate",  group_allocate);
+    RequestManagerRegistry.addMethod("one.group.delete",    group_delete);
+    RequestManagerRegistry.addMethod("one.group.info",      group_info);
 
-    RequestManagerRegistry.addMethod("one.cluster.allocate", cluster_allocate);
-    RequestManagerRegistry.addMethod("one.cluster.info", cluster_info);
-    RequestManagerRegistry.addMethod("one.cluster.delete", cluster_delete);
-    RequestManagerRegistry.addMethod("one.cluster.add", cluster_add);
-    RequestManagerRegistry.addMethod("one.cluster.remove", cluster_remove);
-
-    RequestManagerRegistry.addMethod("one.clusterpool.info", clusterpool_info);
+    RequestManagerRegistry.addMethod("one.grouppool.info",  grouppool_info);
 
     /* Network related methods*/
-
-    RequestManagerRegistry.addMethod("one.vn.allocate", vn_allocate);   
-    RequestManagerRegistry.addMethod("one.vn.info", vn_info); 
-    RequestManagerRegistry.addMethod("one.vn.publish", vn_publish);
-    RequestManagerRegistry.addMethod("one.vn.delete", vn_delete);
     RequestManagerRegistry.addMethod("one.vn.addleases", vn_addleases);
     RequestManagerRegistry.addMethod("one.vn.rmleases", vn_rmleases);
+    RequestManagerRegistry.addMethod("one.vn.hold", vn_hold);
+    RequestManagerRegistry.addMethod("one.vn.release", vn_release);
+    RequestManagerRegistry.addMethod("one.vn.allocate", vn_allocate);
+    RequestManagerRegistry.addMethod("one.vn.update", vn_update);
+    RequestManagerRegistry.addMethod("one.vn.delete", vn_delete);
+    RequestManagerRegistry.addMethod("one.vn.info", vn_info); 
+    RequestManagerRegistry.addMethod("one.vn.chown", vn_chown);
+    RequestManagerRegistry.addMethod("one.vn.chmod", vn_chmod);
 
     RequestManagerRegistry.addMethod("one.vnpool.info", vnpool_info); 
     
-    
     /* User related methods*/
-        
     RequestManagerRegistry.addMethod("one.user.allocate", user_allocate);
+    RequestManagerRegistry.addMethod("one.user.update", user_update);
     RequestManagerRegistry.addMethod("one.user.delete", user_delete);
     RequestManagerRegistry.addMethod("one.user.info", user_info);
     RequestManagerRegistry.addMethod("one.user.passwd", user_change_password);
+    RequestManagerRegistry.addMethod("one.user.chgrp", user_chown);
+    RequestManagerRegistry.addMethod("one.user.chauth", user_change_auth);
 
     RequestManagerRegistry.addMethod("one.userpool.info", userpool_info);
     
     /* Image related methods*/
-    
+    RequestManagerRegistry.addMethod("one.image.persistent", image_persistent);
+    RequestManagerRegistry.addMethod("one.image.enable", image_enable);
+    RequestManagerRegistry.addMethod("one.image.update", image_update);     
     RequestManagerRegistry.addMethod("one.image.allocate", image_allocate);
     RequestManagerRegistry.addMethod("one.image.delete", image_delete);
     RequestManagerRegistry.addMethod("one.image.info", image_info);
-    RequestManagerRegistry.addMethod("one.image.update", image_update);     
-    RequestManagerRegistry.addMethod("one.image.rmattr", image_rm_attribute);
-    RequestManagerRegistry.addMethod("one.image.publish", image_publish);
-    RequestManagerRegistry.addMethod("one.image.persistent", image_persistent);
-    RequestManagerRegistry.addMethod("one.image.enable", image_enable);    
+    RequestManagerRegistry.addMethod("one.image.chown", image_chown);
+    RequestManagerRegistry.addMethod("one.image.chmod", image_chmod);
+    RequestManagerRegistry.addMethod("one.image.chtype", image_chtype);
 
     RequestManagerRegistry.addMethod("one.imagepool.info", imagepool_info);
-    
+
+    /* ACL related methods */
+    RequestManagerRegistry.addMethod("one.acl.addrule", acl_addrule);
+    RequestManagerRegistry.addMethod("one.acl.delrule", acl_delrule);
+    RequestManagerRegistry.addMethod("one.acl.info",    acl_info);
 };
 
 /* -------------------------------------------------------------------------- */
