@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -22,7 +22,9 @@ void VirtualMachineXML::init_attributes()
 {
     vector<string> result;
 
-    oid = atoi(((*this)["/VM/ID"] )[0].c_str() );
+    oid = atoi(((*this)["/VM/ID"] )[0].c_str());
+    uid = atoi(((*this)["/VM/UID"])[0].c_str());
+    gid = atoi(((*this)["/VM/GID"])[0].c_str());
 
     result = ((*this)["/VM/TEMPLATE/MEMORY"]);
     if (result.size() > 0)
