@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -27,9 +27,10 @@ module OpenNebulaJSON
             end
 
             self.allocate(host_hash['name'],
-                        host_hash['im_mad'],
-                        host_hash['vm_mad'],
-                        host_hash['tm_mad'])
+                          host_hash['im_mad'],
+                          host_hash['vm_mad'],
+                          host_hash['vnm_mad'],
+                          host_hash['tm_mad'])
         end
 
         def delete
@@ -49,11 +50,17 @@ module OpenNebulaJSON
             rc = case action_hash['perform']
                 when "enable"  then self.enable
                 when "disable" then self.disable
+                when "update" then self.update(action_hash['params'])
                 else
                     error_msg = "#{action_hash['perform']} action not " <<
                                 " available for this resource"
                     OpenNebula::Error.new(error_msg)
             end
         end
+
+        def update(params=Hash.new)
+            super(params['template_raw'])
+        end
+
     end
 end
