@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -32,7 +32,7 @@ module OpenNebulaJSON
                 template = template_to_str(image_hash)
             end
 
-            OpenNebula::ImageRepository.new.create(self, template)
+            self.allocate(template)
         end
 
         def perform_action(template_json)
@@ -42,27 +42,42 @@ module OpenNebulaJSON
             end
 
             rc = case action_hash['perform']
-                when "disable"       then self.disable
-                when "enable"        then self.enable
-                when "nonpersistent" then self.nonpersistent
-                when "persistent"    then self.persistent
-                when "publish"       then self.publish
-                when "rm_attr"       then self.remove_attr(action_hash['params'])
-                when "unpublish"     then self.unpublish
-                when "update"        then self.update(action_hash['params'])
-                else
-                    error_msg = "#{action_hash['perform']} action not " <<
-                                " available for this resource"
-                    OpenNebula::Error.new(error_msg)
-            end
+                 when "disable"       then self.disable
+                 when "enable"        then self.enable
+                 when "nonpersistent" then self.nonpersistent
+                 when "persistent"    then self.persistent
+                 when "publish"       then self.publish
+                 when "rm_attr"       then self.remove_attr(action_hash['params'])
+                 when "unpublish"     then self.unpublish
+                 when "update"        then self.update(action_hash['params'])
+                 when "chown"         then self.chown(action_hash['params'])
+                 when "chmod"         then self.chmod_octet(action_hash['params'])
+                 when "chtype"        then self.chtype(action_hash['params'])
+                 else
+                     error_msg = "#{action_hash['perform']} action not " <<
+                         " available for this resource"
+                     OpenNebula::Error.new(error_msg)
+                 end
         end
 
         def update(params=Hash.new)
-            super(params['name'], params['value'])
+            super(params['template_raw'])
         end
 
         def remove_attr(params=Hash.new)
             super(params['name'])
         end
+
+        def chown(params=Hash.new)
+            super(params['owner_id'].to_i,params['group_id'].to_i)
+        end
+
+        def chmod_octet(params=Hash.new)
+            super(params['octet'])
+        end
+
+        def chtype(params=Hash.new)
+            super(params['type'])
+        end
     end
 end
