@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -26,8 +26,9 @@ module OpenNebulaJSON
                 return user_hash
             end
 
-            password = Digest::SHA1.hexdigest(user_hash['password'])
-            self.allocate(user_hash['name'], password)
+            self.allocate(user_hash['name'],
+                          user_hash['password'],
+                          user_hash['auth_driver'])
         end
 
         def perform_action(template_json)
@@ -37,17 +38,42 @@ module OpenNebulaJSON
             end
 
             rc = case action_hash['perform']
-                when "passwd" then self.passwd(action_hash['params'])
-                else
-                    error_msg = "#{action_hash['perform']} action not " <<
-                                " available for this resource"
-                    OpenNebula::Error.new(error_msg)
-            end
+                 when "passwd"       then self.passwd(action_hash['params'])
+                 when "chgrp"        then self.chgrp(action_hash['params'])
+                 when "chauth"       then self.chauth(action_hash['params'])
+                 when "update"       then self.update(action_hash['params'])
+                 when "addgroup"     then self.addgroup(action_hash['params'])
+                 when "delgroup"     then self.delgroup(action_hash['params'])
+                 else
+                     error_msg = "#{action_hash['perform']} action not " <<
+                         " available for this resource"
+                     OpenNebula::Error.new(error_msg)
+                 end
         end
 
         def passwd(params=Hash.new)
-            password = Digest::SHA1.hexdigest(params['password'])
-            super(password)
+            super(params['password'])
         end
+
+        def chgrp(params=Hash.new)
+            super(params['group_id'].to_i)
+        end
+
+        def chauth(params=Hash.new)
+            super(params['auth_driver'])
+        end
+
+        def update(params=Hash.new)
+            super(params['template_raw'])
+        end
+
+        def addgroup(params=Hash.new)
+            super(params['group_id'].to_i)
+        end
+
+        def delgroup(params=Hash.new)
+            super(params['group_id'].to_i)
+        end
+
     end
 end
