@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -14,8 +14,6 @@
 /* limitations under the License.                                             */
 /* -------------------------------------------------------------------------- */
 
-#include <sqlite3.h>
-
 #include "Template.h"
 #include "template_syntax.h"
 
@@ -217,6 +215,11 @@ int Template::erase(const string& name)
 
     index = attributes.equal_range(name);
 
+    if (index.first == index.second )
+    {
+        return 0;
+    }
+
     for ( i = index.first,j=0 ; i != index.second ; i++,j++ )
     {
         Attribute * attr = i->second;
@@ -277,15 +280,12 @@ int Template::get(
 /* -------------------------------------------------------------------------- */
 
 void Template::get(
-        string& name,
+        const string& name,
         string& value) const
 {
     vector<const Attribute *>   attrs;
     const SingleAttribute *     sattr;
     int                         rc;
-
-    transform (name.begin(),name.end(),name.begin(),(int(*)(int))toupper);
-
     rc = get(name,attrs);
 
     if  (rc == 0)
@@ -309,8 +309,8 @@ void Template::get(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void Template::get(
-        string& name,
+bool Template::get(
+        const string& name,
         int&    value) const
 {
     string sval;
@@ -320,12 +320,13 @@ void Template::get(
     if ( sval == "" )
     {
         value = 0;
-        return;
+        return false;
     }
 
     istringstream iss(sval);
 
     iss >> value;
+    return true;
 }
 
 /* -------------------------------------------------------------------------- */
@@ -445,12 +446,7 @@ Attribute * Template::vector_xml_att(const xmlNode * node)
 int Template::from_xml(const string &xml_str)
 {
     xmlDocPtr xml_doc = 0;
-
     xmlNode * root_element;
-    xmlNode * cur_node = 0;
-
-    Attribute * attr;
-
 
     // Parse xml string as libxml document
     xml_doc = xmlParseMemory (xml_str.c_str(),xml_str.length());
@@ -460,6 +456,44 @@ int Template::from_xml(const string &xml_str)
         return -1;
     }
 
+    // Get the <TEMPLATE> element
+    root_element = xmlDocGetRootElement(xml_doc);
+    if( root_element == 0 )
+    {
+        return -1;
+    }
+
+    rebuild_attributes(root_element);
+
+    xmlFreeDoc(xml_doc);
+
+    return 0;
+}
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+int Template::from_xml_node(const xmlNodePtr node)
+{
+    if (node == 0)
+    {
+        return -1;
+    }
+
+    rebuild_attributes(node);
+
+    return 0;
+}
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+void Template::rebuild_attributes(const xmlNode * root_element)
+{
+    xmlNode * cur_node = 0;
+
+    Attribute * attr;
+
     //Clear the template if not empty
     if (!attributes.empty())
     {
@@ -473,9 +507,6 @@ int Template::from_xml(const string &xml_str)
         attributes.clear();
     }
 
-    // Get the <TEMPLATE> element
-    root_element = xmlDocGetRootElement(xml_doc);
-
     // Get the root's children and try to build attributes.
     for (cur_node = root_element->children;
          cur_node != 0;
@@ -497,10 +528,6 @@ int Template::from_xml(const string &xml_str)
             }
         }
     }
-
-    xmlFreeDoc(xml_doc);
-
-    return 0;
 }
 
 /* -------------------------------------------------------------------------- */
