@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -36,33 +36,37 @@ require 'TMScript'
 
 class TransferManager < OpenNebulaDriver
 
-    def initialize(plugin, num)
-        super(num, true)
-        
+    def initialize(plugin, options={})
+        @options={
+            :threaded => true
+        }.merge!(options)
+
+        super('', @options)
+
         @plugin=plugin
-        
+
         # register actions
         register_action(:TRANSFER, method("action_transfer"))
     end
-    
+
     def action_transfer(number, script_file)
         script_text=""
-        
+
         if File.exist?(script_file)
             open(script_file) {|f|
                 script_text=f.read
             }
-        
+
             script=TMScript.new(script_text, log_method(number))
             res=script.execute(@plugin)
-        
+
             if res[0]
                 send_message("TRANSFER", RESULT[:success], number)
             else
                 send_message("TRANSFER", RESULT[:failure], number, res[1])
             end
         else
-            send_message("TRANSFER", RESULT[:failure], number, 
+            send_message("TRANSFER", RESULT[:failure], number,
                 "Transfer file not found: #{script_file}")
         end
     end
@@ -80,7 +84,9 @@ tm_conf=ETC_LOCATION+tm_conf if tm_conf[0] != ?/
 
 plugin=TMPlugin.new(tm_conf)
 
-tm=TransferManager.new(plugin, 15)
+tm=TransferManager.new(plugin,
+    :concurrency => 15)
+
 tm.start_driver
 
 
