@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -16,8 +16,6 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
-CONTEXT_FILE=$1
-
 while (( "$#" )); do
     if [ "$#" == "1" ]; then
         DST=$1
@@ -49,27 +47,26 @@ fi
 ISO_DIR="$TMP_DIR/isofiles"
 
 
-exec_and_log "mkdir -p $ISO_DIR"
+exec_and_log "mkdir -p $ISO_DIR" \
+    "Error creating directory $ISO_DIR"
 
 for f in $SRC; do
     case $f in
     http://*)
-        exec_and_log "$WGET -O $ISO_DIR $f"
+        exec_and_log "$WGET -P $ISO_DIR $f" \
+            "Error downloading $f"
         ;;
 
     *)
-        if [ $SECURE_CONTEXT -eq 0 -o "$f" == "$CONTEXT_FILE" ]; then
-            exec_and_log "cp -R $f $ISO_DIR"
-        else
-            log "not copying potentialy dangerous file $f"
-        fi
-
-        exec_and_log "cp -R $f $ISO_DIR"
+        exec_and_log "cp -R $f $ISO_DIR" \
+            "Error copying $f to $ISO_DIR"
         ;;
     esac
 done
 
-exec_and_log "$MKISOFS -o $TMP_DIR/$DST_FILE -J -R $ISO_DIR"
-exec_and_log "$SCP $TMP_DIR/$DST_FILE $DST"
-exec_and_log "rm -rf $TMP_DIR"
-
+exec_and_log "$MKISOFS -o $TMP_DIR/$DST_FILE -J -R $ISO_DIR" \
+    "Error creating iso fs"
+exec_and_log "$SCP $TMP_DIR/$DST_FILE $DST" \
+    "Error copying $TMP_DIR/$DST_FILE to $DST"
+exec_and_log "rm -rf $TMP_DIR" \
+    "Error deleting $TMP_DIR"
