@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -31,4 +31,5 @@ SRC_PATH=`arg_path $SRC`
 SRC_HOST=`arg_host $SRC`
 
 log "Deleting $SRC_PATH"
-exec_and_log "$SSH $SRC_HOST rm -rf $SRC_PATH"
+exec_and_log "$SSH $SRC_HOST rm -rf $SRC_PATH" \
+    "Error deleting $SRC_PATH"
