@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -32,7 +32,12 @@ DST_PATH=`arg_path $DST`
 DST_HOST=`arg_host $DST`
 DST_DIR=`dirname $DST_PATH`
 
-exec_and_log "$SSH $DST_HOST mkdir -p $DST_DIR"
-exec_and_log "$SSH $DST_HOST $DD if=/dev/zero of=$DST_PATH bs=1 count=1 seek=${SIZE}M"
-exec_and_log "$SSH $DST_HOST $MKFS -t $FSTYPE -F $DST_PATH"
+MKFS_CMD=`mkfs_command $DST_PATH $FSTYPE`
+
+exec_and_log "$SSH $DST_HOST mkdir -p $DST_DIR" \
+    "Error creating directory $DST_DIR"
+exec_and_log "$SSH $DST_HOST $DD if=/dev/zero of=$DST_PATH bs=1 count=1 seek=${SIZE}M" \
+    "Could not create image $DST_PATH"
+exec_and_log "$SSH $DST_HOST $MKFS_CMD" \
+    "Unable to create filesystem $FSTYPE in $DST_PATH"
 exec_and_log "$SSH $DST_HOST chmod a+rw $DST_PATH"
