@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -35,7 +35,14 @@ DST_HOST=`arg_host $DST`
 
 DST_DIR=`dirname $DST_PATH`
 
-log "Moving $SRC_PATH"
-exec_and_log "$SSH $DST_HOST mkdir -p $DST_DIR"
-exec_and_log "$SCP -r $SRC $DST"
-exec_and_log "$SSH $SRC_HOST rm -rf $SRC_PATH"
+if full_src_and_dst_equal; then
+    log "Not moving $SRC to $DST, they are the same path"
+else
+    log "Moving $SRC_PATH"
+    exec_and_log "$SSH $DST_HOST mkdir -p $DST_DIR" \
+        "Unable to create directory $DST_DIR"
+    exec_and_log "$SCP -r $SRC $DST" \
+        "Could not copy $SRC to $DST"
+    exec_and_log "$SSH $SRC_HOST rm -rf $SRC_PATH"
+fi
+
