@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,6 +25,7 @@
 
 #include "VirtualMachine.h"
 #include "VirtualNetworkPool.h"
+#include "ImagePool.h"
 #include "NebulaLog.h"
 
 #include "Nebula.h"
@@ -36,11 +37,14 @@
 /* Virtual Machine :: Constructor/Destructor                                  */
 /* ************************************************************************** */
 
-VirtualMachine::VirtualMachine(int id, VirtualMachineTemplate * _vm_template):
-        PoolObjectSQL(id),
-        uid(-1),
+VirtualMachine::VirtualMachine(int           id,
+                               int           _uid,
+                               int           _gid,
+                               const string& _uname,
+                               const string& _gname,
+                               VirtualMachineTemplate * _vm_template):
+        PoolObjectSQL(id,VM,"",_uid,_gid,_uname,_gname,table),
         last_poll(0),
-        name(""),
         state(INIT),
         lcm_state(LCM_INIT),
         stime(time(0)),
@@ -50,31 +54,25 @@ VirtualMachine::VirtualMachine(int id, VirtualMachineTemplate * _vm_template):
         cpu(0),
         net_tx(0),
         net_rx(0),
-        last_seq(-1),
         history(0),
         previous_history(0),
         _log(0)
 {
     if (_vm_template != 0)
     {
-        vm_template = _vm_template;
+        obj_template = _vm_template;
     }
     else
     {
-        vm_template = new VirtualMachineTemplate;
+        obj_template = new VirtualMachineTemplate;
     }
 }
 
 VirtualMachine::~VirtualMachine()
 {
-    if ( history != 0 )
+    for (unsigned int i=0 ; i < history_records.size() ; i++)
     {
-        delete history;
-    }
-
-    if ( previous_history != 0 )
-    {
-        delete previous_history;
+            delete history_records[i];
     }
 
     if ( _log != 0 )
@@ -82,9 +80,9 @@ VirtualMachine::~VirtualMachine()
         delete _log;
     }
 
-    if ( vm_template != 0 )
+    if ( obj_template != 0 )
     {
-        delete vm_template;
+        delete obj_template;
     }
 }
 
@@ -95,133 +93,62 @@ VirtualMachine::~VirtualMachine()
 const char * VirtualMachine::table = "vm_pool";
 
 const char * VirtualMachine::db_names =
-    "oid,uid,name,last_poll, state,lcm_state,stime,etime,deploy_id"
-    ",memory,cpu,net_tx,net_rx,last_seq, template";
-
-const char * VirtualMachine::extended_db_names =
-    "vm_pool.oid, vm_pool.uid, vm_pool.name, vm_pool.last_poll, vm_pool.state, "
-    "vm_pool.lcm_state, vm_pool.stime, vm_pool.etime, vm_pool.deploy_id, "
-    "vm_pool.memory, vm_pool.cpu, vm_pool.net_tx, vm_pool.net_rx, "
-    "vm_pool.last_seq, vm_pool.template";
+    "oid, name, body, uid, gid, last_poll, state, lcm_state, "
+    "owner_u, group_u, other_u";
 
 const char * VirtualMachine::db_bootstrap = "CREATE TABLE IF NOT EXISTS "
-        "vm_pool ("
-        "oid INTEGER PRIMARY KEY,uid INTEGER,name TEXT,"
-        "last_poll INTEGER, state INTEGER,lcm_state INTEGER,"
-        "stime INTEGER,etime INTEGER,deploy_id TEXT,memory INTEGER,cpu INTEGER,"
-        "net_tx INTEGER,net_rx INTEGER, last_seq INTEGER, template TEXT)";
+        "vm_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128), body TEXT, uid INTEGER, "
+        "gid INTEGER, last_poll INTEGER, state INTEGER, lcm_state INTEGER, "
+        "owner_u INTEGER, group_u INTEGER, other_u INTEGER)";
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachine::select_cb(void *nil, int num, char **values, char **names)
-{
-    if ((values[OID] == 0) ||
-        (values[UID] == 0) ||
-        (values[NAME] == 0) ||
-        (values[LAST_POLL] == 0) ||
-        (values[STATE] == 0) ||
-        (values[LCM_STATE] == 0) ||
-        (values[STIME] == 0) ||
-        (values[ETIME] == 0) ||
-        (values[MEMORY] == 0) ||
-        (values[CPU] == 0) ||
-        (values[NET_TX] == 0) ||
-        (values[NET_RX] == 0) ||
-        (values[LAST_SEQ] == 0) ||
-        (values[TEMPLATE] == 0) ||
-        (num != LIMIT ))
-    {
-        return -1;
-    }
-
-    oid  = atoi(values[OID]);
-    uid  = atoi(values[UID]);
-    name = values[NAME];
-
-    last_poll = static_cast<time_t>(atoi(values[LAST_POLL]));
-
-    state     = static_cast<VmState>(atoi(values[STATE]));
-    lcm_state = static_cast<LcmState>(atoi(values[LCM_STATE]));
-
-    stime = static_cast<time_t>(atoi(values[STIME]));
-    etime = static_cast<time_t>(atoi(values[ETIME]));
-
-    if ( values[DEPLOY_ID] != 0 )
-    {
-        deploy_id = values[DEPLOY_ID];
-    }
-
-    memory      = atoi(values[MEMORY]);
-    cpu         = atoi(values[CPU]);
-    net_tx      = atoi(values[NET_TX]);
-    net_rx      = atoi(values[NET_RX]);
-    last_seq    = atoi(values[LAST_SEQ]);
-
-    // Virtual Machine template
-    vm_template->from_xml(values[TEMPLATE]);
-
-    return 0;
-}
-
-/* -------------------------------------------------------------------------- */
-
 int VirtualMachine::select(SqlDB * db)
 {
     ostringstream   oss;
     ostringstream   ose;
 
     int             rc;
-    int             boid;
+    int             last_seq;
 
-    string          filename;
     Nebula&         nd = Nebula::instance();
 
-    set_callback(
-        static_cast<Callbackable::Callback>(&VirtualMachine::select_cb));
+    // Rebuild the VirtualMachine object
+    rc = PoolObjectSQL::select(db);
 
-    oss << "SELECT " << db_names << " FROM " << table << " WHERE oid = " << oid;
-
-    boid = oid;
-    oid  = -1;
-
-    rc = db->exec(oss,this);
-
-    unset_callback();
-
-    if ((rc != 0) || (oid != boid ))
+    if( rc != 0 )
     {
-        goto error_id;
+        return rc;
     }
 
-    //Get the History Records
-
-    if ( last_seq != -1 )
+    //Get History Records. Current history is built in from_xml() (if any).
+    if( hasHistory() )
     {
-        history = new History(oid, last_seq);
+        last_seq = history->seq - 1;
 
-        rc = history->select(db);
-
-        if (rc != 0)
+        for (int i = last_seq; i >= 0; i--)
         {
-            goto error_history;
-        }
-    }
+            History * hp;
 
-    if ( last_seq > 0 )
-    {
-        previous_history = new History(oid, last_seq - 1);
+            hp = new History(oid, i);
+            rc = hp->select(db);
 
-        rc = previous_history->select(db);
+            if ( rc != 0)
+            {
+                goto error_previous_history;
+            }
 
-        if ( rc != 0)
-        {
-            goto error_previous_history;
+            history_records[i] = hp;
+
+            if ( i == last_seq )
+            {
+                previous_history = hp;
+            }
         }
     }
 
     //Create support directory for this VM
-
     oss.str("");
     oss << nd.get_var_location() << oid;
 
@@ -229,7 +156,6 @@ int VirtualMachine::select(SqlDB * db)
     chmod(oss.str().c_str(), 0777);
 
     //Create Log support for this VM
-
     try
     {
         _log = new FileLog(nd.get_vm_log_filename(oid),Log::DEBUG);
@@ -244,19 +170,10 @@ int VirtualMachine::select(SqlDB * db)
 
     return 0;
 
-error_id:
-    ose << "Error getting VM id: " << oid;
-    log("VMM", Log::ERROR, ose);
-    return -1;
-
-error_history:
-    ose << "Can not get history for VM id: " << oid;
-    log("ONE", Log::ERROR, ose);
-    return -1;
-
 error_previous_history:
     ose << "Can not get previous history record (seq:" << history->seq
         << ") for VM id: " << oid;
+
     log("ONE", Log::ERROR, ose);
     return -1;
 }
@@ -270,19 +187,36 @@ int VirtualMachine::insert(SqlDB * db, string& error_str)
     string name;
 
     SingleAttribute *   attr;
+    string              aname;
     string              value;
     ostringstream       oss;
 
 
-    // -----------------------------------------------------------------------
+    // ------------------------------------------------------------------------
+    // Check template for restricted attributes
+    // ------------------------------------------------------------------------
+
+    if ( uid != 0 && gid != GroupPool::ONEADMIN_ID )
+    {
+        VirtualMachineTemplate *vt = 
+            static_cast<VirtualMachineTemplate *>(obj_template);
+        
+        if (vt->check(aname))
+        {
+            goto error_restricted;            
+        }
+    }
+
+    // ------------------------------------------------------------------------
     // Set a name if the VM has not got one and VM_ID
     // ------------------------------------------------------------------------
+
     oss << oid;
     value = oss.str();
 
     attr = new SingleAttribute("VMID",value);
 
-    vm_template->set(attr);
+    obj_template->set(attr);
 
     get_template_attribute("NAME",name);
 
@@ -292,8 +226,11 @@ int VirtualMachine::insert(SqlDB * db, string& error_str)
         oss << "one-" << oid;
         name = oss.str();
 
-        attr = new SingleAttribute("NAME",name);
-        vm_template->set(attr);
+        replace_template_attribute("NAME", name);
+    }
+    else if ( name.length() > 128 )
+    {
+        goto error_name_length;
     }
 
     this->name = name;
@@ -302,11 +239,11 @@ int VirtualMachine::insert(SqlDB * db, string& error_str)
     // Get network leases
     // ------------------------------------------------------------------------
 
-    rc = get_network_leases();
+    rc = get_network_leases(error_str);
 
     if ( rc != 0 )
     {
-        goto error_leases;
+        goto error_leases_rollback;
     }
 
     // ------------------------------------------------------------------------
@@ -317,21 +254,23 @@ int VirtualMachine::insert(SqlDB * db, string& error_str)
 
     if ( rc != 0 )
     {
-        goto error_images;
+        // The get_disk_images method has an internal rollback for
+        // the acquired images, release_disk_images() would release all disks
+        goto error_leases_rollback;
     }
 
     // -------------------------------------------------------------------------
     // Parse the context & requirements
     // -------------------------------------------------------------------------
 
-    rc = parse_context();
+    rc = parse_context(error_str);
 
     if ( rc != 0 )
     {
         goto error_context;
     }
 
-    rc = parse_requirements();
+    rc = parse_requirements(error_str);
 
     if ( rc != 0 )
     {
@@ -344,7 +283,7 @@ int VirtualMachine::insert(SqlDB * db, string& error_str)
     // Insert the VM
     // ------------------------------------------------------------------------
 
-    rc = insert_replace(db, false);
+    rc = insert_replace(db, false, error_str);
 
     if ( rc != 0 )
     {
@@ -354,37 +293,41 @@ int VirtualMachine::insert(SqlDB * db, string& error_str)
     return 0;
 
 error_update:
-    error_str = "Can not insert VM in the database.";
-    goto error_common;
-
-error_leases:
-    error_str = "Could not get network lease for VM.";
-    NebulaLog::log("ONE",Log::ERROR, error_str);
-    release_network_leases();
-    return -1;
-
-error_images:
-    goto error_common;
+    goto error_rollback;
 
 error_context:
-    error_str = "Could not parse CONTEXT for VM.";
-    goto error_common;
+    goto error_rollback;
 
 error_requirements:
-    error_str = "Could not parse REQUIREMENTS for VM.";
+    goto error_rollback;
+
+error_rollback:
+    release_disk_images();
+
+error_leases_rollback:
+    release_network_leases();
+    goto error_common;
+
+error_restricted:
+    oss << "VM Template includes a restricted attribute " << aname << "."; 
+    error_str = oss.str(); 
+    goto error_common;
+
+error_name_length:
+    oss << "NAME is too long; max length is 128 chars.";
+    error_str = oss.str(); 
     goto error_common;
 
 error_common:
     NebulaLog::log("ONE",Log::ERROR, error_str);
-    release_network_leases();
-    release_disk_images();
+
     return -1;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachine::parse_context()
+int VirtualMachine::parse_context(string& error_str)
 {
     int rc, num;
 
@@ -394,18 +337,23 @@ int VirtualMachine::parse_context()
     string *            str;
     string              parsed;
 
-    num = vm_template->remove("CONTEXT", array_context);
+    num = obj_template->remove("CONTEXT", array_context);
 
     if ( num == 0 )
     {
         return 0;
     }
+    else if ( num > 1 )
+    {
+        error_str = "Only one CONTEXT attribute can be defined.";
+        return -1;
+    }
 
     context = dynamic_cast<VectorAttribute *>(array_context[0]);
 
     if ( context == 0 )
     {
-        NebulaLog::log("ONE",Log::ERROR, "Wrong format for CONTEXT attribute");
+        error_str = "Wrong format for CONTEXT attribute.";
         return -1;
     }
 
@@ -440,7 +388,7 @@ int VirtualMachine::parse_context()
             context_parsed->replace("TARGET", dev_prefix);
         }
 
-        vm_template->set(context_parsed);
+        obj_template->set(context_parsed);
     }
 
     /* --- Delete old context attributes --- */
@@ -466,7 +414,7 @@ void VirtualMachine::parse_graphics()
     vector<Attribute *> array_graphics;
     VectorAttribute *   graphics;
 
-    num = vm_template->get("GRAPHICS", array_graphics);
+    num = obj_template->get("GRAPHICS", array_graphics);
 
     if ( num == 0 )
     {
@@ -504,7 +452,7 @@ void VirtualMachine::parse_graphics()
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachine::parse_requirements()
+int VirtualMachine::parse_requirements(string& error_str)
 {
     int rc, num;
 
@@ -513,18 +461,23 @@ int VirtualMachine::parse_requirements()
 
     string              parsed;
 
-    num = vm_template->remove("REQUIREMENTS", array_reqs);
+    num = obj_template->remove("REQUIREMENTS", array_reqs);
 
     if ( num == 0 )
     {
         return 0;
     }
+    else if ( num > 1 )
+    {
+        error_str = "Only one REQUIREMENTS attribute can be defined.";
+        return -1;
+    }
 
     reqs = dynamic_cast<SingleAttribute *>(array_reqs[0]);
 
     if ( reqs == 0 )
     {
-        NebulaLog::log("ONE",Log::ERROR,"Wrong format for REQUIREMENTS");
+        error_str = "Wrong format for REQUIREMENTS attribute.";
         return -1;
     }
 
@@ -535,7 +488,7 @@ int VirtualMachine::parse_requirements()
         SingleAttribute * reqs_parsed;
 
         reqs_parsed = new SingleAttribute("REQUIREMENTS",parsed);
-        vm_template->set(reqs_parsed);
+        obj_template->set(reqs_parsed);
     }
 
     /* --- Delete old requirements attributes --- */
@@ -554,29 +507,21 @@ int VirtualMachine::parse_requirements()
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
-int VirtualMachine::update(SqlDB * db)
-{
-    return insert_replace(db, true);
-}
-
-/* ------------------------------------------------------------------------ */
-/* ------------------------------------------------------------------------ */
-
-int VirtualMachine::insert_replace(SqlDB *db, bool replace)
+int VirtualMachine::insert_replace(SqlDB *db, bool replace, string& error_str)
 {
     ostringstream   oss;
     int             rc;
 
-    string xml_template;
+    string xml_body;
     char * sql_deploy_id;
     char * sql_name;
-    char * sql_template;
+    char * sql_xml;
 
     sql_deploy_id = db->escape_str(deploy_id.c_str());
 
     if ( sql_deploy_id == 0 )
     {
-        goto error_deploy;
+        goto error_generic;
     }
 
     sql_name =  db->escape_str(name.c_str());
@@ -586,14 +531,17 @@ int VirtualMachine::insert_replace(SqlDB *db, bool replace)
         goto error_name;
     }
 
-    vm_template->to_xml(xml_template);
-    sql_template = db->escape_str(xml_template.c_str());
+    sql_xml = db->escape_str(to_xml(xml_body).c_str());
 
-    if ( sql_template == 0 )
+    if ( sql_xml == 0 )
     {
-        goto error_template;
+        goto error_body;
     }
 
+    if ( validate_xml(sql_xml) != 0 )
+    {
+        goto error_xml;
+    }
 
     if(replace)
     {
@@ -606,150 +554,59 @@ int VirtualMachine::insert_replace(SqlDB *db, bool replace)
 
     oss << " INTO " << table << " ("<< db_names <<") VALUES ("
         <<          oid             << ","
-        <<          uid             << ","
         << "'" <<   sql_name        << "',"
+        << "'" <<   sql_xml         << "',"
+        <<          uid             << ","
+        <<          gid             << ","
         <<          last_poll       << ","
         <<          state           << ","
         <<          lcm_state       << ","
-        <<          stime           << ","
-        <<          etime           << ","
-        << "'" <<   sql_deploy_id   << "',"
-        <<          memory          << ","
-        <<          cpu             << ","
-        <<          net_tx          << ","
-        <<          net_rx          << ","
-        <<          last_seq        << ","
-        << "'" <<   sql_template    << "')";
+        <<          owner_u         << ","
+        <<          group_u         << ","
+        <<          other_u         << ")";
 
     db->free_str(sql_deploy_id);
     db->free_str(sql_name);
-    db->free_str(sql_template);
+    db->free_str(sql_xml);
 
     rc = db->exec(oss);
 
     return rc;
 
-error_template:
+error_xml:
+    db->free_str(sql_deploy_id);
     db->free_str(sql_name);
+    db->free_str(sql_xml);
+
+    error_str = "Error transforming the VM to XML.";
+
+    goto error_common;
+
+error_body:
+    db->free_str(sql_deploy_id);
+    db->free_str(sql_name);
+    goto error_generic;
+
 error_name:
     db->free_str(sql_deploy_id);
-error_deploy:
+    goto error_generic;
+
+error_generic:
+    error_str = "Error inserting VM in DB.";
+error_common:
     return -1;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachine::dump(  ostringstream& oss,
-                                    int num,
-                                    char **values,
-                                    char **names)
-{
-    if ((!values[OID])||
-        (!values[UID])||
-        (!values[NAME]) ||
-        (!values[LAST_POLL])||
-        (!values[STATE])||
-        (!values[LCM_STATE])||
-        (!values[STIME])||
-        (!values[ETIME])||
-        (!values[DEPLOY_ID])||
-        (!values[MEMORY])||
-        (!values[CPU])||
-        (!values[NET_TX])||
-        (!values[NET_RX])||
-        (!values[LAST_SEQ])||
-        (!values[TEMPLATE])||
-        (num != LIMIT))
-    {
-        return -1;
-    }
-
-    oss <<
-        "<VM>" <<
-            "<ID>"       << values[OID]      << "</ID>"       <<
-            "<UID>"      << values[UID]      << "</UID>"      <<
-            "<NAME>"     << values[NAME]     << "</NAME>"     <<
-            "<LAST_POLL>"<< values[LAST_POLL]<< "</LAST_POLL>"<<
-            "<STATE>"    << values[STATE]    << "</STATE>"    <<
-            "<LCM_STATE>"<< values[LCM_STATE]<< "</LCM_STATE>"<<
-            "<STIME>"    << values[STIME]    << "</STIME>"    <<
-            "<ETIME>"    << values[ETIME]    << "</ETIME>"    <<
-            "<DEPLOY_ID>"<< values[DEPLOY_ID]<< "</DEPLOY_ID>"<<
-            "<MEMORY>"   << values[MEMORY]   << "</MEMORY>"   <<
-            "<CPU>"      << values[CPU]      << "</CPU>"      <<
-            "<NET_TX>"   << values[NET_TX]   << "</NET_TX>"   <<
-            "<NET_RX>"   << values[NET_RX]   << "</NET_RX>"   <<
-            "<LAST_SEQ>" << values[LAST_SEQ] << "</LAST_SEQ>" <<
-                            values[TEMPLATE];
-    oss << "</VM>";
-
-    return 0;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-int VirtualMachine::dump_extended(  ostringstream& oss,
-                                    int num,
-                                    char **values,
-                                    char **names)
-{
-    if ((!values[OID])||
-        (!values[UID])||
-        (!values[NAME]) ||
-        (!values[LAST_POLL])||
-        (!values[STATE])||
-        (!values[LCM_STATE])||
-        (!values[STIME])||
-        (!values[ETIME])||
-        (!values[DEPLOY_ID])||
-        (!values[MEMORY])||
-        (!values[CPU])||
-        (!values[NET_TX])||
-        (!values[NET_RX])||
-        (!values[LAST_SEQ])||
-        (!values[TEMPLATE])||
-        (num != (LIMIT + History::LIMIT + 1)))
-    {
-        return -1;
-    }
-
-    oss <<
-        "<VM>" <<
-            "<ID>"       << values[OID]      << "</ID>"       <<
-            "<UID>"      << values[UID]      << "</UID>"      <<
-            "<USERNAME>" << values[LIMIT]     << "</USERNAME>"<<
-            "<NAME>"     << values[NAME]     << "</NAME>"     <<
-            "<LAST_POLL>"<< values[LAST_POLL]<< "</LAST_POLL>"<<
-            "<STATE>"    << values[STATE]    << "</STATE>"    <<
-            "<LCM_STATE>"<< values[LCM_STATE]<< "</LCM_STATE>"<<
-            "<STIME>"    << values[STIME]    << "</STIME>"    <<
-            "<ETIME>"    << values[ETIME]    << "</ETIME>"    <<
-            "<DEPLOY_ID>"<< values[DEPLOY_ID]<< "</DEPLOY_ID>"<<
-            "<MEMORY>"   << values[MEMORY]   << "</MEMORY>"   <<
-            "<CPU>"      << values[CPU]      << "</CPU>"      <<
-            "<NET_TX>"   << values[NET_TX]   << "</NET_TX>"   <<
-            "<NET_RX>"   << values[NET_RX]   << "</NET_RX>"   <<
-            "<LAST_SEQ>" << values[LAST_SEQ] << "</LAST_SEQ>" <<
-                            values[TEMPLATE];
-
-    History::dump(oss, num-LIMIT-1, values+LIMIT+1, names+LIMIT+1);
-
-    oss << "</VM>";
-
-    return 0;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
 void VirtualMachine::add_history(
-    int     hid,
-    string& hostname,
-    string& vm_dir,
-    string& vmm_mad,
-    string& tm_mad)
+    int   hid,
+    const string& hostname,
+    const string& vm_dir,
+    const string& vmm_mad,
+    const string& vnm_mad,
+    const string& tm_mad)
 {
     ostringstream os;
     int           seq;
@@ -762,17 +619,12 @@ void VirtualMachine::add_history(
     {
         seq = history->seq + 1;
 
-        if (previous_history != 0)
-        {
-            delete previous_history;
-        }
-
         previous_history = history;
     }
 
-    last_seq = seq;
+    history = new History(oid,seq,hid,hostname,vm_dir,vmm_mad,vnm_mad,tm_mad);
 
-    history = new History(oid,seq,hid,hostname,vm_dir,vmm_mad,tm_mad);
+    history_records.push_back(history);
 };
 
 /* -------------------------------------------------------------------------- */
@@ -788,23 +640,19 @@ void VirtualMachine::cp_history()
     }
 
     htmp = new History(oid,
-            history->seq + 1,
-            history->hid,
-            history->hostname,
-            history->vm_dir,
-            history->vmm_mad_name,
-            history->tm_mad_name);
+                       history->seq + 1,
+                       history->hid,
+                       history->hostname,
+                       history->vm_dir,
+                       history->vmm_mad_name,
+                       history->vnm_mad_name,
+                       history->tm_mad_name);
 
-    if ( previous_history != 0 )
-    {
-        delete previous_history;
-    }
 
     previous_history = history;
+    history          = htmp;
 
-    history = htmp;
-
-    last_seq = history->seq;
+    history_records.push_back(history);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -820,20 +668,18 @@ void VirtualMachine::cp_previous_history()
     }
 
     htmp = new History(oid,
-            history->seq + 1,
-            previous_history->hid,
-            previous_history->hostname,
-            previous_history->vm_dir,
-            previous_history->vmm_mad_name,
-            previous_history->tm_mad_name);
-
-    delete previous_history;
+                       history->seq + 1,
+                       previous_history->hid,
+                       previous_history->hostname,
+                       previous_history->vm_dir,
+                       previous_history->vmm_mad_name,
+                       previous_history->vnm_mad_name,
+                       previous_history->tm_mad_name);
 
     previous_history = history;
+    history          = htmp;
 
-    history = htmp;
-
-    last_seq = history->seq;
+    history_records.push_back(history);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -875,11 +721,13 @@ int VirtualMachine::get_disk_images(string& error_str)
     vector<Attribute  * > disks;
     ImagePool *           ipool;
     VectorAttribute *     disk;
+    vector<int>           acquired_images;
 
     int     n_os = 0; // Number of OS images
     int     n_cd = 0; // Number of CDROMS
     int     n_db = 0; // Number of DATABLOCKS
     string  type;
+    int     image_id;
 
     ostringstream    oss;
     Image::ImageType img_type;
@@ -887,7 +735,7 @@ int VirtualMachine::get_disk_images(string& error_str)
     Nebula& nd = Nebula::instance();
     ipool      = nd.get_ipool();
 
-    num_disks  = vm_template->get("DISK",disks);
+    num_disks  = obj_template->get("DISK",disks);
 
     for(int i=0, index=0; i<num_disks; i++)
     {
@@ -898,10 +746,12 @@ int VirtualMachine::get_disk_images(string& error_str)
             continue;
         }
 
-        rc = ipool->disk_attribute(disk, i, &index, &img_type);
+        rc = ipool->disk_attribute(disk, i, &index, &img_type, uid, image_id);
 
         if (rc == 0 )
         {
+            acquired_images.push_back(image_id);
+
             switch(img_type)
             {
                 case Image::OS:
@@ -954,9 +804,20 @@ error_max_db:
 
 error_image:
     error_str = "Could not get disk image for VM.";
+    goto error_common;
 
 error_common:
-    NebulaLog::log("ONE",Log::ERROR, error_str);
+    ImageManager *  imagem  = nd.get_imagem();
+
+    vector<int>::iterator it;
+
+    for ( it=acquired_images.begin() ; it < acquired_images.end(); it++ )
+    {
+        // Set disk_path and save_id to empty string, this way the image manager
+        // won't try to move any files
+        imagem->release_image(*it,"",-1,"");
+    }
+
     return -1;
 }
 
@@ -970,14 +831,20 @@ void VirtualMachine::release_disk_images()
     int     num_disks;
 
     vector<Attribute const  * > disks;
-    Image *                     img;
-    ImagePool *                 ipool;
+    ImageManager *              imagem;
+
+    string  disk_base_path = "";
 
     Nebula& nd = Nebula::instance();
-    ipool      = nd.get_ipool();
+    imagem     = nd.get_imagem();
 
     num_disks   = get_template_attribute("DISK",disks);
 
+    if (hasHistory() != 0)
+    {
+        disk_base_path = get_local_dir();
+    }
+
     for(int i=0; i<num_disks; i++)
     {
         VectorAttribute const *  disk =
@@ -988,39 +855,27 @@ void VirtualMachine::release_disk_images()
             continue;
         }
 
-        iid = disk->vector_value("IMAGE_ID");
+        iid    = disk->vector_value("IMAGE_ID");
+        saveas = disk->vector_value("SAVE_AS");
 
         if ( iid.empty() )
         {
-            continue;
+            if (!saveas.empty())
+            {
+                imagem->disk_to_image(disk_base_path,i,saveas);
+            }
         }
-
-        img = ipool->get(atoi(iid.c_str()),true);
-
-        if ( img == 0 )
+        else
         {
-            continue;
+            imagem->release_image(iid,disk_base_path,i,saveas);
         }
-
-        img->release_image();
-
-        saveas = disk->vector_value("SAVE_AS");
-
-        if ( !saveas.empty() && saveas == iid )
-        {
-            img->enable(false);
-        }
-
-        ipool->update(img);
-
-        img->unlock();
     }
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachine::get_network_leases()
+int VirtualMachine::get_network_leases(string& estr)
 {
     int                   num_nics, rc;
     vector<Attribute  * > nics;
@@ -1030,7 +885,7 @@ int VirtualMachine::get_network_leases()
     Nebula& nd = Nebula::instance();
     vnpool     = nd.get_vnpool();
 
-    num_nics   = vm_template->get("NIC",nics);
+    num_nics   = obj_template->get("NIC",nics);
 
     for(int i=0; i<num_nics; i++)
     {
@@ -1041,15 +896,19 @@ int VirtualMachine::get_network_leases()
             continue;
         }
 
-        rc = vnpool->nic_attribute(nic, oid);
+        rc = vnpool->nic_attribute(nic, uid, oid);
 
         if (rc == -1)
         {
-            return -1;
+            goto error_vnet; 
         }
     }
 
     return 0;
+
+error_vnet:
+    estr = "Could not get virtual network for VM.";
+    return -1;
 }
 
 /* -------------------------------------------------------------------------- */
@@ -1102,6 +961,8 @@ void VirtualMachine::release_network_leases()
         }
 
         vn->release_lease(ip);
+        vnpool->update(vn);
+
         vn->unlock();
     }
 }
@@ -1167,7 +1028,7 @@ int VirtualMachine::generate_context(string &files)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualMachine::save_disk(int disk_id, int img_id)
+int VirtualMachine::save_disk(int disk_id, int img_id, string& error_str)
 {
     int                   num_disks;
     vector<Attribute  * > disks;
@@ -1179,8 +1040,12 @@ int VirtualMachine::save_disk(int disk_id, int img_id)
     ostringstream oss;
     istringstream iss;
 
+    if ( state == DONE || state == FAILED )
+    {
+        goto error_state;
+    }
 
-    num_disks  = vm_template->get("DISK",disks);
+    num_disks  = obj_template->get("DISK",disks);
 
     for(int i=0; i<num_disks; i++, iss.clear())
     {
@@ -1196,8 +1061,18 @@ int VirtualMachine::save_disk(int disk_id, int img_id)
         iss.str(disk_id_str);
         iss >> tmp_disk_id;
 
-        if( tmp_disk_id == disk_id )
+        if ( tmp_disk_id == disk_id )
         {
+            if(!((disk->vector_value("SAVE_AS")).empty()))
+            {
+                goto error_saved;
+            }
+
+            if(!((disk->vector_value("PERSISTENT")).empty()))
+            {
+                goto error_persistent;
+            }
+
             disk->replace("SAVE", "YES");
 
             oss << (img_id);
@@ -1207,12 +1082,80 @@ int VirtualMachine::save_disk(int disk_id, int img_id)
         }
     }
 
+    goto error_not_found;
+
+error_state:
+    oss << "VM cannot be in DONE or FAILED state.";
+    goto error_common;
+
+error_persistent:
+    oss << "Source image for DISK " << disk_id << " is persistent.";
+    goto error_common;
+
+error_saved:
+    oss << "The DISK " << disk_id << " is already going to be saved.";
+    goto error_common;
+
+error_not_found:
+    oss << "The DISK " << disk_id << " does not exist for VM " << oid << ".";
+
+error_common:
+    error_str = oss.str();
+
     return -1;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void VirtualMachine::set_auth_request(int uid,
+                                      AuthRequest& ar,
+                                      VirtualMachineTemplate *tmpl) 
+{
+    int                   num;
+    vector<Attribute  * > vectors;
+    VectorAttribute *     vector;
+
+    Nebula& nd = Nebula::instance();
+
+    ImagePool *           ipool  = nd.get_ipool();
+    VirtualNetworkPool *  vnpool = nd.get_vnpool();
+
+    num = tmpl->get("DISK",vectors);
+
+    for(int i=0; i<num; i++)
+    {
+
+        vector = dynamic_cast<VectorAttribute * >(vectors[i]);
+
+        if ( vector == 0 )
+        {
+            continue;
+        }
+
+        ipool->authorize_disk(vector,uid,&ar);
+    }
+
+    vectors.clear();
+
+    num = tmpl->get("NIC",vectors);
+
+    for(int i=0; i<num; i++)
+    {
+        vector = dynamic_cast<VectorAttribute * >(vectors[i]);
+
+        if ( vector == 0 )
+        {
+            continue;
+        }
+
+        vnpool->authorize_nic(vector,uid,&ar);
+    }
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 pthread_mutex_t VirtualMachine::lex_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 extern "C"
@@ -1278,33 +1221,40 @@ error_yy:
     pthread_mutex_unlock(&lex_mutex);
     return -1;
 }
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-ostream& operator<<(ostream& os, const VirtualMachine& vm)
-{
-    string vm_str;
-
-    os << vm.to_xml(vm_str);
-
-    return os;
-};
-
-/* -------------------------------------------------------------------------- */
-
 string& VirtualMachine::to_xml(string& xml) const
 {
+    return to_xml_extended(xml,false);
+}
 
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string& VirtualMachine::to_xml_extended(string& xml) const
+{
+    return to_xml_extended(xml,true);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string& VirtualMachine::to_xml_extended(string& xml, bool extended) const
+{
     string template_xml;
     string history_xml;
-
+    string perm_xml;
     ostringstream	oss;
 
     oss << "<VM>"
         << "<ID>"        << oid       << "</ID>"
         << "<UID>"       << uid       << "</UID>"
+        << "<GID>"       << gid       << "</GID>"
+        << "<UNAME>"     << uname     << "</UNAME>" 
+        << "<GNAME>"     << gname     << "</GNAME>" 
         << "<NAME>"      << name      << "</NAME>"
+        << perms_to_xml(perm_xml)
         << "<LAST_POLL>" << last_poll << "</LAST_POLL>"
         << "<STATE>"     << state     << "</STATE>"
         << "<LCM_STATE>" << lcm_state << "</LCM_STATE>"
@@ -1315,12 +1265,29 @@ string& VirtualMachine::to_xml(string& xml) const
         << "<CPU>"       << cpu       << "</CPU>"
         << "<NET_TX>"    << net_tx    << "</NET_TX>"
         << "<NET_RX>"    << net_rx    << "</NET_RX>"
-        << "<LAST_SEQ>"  << last_seq  << "</LAST_SEQ>"
-        << vm_template->to_xml(template_xml);
+        << obj_template->to_xml(template_xml);
 
     if ( hasHistory() )
     {
-        oss << history->to_xml(history_xml);
+        oss << "<HISTORY_RECORDS>";
+
+        if ( extended )
+        {
+            for (unsigned int i=0; i < history_records.size(); i++)
+            {
+                oss << history_records[i]->to_xml(history_xml);
+            }
+        }
+        else
+        {
+            oss << history->to_xml(history_xml);
+        }
+
+        oss << "</HISTORY_RECORDS>";
+    }
+    else
+    {
+        oss << "<HISTORY_RECORDS/>";
     }
 
     oss << "</VM>";
@@ -1330,39 +1297,83 @@ string& VirtualMachine::to_xml(string& xml) const
     return xml;
 }
 
+/* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-string& VirtualMachine::to_str(string& str) const
+int VirtualMachine::from_xml(const string &xml_str)
 {
-    string template_str;
-    string history_str;
+    vector<xmlNodePtr> content;
 
-    ostringstream	oss;
+    int istate;
+    int ilcmstate;
+    int rc = 0;
 
-    oss<< "ID                : " << oid << endl
-       << "UID               : " << uid << endl
-       << "NAME              : " << name << endl
-       << "STATE             : " << state << endl
-       << "LCM STATE         : " << lcm_state << endl
-       << "DEPLOY ID         : " << deploy_id << endl
-       << "MEMORY            : " << memory << endl
-       << "CPU               : " << cpu << endl
-       << "LAST POLL         : " << last_poll << endl
-       << "START TIME        : " << stime << endl
-       << "STOP TIME         : " << etime << endl
-       << "NET TX            : " << net_tx << endl
-       << "NET RX            : " << net_rx << endl
-       << "LAST SEQ          : " << last_seq << endl
-       << "Template" << endl << vm_template->to_str(template_str) << endl;
+    // Initialize the internal XML object
+    update_from_str(xml_str);
 
-    if ( hasHistory() )
+    // Get class base attributes
+    rc += xpath(oid,       "/VM/ID",    -1);
+
+    rc += xpath(uid,       "/VM/UID",   -1);
+    rc += xpath(gid,       "/VM/GID",   -1);
+
+    rc += xpath(uname,     "/VM/UNAME", "not_found");
+    rc += xpath(gname,     "/VM/GNAME", "not_found");
+    rc += xpath(name,      "/VM/NAME",  "not_found");
+
+    rc += xpath(last_poll, "/VM/LAST_POLL", 0);
+    rc += xpath(istate,    "/VM/STATE",     0);
+    rc += xpath(ilcmstate, "/VM/LCM_STATE", 0);
+
+    rc += xpath(stime,     "/VM/STIME",    0);
+    rc += xpath(etime,     "/VM/ETIME",    0);
+    rc += xpath(deploy_id, "/VM/DEPLOY_ID","");
+
+    rc += xpath(memory,    "/VM/MEMORY",   0);
+    rc += xpath(cpu,       "/VM/CPU",      0);
+    rc += xpath(net_tx,    "/VM/NET_TX",   0);
+    rc += xpath(net_rx,    "/VM/NET_RX",   0);
+
+    // Permissions
+    rc += perms_from_xml();
+
+    state     = static_cast<VmState>(istate);
+    lcm_state = static_cast<LcmState>(ilcmstate);
+
+    // Get associated classes
+    ObjectXML::get_nodes("/VM/TEMPLATE", content);
+
+    if (content.empty())
     {
-        oss << "Last History Record" << endl << history->to_str(history_str);
+        return -1;
     }
 
-    str = oss.str();
+    // Virtual Machine template
+    rc += obj_template->from_xml_node(content[0]);
 
-    return str;
+    // Last history entry
+    ObjectXML::free_nodes(content);
+    content.clear();
+
+    ObjectXML::get_nodes("/VM/HISTORY_RECORDS/HISTORY", content);
+
+    if (!content.empty())
+    {
+        history = new History(oid);
+        rc += history->from_xml_node(content[0]);
+
+        history_records.resize(history->seq + 1);
+        history_records[history->seq] = history;
+
+        ObjectXML::free_nodes(content);
+    }
+
+    if (rc != 0)
+    {
+        return -1;
+    }
+
+    return 0;
 }
 
 /* -------------------------------------------------------------------------- */
