@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -26,7 +26,8 @@
 
 VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
                                        vector<const Attribute *> hook_mads,
-                                       const string& hook_location)
+                                       const string& hook_location,
+                                       const string& remotes_location)
     : PoolSQL(db,VirtualMachine::table)
 {
     const VectorAttribute * vattr;
@@ -82,8 +83,19 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
 
         if (cmd[0] != '/')
         {
-            cmd = hook_location + cmd;
-        }
+            ostringstream cmd_os;
+
+            if ( remote )
+            {
+                cmd_os << hook_location << "/" << cmd;     
+            }
+            else
+            {
+                cmd_os << remotes_location << "/hooks/" << cmd;
+            } 
+
+            cmd = cmd_os.str();
+        } 
 
         if ( on == "CREATE" )
         {
@@ -93,6 +105,16 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
 
             add_hook(hook);
         }
+        else if ( on == "PROLOG" )
+        {
+            VirtualMachineStateHook * hook;
+
+            hook = new VirtualMachineStateHook(name, cmd, arg, remote,
+                           VirtualMachine::PROLOG, VirtualMachine::ACTIVE);
+            add_hook(hook);
+
+            state_hook = true;
+        }
         else if ( on == "RUNNING" )
         {
             VirtualMachineStateHook * hook;
@@ -167,6 +189,9 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
 
 int VirtualMachinePool::allocate (
     int            uid,
+    int            gid,
+    const string&  uname,
+    const string&  gname,
     VirtualMachineTemplate * vm_template,
     int *          oid,
     string&        error_str,
@@ -177,7 +202,7 @@ int VirtualMachinePool::allocate (
     // ------------------------------------------------------------------------
     // Build a new Virtual Machine object
     // ------------------------------------------------------------------------
-    vm = new VirtualMachine(-1,vm_template);
+    vm = new VirtualMachine(-1, uid, gid, uname, gname, vm_template);
 
     if (on_hold == true)
     {
@@ -188,8 +213,6 @@ int VirtualMachinePool::allocate (
         vm->state = VirtualMachine::PENDING;
     }
 
-    vm->uid = uid;
-
     // ------------------------------------------------------------------------
     // Insert the Object in the pool
     // ------------------------------------------------------------------------
@@ -239,86 +262,3 @@ int VirtualMachinePool::get_pending(
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
-
-int  VirtualMachinePool::dump_cb(void * _oss,int num,char **values,char **names)
-{
-    ostringstream * oss;
-
-    oss = static_cast<ostringstream *>(_oss);
-
-    return VirtualMachine::dump(*oss, num, values, names);
-}
-
-int  VirtualMachinePool::dump_extended_cb(
-                                void * _oss,int num,char **values,char **names)
-{
-    ostringstream * oss;
-
-    oss = static_cast<ostringstream *>(_oss);
-
-    return VirtualMachine::dump_extended(*oss, num, values, names);
-}
-
-/* -------------------------------------------------------------------------- */
-
-int VirtualMachinePool::dump(   ostringstream&  oss,
-                                bool            extended,
-                                int             state,
-                                const string&   where)
-{
-    int             rc;
-    ostringstream   cmd;
-
-    oss << "<VM_POOL>";
-
-    if(extended)
-    {
-        set_callback(
-            static_cast<Callbackable::Callback>(
-                                        &VirtualMachinePool::dump_extended_cb),
-            static_cast<void *>(&oss));
-
-        cmd << "SELECT " << VirtualMachine::extended_db_names
-        << ", user_pool.user_name, "
-        << History::extended_db_names << " FROM " << VirtualMachine::table
-        << " LEFT OUTER JOIN " << History::table << " ON "
-        << VirtualMachine::table << ".oid = " << History::table << ".vid AND "
-        << History::table << ".seq = " << VirtualMachine::table
-        << ".last_seq LEFT OUTER JOIN (SELECT oid,user_name FROM user_pool) "
-        << "AS user_pool ON "<< VirtualMachine::table << ".uid = user_pool.oid";
-    }
-    else
-    {
-        set_callback(
-            static_cast<Callbackable::Callback>(&VirtualMachinePool::dump_cb),
-            static_cast<void *>(&oss));
-
-        cmd << "SELECT " << VirtualMachine::db_names << " FROM "
-            << VirtualMachine::table;
-    }
-
-    if ( state != -1 )
-    {
-        cmd << " WHERE " << VirtualMachine::table << ".state = " << state;
-    }
-    else
-    {
-        cmd << " WHERE " << VirtualMachine::table << ".state <> 6";
-    }
-
-    if ( !where.empty() )
-    {
-        cmd << " AND " << where;
-    }
-
-    rc = db->exec(cmd,this);
-
-    oss << "</VM_POOL>";
-
-    unset_callback();
-
-    return rc;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
