@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,6 +21,12 @@
 #include <fstream>
 #include <libgen.h>
 
+/* ************************************************************************** */
+/* Virtual Network :: Database Access Functions                               */
+/* ************************************************************************** */
+
+const char * LibVirtDriver::vmware_vnm_name = "vmware";
+
 int LibVirtDriver::deployment_description_vmware(
         const VirtualMachine *  vm,
         const string&           file_name) const
@@ -52,6 +58,7 @@ int LibVirtDriver::deployment_description_vmware(
 
     const VectorAttribute * nic;
 
+    string  network_id = "";
     string  mac        = "";
     string  bridge     = "";
     string  script     = "";
@@ -163,13 +170,8 @@ int LibVirtDriver::deployment_description_vmware(
     // ------------------------------------------------------------------------
 
     file << "\t<devices>" << endl;
-    
-    get_default("DISK","DRIVER",default_driver);
 
-    if (default_driver.empty())
-    {
-        default_driver = "raw";
-    }
+    get_default("DISK","DRIVER",default_driver);
 
     num = vm->get_template_attribute("DISK",attrs);
 
@@ -219,12 +221,14 @@ int LibVirtDriver::deployment_description_vmware(
         if ( type == "BLOCK" )
         {
             file << "\t\t<disk type='block' device='disk'>" << endl;
-            file << "\t\t\t<source dev='" << source << "'/>"  << endl;
+            file << "\t\t\t<source file=[" <<  datastore << "] " << vm->get_oid()
+                << "/images/disk."  << i << "'/>"  << endl;
         }
         else if ( type == "CDROM" )
         {
             file << "\t\t<disk type='file' device='cdrom'>" << endl;
-            file << "\t\t\t<source file='" << source << "'/>"  << endl;
+            file << "\t\t\t<source file=[" <<  datastore << "] " << vm->get_oid()
+                << "/images/disk."  << i << ".iso'/>"  << endl;
         }
         else
         {
@@ -243,16 +247,18 @@ int LibVirtDriver::deployment_description_vmware(
         {
             file << "/>" << endl;
         }
-        
-        file << "\t\t\t<driver name='";
 
         if ( !driver.empty() )
         {
-            file << driver << "'/>" << endl;
+            file << "\t\t\t<driver name='" << driver << "'/>" << endl;
         }
         else
         {
-            file << default_driver << "'/>" << endl;
+            if (!default_driver.empty())
+            {
+                file << "\t\t\t<driver name='" <<
+                        default_driver << "'/>" << endl;
+            }
         }
 
         if (readonly)
@@ -278,7 +284,7 @@ int LibVirtDriver::deployment_description_vmware(
         {
             file << "\t\t<disk type='file' device='cdrom'>" << endl;
             file << "\t\t\t<source file='[" <<  datastore <<"] " << vm->get_oid()
-                 << "/disk." << num << "'/>" << endl;
+                 << "/disk." << num << ".iso'/>" << endl;
             file << "\t\t\t<target dev='" << target << "'/>" << endl;
             file << "\t\t\t<readonly/>" << endl;
             file << "\t\t</disk>" << endl;
@@ -307,22 +313,25 @@ int LibVirtDriver::deployment_description_vmware(
             continue;
         }
 
-        bridge = nic->vector_value("BRIDGE");
-        mac    = nic->vector_value("MAC");
-        target = nic->vector_value("TARGET");
-        script = nic->vector_value("SCRIPT");
-        model  = nic->vector_value("MODEL");
+        network_id = nic->vector_value("NETWORK_ID");
+        mac        = nic->vector_value("MAC");
+        target     = nic->vector_value("TARGET");
+        script     = nic->vector_value("SCRIPT");
+        model      = nic->vector_value("MODEL");
 
-        if ( bridge.empty() )
+        if (vm->get_vnm_mad() == LibVirtDriver::vmware_vnm_name)
         {
-            file << "\t\t<interface type='ethernet'>" << endl;
+            bridge = "one-pg-";
+            bridge.append(network_id);
         }
         else
         {
-            file << "\t\t<interface type='bridge'>" << endl;
-            file << "\t\t\t<source bridge='" << bridge << "'/>" << endl;
+            bridge = nic->vector_value("BRIDGE");
         }
 
+        file << "\t\t<interface type='bridge'>" << endl;
+        file << "\t\t\t<source bridge='" << bridge << "'/>" << endl;
+
         if( !mac.empty() )
         {
             file << "\t\t\t<mac address='" << mac << "'/>" << endl;
