@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -354,7 +354,7 @@ int XenDriver::deployment_description(
 
         if( !model.empty() )
         {
-            file << "type=" << model;
+            file << "model=" << model;
             pre_char = ',';
         }
 
