@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -33,34 +33,46 @@ require "CommandManager"
 
 class DummyDriver < VirtualMachineDriver
     def initialize
-        super(15,true)
+        super('',
+            :concurrency => 15,
+            :threaded => true
+        )
     end
 
-    def deploy(id, host, remote_dfile, not_used)
-        send_message(ACTION[:deploy],RESULT[:success],id,"dummy")
+    def deploy(id, drv_message)
+        msg = decode(drv_message)
+
+        host = msg.elements["HOST"].text
+        name = msg.elements["VM/NAME"].text
+
+        send_message(ACTION[:deploy],RESULT[:success],id,"#{host}:#{name}:dummy")
     end
 
-    def shutdown(id, host, deploy_id, not_used)
+    def shutdown(id, drv_message)
         send_message(ACTION[:shutdown],RESULT[:success],id)
     end
 
-    def cancel(id, host, deploy_id, not_used)
+    def reboot(id, drv_message)
+        send_message(ACTION[:reboot],RESULT[:success],id)
+    end
+
+    def cancel(id, drv_message)
         send_message(ACTION[:cancel],RESULT[:success],id)
     end
 
-    def save(id, host, deploy_id, file)
+    def save(id, drv_message)
         send_message(ACTION[:save],RESULT[:success],id)
     end
 
-    def restore(id, host, deploy_id , file)
+    def restore(id, drv_message)
         send_message(ACTION[:restore],RESULT[:success],id)
     end
 
-    def migrate(id, host, deploy_id, dest_host)
+    def migrate(id, drv_message)
         send_message(ACTION[:migrate],RESULT[:success],id)
     end
 
-    def poll(id, host, deploy_id, not_used)
+    def poll(id, drv_message)
         # monitor_info: string in the form "VAR=VAL VAR=VAL ... VAR=VAL"
         # known VAR are in POLL_ATTRIBUTES. VM states VM_STATES
         monitor_info = "#{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:active]} " \
