@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 # ---------------------------------------------------------------------------- #
-# Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
 #                                                                              #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may      #
 # not use this file except in compliance with the License. You may obtain      #
@@ -15,9 +15,7 @@
 # limitations under the License.                                               #
 # ---------------------------------------------------------------------------- #
 
-# ---------------------------------------------------------------------------- #
-# Set up the environment for the driver                                        #
-# ---------------------------------------------------------------------------- #
+# Set up the environment for the driver
 
 EC2_LOCATION = ENV["EC2_HOME"]
 
@@ -45,35 +43,32 @@ require "VirtualMachineDriver"
 require "CommandManager"
 require "rexml/document"
 
-# ---------------------------------------------------------------------------- #
-# The main class for the EC2 driver                                            #
-# ---------------------------------------------------------------------------- #
-
+# The main class for the EC2 driver
 class EC2Driver < VirtualMachineDriver
 
-    # ------------------------------------------------------------------------ #
-    # EC2 commands constants                                                   #
-    # ------------------------------------------------------------------------ #
+    # EC2 commands constants
     EC2 = {
         :run       => "#{EC2_LOCATION}/bin/ec2-run-instances",
         :terminate => "#{EC2_LOCATION}/bin/ec2-terminate-instances",
         :describe  => "#{EC2_LOCATION}/bin/ec2-describe-instances",
         :associate => "#{EC2_LOCATION}/bin/ec2-associate-address",
-        :authorize => "#{EC2_LOCATION}bin/ec2-authorize"
+        :reboot    => "#{EC2_LOCATION}/bin/ec2-reboot-instances",
+        :authorize => "#{EC2_LOCATION}/bin/ec2-authorize"
     }
 
-    # ------------------------------------------------------------------------ #
-    # EC2 constructor, loads defaults for the EC2Driver                        #
-    # ------------------------------------------------------------------------ #
+    # EC2 constructor, loads defaults for the EC2Driver
     def initialize(ec2_conf = nil)
-        
+
         if !EC2_JVM_CONCURRENCY
             concurrency = 5
         else
             concurrency = EC2_JVM_CONCURRENCY.to_i
         end
 
-        super(concurrency,true)
+        super('',
+            :concurrency => concurrency,
+            :threaded => true
+        )
 
         @defaults = Hash.new
 
@@ -87,19 +82,20 @@ class EC2Driver < VirtualMachineDriver
             ec2 = xml.root.elements["EC2"]
 
             return if !ec2
-            
+
             @defaults["KEYPAIR"]         = ec2_value(ec2,"KEYPAIR")
             @defaults["AUTHORIZEDPORTS"] = ec2_value(ec2,"AUTHORIZEDPORTS")
             @defaults["INSTANCETYPE"]    = ec2_value(ec2,"INSTANCETYPE")
         end
     end
 
-    # ------------------------------------------------------------------------ #
-    # DEPLOY action, also sets ports and ip if needed                          #
-    # ------------------------------------------------------------------------ #
-    def deploy(id, host, remote_dfile, not_used)
+    # DEPLOY action, also sets ports and ip if needed
+    def deploy(id, drv_message)
+        msg = decode(drv_message)
 
-        local_dfile = get_local_deployment_file(remote_dfile)
+        host        = msg.elements["HOST"].text
+
+        local_dfile = msg.elements["LOCAL_DEPLOYMENT_FILE"].text
 
         if !local_dfile
             send_message(ACTION[:deploy],RESULT[:failure],id,
@@ -110,12 +106,12 @@ class EC2Driver < VirtualMachineDriver
         tmp = File.new(local_dfile)
         xml = REXML::Document.new tmp
         tmp.close()
-        
+
         ec2 = nil
 
         all_ec2_elements = xml.root.get_elements("EC2")
-    
-        # First, let's see if we have an EC2 site that matches 
+
+        # First, let's see if we have an EC2 site that matches
         # our desired host name
         all_ec2_elements.each { |element|
             cloud=element.elements["CLOUD"]
@@ -125,14 +121,15 @@ class EC2Driver < VirtualMachineDriver
         }
 
         if !ec2
-            # If we don't find the EC2 site, and ONE just 
+            # If we don't find the EC2 site, and ONE just
             # knows about one EC2 site, let's use that
             if all_ec2_elements.size == 1
                 ec2 = all_ec2_elements[0]
             else
                 send_message(ACTION[:deploy],RESULT[:failure],id,
-                    "Can not find EC2 element in deployment file #{local_dfile}" +
-                    " or couldn't find any EC2 site matching one of the template.")
+                    "Can not find EC2 element in deployment file "<<
+                    "#{local_dfile} or couldn't find any EC2 site matching "<<
+                    "one of the template.")
                 return
             end
         end
@@ -181,24 +178,46 @@ class EC2Driver < VirtualMachineDriver
         send_message(ACTION[:deploy],RESULT[:success],id,deploy_id)
     end
 
-    # ------------------------------------------------------------------------ #
-    # Shutdown a EC2 instance                                                  #
-    # ------------------------------------------------------------------------ #
-    def shutdown(id, host, deploy_id, not_used)
+    # Shutdown a EC2 instance
+    def shutdown(id, drv_message)
+        msg = decode(drv_message)
+
+        host      = msg.elements["HOST"].text
+        deploy_id = msg.elements["DEPLOY_ID"].text
+
         ec2_terminate(ACTION[:shutdown], id, deploy_id)
     end
+   
+    # Reboot a EC2 instance 
+    def reboot(id, drv_message)
+        cmd = "#{EC2_LOCATION}/bin/ec2-reboot-instances #{deploy_id}"
+        exe = LocalCommand.run(cmd, log_method(id))
+
+        if exe.code != 0
+            result = RESULT[:failure]
+        else
+            result = RESULT[:success]
+        end
+
+        send_message(action,result,id)
+    end
+
+    # Cancel a EC2 instance
+    def cancel(id, drv_message)
+        msg = decode(drv_message)
+
+        host      = msg.elements["HOST"].text
+        deploy_id = msg.elements["DEPLOY_ID"].text
 
-    # ------------------------------------------------------------------------ #
-    # Cancel a EC2 instance                                                    #
-    # ------------------------------------------------------------------------ #
-    def cancel(id, host, deploy_id, not_used)
         ec2_terminate(ACTION[:cancel], id, deploy_id)
     end
 
-    # ------------------------------------------------------------------------ #
-    # Get info (IP, and state) for a EC2 instance                              #
-    # ------------------------------------------------------------------------ #
-    def poll(id, host, deploy_id, not_used)
+    # Get info (IP, and state) for a EC2 instance
+    def poll(id, drv_message)
+        msg = decode(drv_message)
+
+        host      = msg.elements["HOST"].text
+        deploy_id = msg.elements["DEPLOY_ID"].text
 
         info =  "#{POLL_ATTRIBUTE[:usedmemory]}=0 " \
                 "#{POLL_ATTRIBUTE[:usedcpu]}=0 " \
@@ -224,8 +243,8 @@ class EC2Driver < VirtualMachineDriver
                 when "pending"
                     info << " #{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:active]}"
                 when "running"
-                    info << " #{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:active]}" \
-                            " IP=#{monitor_data[1]}"
+                    info<<" #{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:active]}"<<
+                        " IP=#{monitor_data[1]}"
                 when "shutting-down","terminated"
                     info << " #{POLL_ATTRIBUTE[:state]}=#{VM_STATE[:deleted]}"
             end
@@ -262,9 +281,7 @@ private
     end
 end
 
-# ---------------------------------------------------------------------------- #
 # EC2Driver Main program
-# ---------------------------------------------------------------------------- #
 
 ec2_conf = ARGV.last
 
