@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -17,33 +17,37 @@
 
 #include "VirtualNetwork.h"
 #include "VirtualNetworkPool.h"
+#include "VirtualNetworkTemplate.h"
 
 #include "NebulaLog.h"
 #include "RangedLeases.h"
 #include "FixedLeases.h"
 
 #include "AuthManager.h"
-#include "UserPool.h"
+
+#define TO_UPPER(S) transform(S.begin(),S.end(),S.begin(),(int(*)(int))toupper)
 
 /* ************************************************************************** */
 /* Virtual Network :: Constructor/Destructor                                  */
 /* ************************************************************************** */
 
-VirtualNetwork::VirtualNetwork(VirtualNetworkTemplate *_vn_template):
-                PoolObjectSQL(-1),
-                name(""),
-                uid(-1),
-                bridge(""),
-                type(UNINITIALIZED),
-                leases(0)
+VirtualNetwork::VirtualNetwork(int                      _uid,
+                               int                      _gid,
+                               const string&            _uname,
+                               const string&            _gname,
+                               VirtualNetworkTemplate * _vn_template):
+            PoolObjectSQL(-1,NET,"",_uid,_gid,_uname,_gname,table),
+            bridge(""),
+            type(UNINITIALIZED),
+            leases(0)
 {
     if (_vn_template != 0)
     {
-        vn_template = _vn_template;
+        obj_template = _vn_template;
     }
     else
     {
-        vn_template = new VirtualNetworkTemplate;
+        obj_template = new VirtualNetworkTemplate;
     }
 };
 
@@ -57,9 +61,9 @@ VirtualNetwork::~VirtualNetwork()
         delete leases;
     }
 
-    if (vn_template != 0)
+    if (obj_template != 0)
     {
-        delete vn_template;
+        delete obj_template;
     }
 }
 
@@ -69,125 +73,66 @@ VirtualNetwork::~VirtualNetwork()
 
 const char * VirtualNetwork::table        = "network_pool";
 
-const char * VirtualNetwork::db_names     =
-                                "oid,uid,name,type,bridge,public,template";
-
-const char * VirtualNetwork::extended_db_names =
-    "network_pool.oid, network_pool.uid, network_pool.name, network_pool.type, "
-    "network_pool.bridge, network_pool.public, network_pool.template";
+const char * VirtualNetwork::db_names =
+        "oid, name, body, uid, gid, owner_u, group_u, other_u";
 
 const char * VirtualNetwork::db_bootstrap = "CREATE TABLE IF NOT EXISTS"
-    " network_pool ("
-     "oid INTEGER PRIMARY KEY, uid INTEGER, name VARCHAR(256), type INTEGER, "
-     "bridge TEXT, public INTEGER, template TEXT, UNIQUE(name))";
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-int VirtualNetwork::select_cb(void * nil, int num, char **values, char **names)
-{
-    if ((!values[OID])      ||
-        (!values[UID])      ||
-        (!values[NAME])     ||
-        (!values[TYPE])     ||
-        (!values[BRIDGE])   ||
-        (!values[PUBLIC])   ||
-        (!values[TEMPLATE]) ||
-        (num != LIMIT ))
-    {
-        return -1;
-    }
-
-    oid         = atoi(values[OID]);
-    uid         = atoi(values[UID]);
-
-    name        = values[NAME];
-
-    type        = (NetworkType)atoi(values[TYPE]);
-
-    bridge      = values[BRIDGE];
-
-    public_vnet = atoi(values[PUBLIC]);
-
-    // Virtual Network template
-    vn_template->from_xml(values[TEMPLATE]);
-
-    return 0;
-}
+    " network_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128),"
+    " body TEXT, uid INTEGER, gid INTEGER, "
+    "owner_u INTEGER, group_u INTEGER, other_u INTEGER, UNIQUE(name,uid))";
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 int VirtualNetwork::select(SqlDB * db)
+{
+    int             rc;
+
+    rc = PoolObjectSQL::select(db);
+
+    if ( rc != 0 )
+    {
+        return rc;
+    }
+
+    return select_leases(db);
+}
+
+/* -------------------------------------------------------------------------- */
+
+int VirtualNetwork::select(SqlDB * db, const string& name, int uid)
+{
+    int             rc;
+
+    rc = PoolObjectSQL::select(db,name,uid);
+
+    if ( rc != 0 )
+    {
+        return rc;
+    }
+
+    return select_leases(db);
+}
+
+/* -------------------------------------------------------------------------- */
+
+int VirtualNetwork::select_leases(SqlDB * db)
 {
     ostringstream   oss;
     ostringstream   ose;
 
-    int             rc;
-    int             boid;
-
     string          network_address;
 
-    unsigned int default_size = VirtualNetworkPool::default_size();
     unsigned int mac_prefix   = VirtualNetworkPool::mac_prefix();
 
-    set_callback(
-        static_cast<Callbackable::Callback>(&VirtualNetwork::select_cb));
-
-    oss << "SELECT " << db_names << " FROM " << table << " WHERE oid = " << oid;
-
-    boid = oid;
-    oid  = -1;
-
-    rc = db->exec(oss, this);
-
-    unset_callback();
-
-    if ((rc != 0) || (oid != boid ))
-    {
-        goto error_id;
-    }
-
     //Get the leases
     if (type == RANGED)
     {
-        string  nclass = "";
-        int     size = 0;
-
-        // retrieve specific information from the template
-        get_template_attribute("NETWORK_ADDRESS",network_address);
-
-        if (network_address.empty())
-        {
-            goto error_addr;
-        }
-
-        get_template_attribute("NETWORK_SIZE",nclass);
-
-        if ( nclass == "B" || nclass == "b" )
-        {
-            size = 65534;
-        }
-        else if ( nclass == "C" || nclass == "c" )
-        {
-            size = 254;
-        }
-        else if (!nclass.empty()) //Assume it's a number
-        {
-            istringstream iss(nclass);
-            iss >> size;
-        }
-
-        if (size == 0)
-        {
-            size = default_size;
-        }
-
         leases = new RangedLeases(db,
                                   oid,
-                                  size,
                                   mac_prefix,
-                                  network_address);
+                                  ip_start,
+                                  ip_end);
     }
     else if(type == FIXED)
     {
@@ -207,9 +152,6 @@ int VirtualNetwork::select(SqlDB * db)
 
     return leases->select(db);
 
-error_id:
-    ose << "Error getting Virtual Network nid: " << oid;
-    goto error_common;
 
 error_leases:
     ose << "Error getting Virtual Network leases nid: " << oid;
@@ -219,9 +161,6 @@ error_type:
     ose << "Wrong type of Virtual Network: " << type;
     goto error_common;
 
-error_addr:
-    ose << "Network address is not defined nid: " << oid;
-
 error_common:
     NebulaLog::log("VNM", Log::ERROR, ose);
     return -1;
@@ -230,52 +169,15 @@ error_common:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualNetwork::dump(ostringstream& oss,
-                         int            num,
-                         char **        values,
-                         char **        names)
-{
-    if ((!values[OID])      ||
-        (!values[UID])      ||
-        (!values[NAME])     ||
-        (!values[TYPE])     ||
-        (!values[BRIDGE])   ||
-        (!values[PUBLIC])   ||
-        (!values[TEMPLATE]) ||
-        (!values[LIMIT])    ||
-        (num != LIMIT + 2 ))
-    {
-        return -1;
-    }
-
-    oss <<
-        "<VNET>" <<
-            "<ID>"          << values[OID]      << "</ID>"          <<
-            "<UID>"         << values[UID]      << "</UID>"         <<
-            "<USERNAME>"    << values[LIMIT+1]  << "</USERNAME>"    <<
-            "<NAME>"        << values[NAME]     << "</NAME>"        <<
-            "<TYPE>"        << values[TYPE]     << "</TYPE>"        <<
-            "<BRIDGE>"      << values[BRIDGE]   << "</BRIDGE>"      <<
-            "<PUBLIC>"      << values[PUBLIC]   << "</PUBLIC>"      <<
-            "<TOTAL_LEASES>"<< values[LIMIT]    << "</TOTAL_LEASES>"
-                            << values[TEMPLATE] <<
-        "</VNET>";
-
-    return 0;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
 int VirtualNetwork::insert(SqlDB * db, string& error_str)
 {
     ostringstream   ose;
     int             rc;
 
-    string          pub;
+    string          vlan_attr;
     string          s_type;
+    string          ranged_error_str;
 
-    unsigned int default_size = VirtualNetworkPool::default_size();
     unsigned int mac_prefix   = VirtualNetworkPool::mac_prefix();
 
     //--------------------------------------------------------------------------
@@ -283,9 +185,9 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
     //--------------------------------------------------------------------------
 
     // ------------ TYPE ----------------------
-    get_template_attribute("TYPE",s_type);
+    erase_template_attribute("TYPE",s_type);
 
-    transform(s_type.begin(),s_type.end(),s_type.begin(),(int(*)(int))toupper);
+    TO_UPPER(s_type);
 
     if (s_type == "RANGED")
     {
@@ -295,91 +197,81 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
     {
         type = VirtualNetwork::FIXED;
     }
+    else if ( s_type.empty() )
+    {
+        goto error_type_defined;
+    }
     else
     {
-        goto error_type;
+        goto error_wrong_type;
     }
 
     // ------------ NAME ----------------------
 
-    get_template_attribute("NAME",name);
+    erase_template_attribute("NAME",name);
 
     if (name.empty())
     {
         goto error_name;
     }
 
+    // ------------ PHYDEV --------------------
+
+    erase_template_attribute("PHYDEV",phydev);
+
+    // ------------ VLAN_ID -------------------
+
+    erase_template_attribute("VLAN_ID",vlan_id);
+
+    // ------------ VLAN ----------------------
+
+    erase_template_attribute("VLAN", vlan_attr);
+
+    TO_UPPER(vlan_attr);
+
+    vlan = (vlan_attr == "YES") || (vlan_attr.empty() && !phydev.empty());
+
     // ------------ BRIDGE --------------------
 
-    get_template_attribute("BRIDGE",bridge);
+    erase_template_attribute("BRIDGE",bridge);
 
     if (bridge.empty())
     {
-        goto error_bridge;
+        if (phydev.empty())
+        {
+            goto error_bridge;
+        }
+        else
+        {
+            ostringstream oss;
+
+            oss << "onebr" << oid;
+ 
+            bridge = oss.str();
+        }
     }
 
-    // ------------ PUBLIC --------------------
-
-    get_template_attribute("PUBLIC", pub);
-
-    transform (pub.begin(), pub.end(), pub.begin(), (int(*)(int))toupper);
-
-    public_vnet = (pub == "YES");
-
-    vn_template->erase("PUBLIC");
-
     //--------------------------------------------------------------------------
     // Get the leases
     //--------------------------------------------------------------------------
     if (type == VirtualNetwork::RANGED)
     {
-        string nclass = "";
-        string naddr  = "";
-        int    size   = 0;
 
-        // retrieve specific information from template
-        get_template_attribute("NETWORK_ADDRESS",naddr);
+        int rc;
 
-        if (naddr.empty())
+        rc = RangedLeases::process_template(this, ip_start, ip_end,
+                ranged_error_str);
+
+        if ( rc != 0 )
         {
-            goto error_addr;
-        }
-
-        get_template_attribute("NETWORK_SIZE",nclass);
-
-        if ( nclass == "B" || nclass == "b"  )
-        {
-            size = 65534;
-        }
-        else if ( nclass == "C" || nclass == "c"  )
-        {
-            size = 254;
-        }
-        else if (!nclass.empty())//Assume its a number
-        {
-            istringstream iss(nclass);
-
-            iss >> size;
-        }
-
-        if (size == 0)
-        {
-            SingleAttribute * attribute;
-            ostringstream     oss;
-
-            oss << default_size;
-
-            attribute = new SingleAttribute("NETWORK_SIZE",oss.str());
-            vn_template->set(attribute);
-
-            size = default_size;
+            goto error_ranged;
         }
 
         leases = new RangedLeases(db,
                                   oid,
-                                  size,
                                   mac_prefix,
-                                  naddr);
+                                  ip_start,
+                                  ip_end);
     }
     else // VirtualNetwork::FIXED
     {
@@ -391,6 +283,8 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
                                  oid,
                                  mac_prefix,
                                  vector_leases);
+
+        remove_template_attribute("LEASES");
     }
 
     if (leases == 0)
@@ -401,7 +295,7 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
     //--------------------------------------------------------------------------
     // Insert the Virtual Network
     //--------------------------------------------------------------------------
-    rc = insert_replace(db, false);
+    rc = insert_replace(db, false, error_str);
 
     if ( rc != 0 )
     {
@@ -410,32 +304,34 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
 
     return 0;
 
-error_type:
-    ose << "Wrong type in template for Virtual Network, id: ";
+error_type_defined:
+    ose << "No TYPE in template for Virtual Network.";
+    goto error_common;
+
+error_wrong_type:
+    ose << "Wrong type \""<< s_type <<"\" in template for Virtual Network.";
     goto error_common;
 
 error_name:
-    ose << "No NAME in template for Virtual Network, id: ";
+    ose << "No NAME in template for Virtual Network.";
     goto error_common;
 
 error_bridge:
-    ose << "No BRIDGE in template for Virtual Network, id: ";
+    ose << "No BRIDGE in template for Virtual Network.";
     goto error_common;
 
 error_update:
-    ose << "Can not update Virtual Network, id: ";
+    ose << error_str;
     goto error_common;
 
-error_addr:
-    ose << "Network address is not defined, id: ";
+error_ranged:
+    ose << ranged_error_str;
     goto error_common;
 
 error_null_leases:
-    ose << "Error getting Virtual Network leases, id: ";
+    ose << "Error getting Virtual Network leases.";
 
 error_common:
-    ose << oid << ".";
-
     error_str = ose.str();
     NebulaLog::log("VNM", Log::ERROR, ose);
     return -1;
@@ -444,23 +340,16 @@ error_common:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-
-int VirtualNetwork::update(SqlDB * db)
-{
-    return insert_replace(db, true);
-}
-
-int VirtualNetwork::insert_replace(SqlDB *db, bool replace)
+int VirtualNetwork::insert_replace(SqlDB *db, bool replace, string& error_str)
 {
     ostringstream   oss;
     int             rc;
 
 
-    string xml_template;
+    string xml_body;
 
     char * sql_name;
-    char * sql_bridge;
-    char * sql_template;
+    char * sql_xml;
 
 
     sql_name = db->escape_str(name.c_str());
@@ -470,22 +359,18 @@ int VirtualNetwork::insert_replace(SqlDB *db, bool replace)
         goto error_name;
     }
 
-    sql_bridge = db->escape_str(bridge.c_str());
+    sql_xml = db->escape_str(to_xml(xml_body).c_str());
 
-    if ( sql_bridge == 0 )
+    if ( sql_xml == 0 )
     {
-        goto error_bridge;
+        goto error_body;
     }
 
-    vn_template->to_xml(xml_template);
-    sql_template = db->escape_str(xml_template.c_str());
-
-    if ( sql_template == 0 )
+    if ( validate_xml(sql_xml) != 0 )
     {
-        goto error_template;
+        goto error_xml;
     }
 
-
     // Construct the SQL statement to Insert or Replace
     if(replace)
     {
@@ -498,92 +383,158 @@ int VirtualNetwork::insert_replace(SqlDB *db, bool replace)
 
     oss << " INTO " << table << " (" << db_names << ") VALUES ("
         <<          oid         << ","
-        <<          uid         << ","
         << "'" <<   sql_name    << "',"
-        <<          type        << ","
-        << "'" <<   sql_bridge  << "',"
-        <<          public_vnet << ","
-        << "'" <<   sql_template<< "')";
+        << "'" <<   sql_xml     << "',"
+        <<          uid         << ","
+        <<          gid         << ","
+        <<          owner_u     << ","
+        <<          group_u     << ","
+        <<          other_u     << ")";
 
     rc = db->exec(oss);
 
     db->free_str(sql_name);
-    db->free_str(sql_bridge);
-    db->free_str(sql_template);
+    db->free_str(sql_xml);
 
     return rc;
 
-error_template:
-    db->free_str(sql_bridge);
-error_bridge:
+error_xml:
     db->free_str(sql_name);
+    db->free_str(sql_xml);
+
+    error_str = "Error transforming the Virtual Network to XML.";
+
+    goto error_common;
+
+error_body:
+    db->free_str(sql_name);
+    goto error_generic;
+
 error_name:
+    goto error_generic;
+
+error_generic:
+    error_str = "Error inserting Virtual Network in DB.";
+error_common:
     return -1;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualNetwork::vn_drop(SqlDB * db)
+int VirtualNetwork::drop(SqlDB * db)
 {
-    ostringstream   oss;
-    int             rc;
+    int rc;
 
-    if ( leases != 0 )
+    rc = PoolObjectSQL::drop(db);
+
+    if ( rc == 0 && leases != 0 )
     {
-        leases->drop(db);
-    }
-
-    oss << "DELETE FROM " << table << " WHERE OID=" << oid;
-
-    rc = db->exec(oss);
-
-    if ( rc == 0 )
-    {
-        set_valid(false);
+        rc += leases->drop(db);
     }
 
     return rc;
 }
 
-
-
 /* ************************************************************************** */
 /* Virtual Network :: Misc                                                    */
 /* ************************************************************************** */
 
-ostream& operator<<(ostream& os, VirtualNetwork& vn)
-{
-    string vnet_xml;
-
-    os << vn.to_xml(vnet_xml);
-
-    return os;
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
 string& VirtualNetwork::to_xml(string& xml) const
 {
-    ostringstream os;
+    return to_xml_extended(xml,false);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string& VirtualNetwork::to_xml_extended(string& xml) const
+{
+    return to_xml_extended(xml,true);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string& VirtualNetwork::to_xml_extended(string& xml, bool extended) const
+{
+    ostringstream   os;
 
     string template_xml;
     string leases_xml;
+    string perm_str;
+
+    // Total leases is the number of used leases.
+    int total_leases = 0;
+
+    if (leases != 0)
+    {
+        total_leases = leases->n_used;
+    }
 
     os <<
         "<VNET>" <<
-            "<ID>"      << oid          << "</ID>"   <<
-            "<UID>"     << uid          << "</UID>"  <<
-            "<NAME>"    << name         << "</NAME>" <<
-            "<TYPE>"    << type         << "</TYPE>" <<
-            "<BRIDGE>"  << bridge       << "</BRIDGE>" <<
-            "<PUBLIC>"  << public_vnet  << "</PUBLIC>" <<
-            vn_template->to_xml(template_xml);
-    if (leases)
+            "<ID>"     << oid    << "</ID>"    <<
+            "<UID>"    << uid    << "</UID>"   <<
+            "<GID>"    << gid    << "</GID>"   <<
+            "<UNAME>"  << uname  << "</UNAME>" << 
+            "<GNAME>"  << gname  << "</GNAME>" <<
+            "<NAME>"   << name   << "</NAME>"  <<
+            perms_to_xml(perm_str)              <<
+            "<TYPE>"   << type   << "</TYPE>"  <<
+            "<BRIDGE>" << bridge << "</BRIDGE>"<<
+            "<VLAN>"   << vlan   << "</VLAN>";
+
+    if (!phydev.empty())
     {
-        os << leases->to_xml(leases_xml);
+        os << "<PHYDEV>" << phydev << "</PHYDEV>";
     }
+    else
+    {
+        os << "<PHYDEV/>";
+    }
+
+    if (!vlan_id.empty())
+    {
+        os << "<VLAN_ID>" << vlan_id << "</VLAN_ID>";
+    }
+    else
+    {
+        os << "<VLAN_ID/>";
+    }
+
+    if ( type == RANGED )
+    {
+        string st_ip_start;
+        string st_ip_end;
+
+        Leases::Lease::ip_to_string(ip_start, st_ip_start);
+        Leases::Lease::ip_to_string(ip_end,   st_ip_end);
+
+        os <<
+            "<RANGE>" <<
+                "<IP_START>" << st_ip_start << "</IP_START>" <<
+                "<IP_END>"   << st_ip_end   << "</IP_END>"   <<
+            "</RANGE>";
+    }
+
+    os  <<  "<TOTAL_LEASES>"<< total_leases << "</TOTAL_LEASES>"<<
+            obj_template->to_xml(template_xml);
+
+    if (extended)
+    {
+        if (leases != 0)
+        {
+            os <<   "<LEASES>"                  <<
+                    leases->to_xml(leases_xml)     <<
+                    "</LEASES>";
+        }
+        else
+        {
+            os << "<LEASES/>";
+        }
+    }
+
     os << "</VNET>";
 
     xml = os.str();
@@ -594,40 +545,67 @@ string& VirtualNetwork::to_xml(string& xml) const
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-string& VirtualNetwork::to_str(string& str) const
+int VirtualNetwork::from_xml(const string &xml_str)
 {
-    ostringstream os;
+    vector<xmlNodePtr> content;
 
-    string template_str;
-    string leases_str;
+    int rc = 0;
+    int int_type;
 
-    os << "ID                : " << oid << endl;
-    os << "UID               : " << uid << endl;
-    os << "NAME              : " << name << endl;
-    os << "Type              : ";
-    if ( type==VirtualNetwork::RANGED )
+    // Initialize the internal XML object
+    update_from_str(xml_str);
+
+    // Get class base attributes
+    rc += xpath(oid,        "/VNET/ID",     -1);
+    rc += xpath(uid,        "/VNET/UID",    -1);
+    rc += xpath(gid,        "/VNET/GID",    -1);
+    rc += xpath(uname,      "/VNET/UNAME",  "not_found");
+    rc += xpath(gname,      "/VNET/GNAME",  "not_found");
+    rc += xpath(name,       "/VNET/NAME",   "not_found");
+    rc += xpath(int_type,   "/VNET/TYPE",   -1);
+    rc += xpath(bridge,     "/VNET/BRIDGE", "not_found");
+    rc += xpath(vlan,       "/VNET/VLAN",   0);
+
+    // Permissions
+    rc += perms_from_xml();
+
+    xpath(phydev,  "/VNET/PHYDEV", "");
+    xpath(vlan_id, "/VNET/VLAN_ID","");
+
+    type = static_cast<NetworkType>(int_type);
+
+    // Get associated classes
+    ObjectXML::get_nodes("/VNET/TEMPLATE", content);
+
+    if (content.empty())
     {
-        os << "Ranged" << endl;
-    }
-    else
-    {
-       os << "Fixed" << endl;
+        return -1;
     }
 
-    os << "Bridge            : " << bridge      << endl;
-    os << "Public            : " << public_vnet << endl << endl;
+    // Virtual Network template
+    rc += obj_template->from_xml_node(content[0]);
 
-    os << "....: Template :...." << vn_template->to_str(template_str) << endl <<
-endl;
+    ObjectXML::free_nodes(content);
 
-    if (leases)
+    // Ranged Leases
+    if (type == RANGED)
     {
-        os << "....: Leases :...." << endl << leases->to_str(leases_str) << endl;
+        string st_ip_start;
+        string st_ip_end;
+
+        rc += xpath(st_ip_start,    "/VNET/RANGE/IP_START", "0");
+        rc += xpath(st_ip_end,      "/VNET/RANGE/IP_END",   "0");
+
+        Leases::Lease::ip_to_number(st_ip_start, ip_start);
+        Leases::Lease::ip_to_number(st_ip_end,   ip_end);
     }
 
-    str = os.str();
+    if (rc != 0)
+    {
+        return -1;
+    }
 
-    return str;
+    return 0;
 }
 
 /* -------------------------------------------------------------------------- */
@@ -637,15 +615,11 @@ int VirtualNetwork::nic_attribute(VectorAttribute *nic, int vid)
 {
     int rc;
 
-    string  model;
     string  ip;
     string  mac;
 
     ostringstream  vnid;
 
-    map<string,string> new_nic;
-
-    model   = nic->vector_value("MODEL");
     ip      = nic->vector_value("IP");
     vnid   << oid;
 
@@ -671,18 +645,30 @@ int VirtualNetwork::nic_attribute(VectorAttribute *nic, int vid)
     //                       NEW NIC ATTRIBUTES
     //--------------------------------------------------------------------------
 
-    new_nic.insert(make_pair("NETWORK"   ,name));
-    new_nic.insert(make_pair("MAC"       ,mac));
-    new_nic.insert(make_pair("BRIDGE"    ,bridge));
-    new_nic.insert(make_pair("NETWORK_ID",vnid.str()));
-    new_nic.insert(make_pair("IP"        ,ip));
+    nic->replace("NETWORK"   ,name);
+    nic->replace("NETWORK_ID",vnid.str());
+    nic->replace("BRIDGE"    ,bridge);
+    nic->replace("MAC"       ,mac);
+    nic->replace("IP"        ,ip);
 
-    if (!model.empty())
+    if ( vlan == 1 )
     {
-        new_nic.insert(make_pair("MODEL",model));
+        nic->replace("VLAN", "YES");
+    }
+    else
+    {
+        nic->replace("VLAN", "NO");
     }
 
-    nic->replace(new_nic);
+    if (!phydev.empty())
+    {
+        nic->replace("PHYDEV", phydev);
+    }
+
+    if (!vlan_id.empty())
+    {
+        nic->replace("VLAN_ID", vlan_id);
+    }
 
     return 0;
 }
@@ -715,3 +701,29 @@ int VirtualNetwork::remove_leases(VirtualNetworkTemplate * leases_template,
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
+
+int VirtualNetwork::hold_leases(VirtualNetworkTemplate * leases_template,
+                                string&                  error_msg)
+{
+    vector<const Attribute *> vector_leases;
+
+    leases_template->get("LEASES", vector_leases);
+
+    return leases->hold_leases(vector_leases, error_msg);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VirtualNetwork::free_leases(VirtualNetworkTemplate * leases_template,
+                                  string&                  error_msg)
+{
+    vector<const Attribute *> vector_leases;
+
+    leases_template->get("LEASES", vector_leases);
+
+    return leases->free_leases(vector_leases, error_msg);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
