@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2011, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -15,8 +15,10 @@
 /* -------------------------------------------------------------------------- */
 
 #include "VirtualNetworkPool.h"
+#include "UserPool.h"
 #include "NebulaLog.h"
-
+#include "Nebula.h"
+#include "PoolObjectAuth.h"
 #include "AuthManager.h"
 #include <sstream>
 #include <ctype.h>
@@ -71,166 +73,152 @@ VirtualNetworkPool::VirtualNetworkPool(SqlDB * db,
 
 int VirtualNetworkPool::allocate (
     int            uid,
+    int            gid,
+    const string&  uname,
+    const string&  gname,
     VirtualNetworkTemplate * vn_template,
     int *          oid,
     string&        error_str)
 {
-    VirtualNetwork *    vn;
+    VirtualNetwork *         vn;
+    VirtualNetwork *         vn_aux = 0;
+    string          name;
+    ostringstream   oss;
 
-    vn = new VirtualNetwork(vn_template);
+    vn = new VirtualNetwork(uid, gid, uname, gname, vn_template);
 
-    vn->uid = uid;
+    // Check name
+    vn->get_template_attribute("NAME", name);
+
+    if ( name.empty() )
+    {
+        goto error_name;
+    }
+
+    if ( name.length() > 128 )
+    {
+        goto error_name_length;
+    }
+
+    // Check for duplicates
+    vn_aux = get(name,uid,false);
+
+    if( vn_aux != 0 )
+    {
+        goto error_duplicated;
+    }
+
+    vn = new VirtualNetwork(uid, gid, uname, gname, vn_template);
 
     *oid = PoolSQL::allocate(vn, error_str);
 
+    return *oid;
+
+error_name:
+    oss << "NAME cannot be empty.";
+
+    goto error_common;
+
+error_name_length:
+    oss << "NAME is too long; max length is 128 chars.";
+    goto error_common;
+
+error_duplicated:
+    oss << "NAME is already taken by NET "
+        << vn_aux->get_oid() << ".";
+
+error_common:
+    delete vn;
+
+    *oid = -1;
+    error_str = oss.str();
+
     return *oid;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualNetworkPool::get_cb(void * _oid, int num, char **values,char **names)
+VirtualNetwork * VirtualNetworkPool::get_nic_by_name(VectorAttribute * nic, 
+                                                     const string&     name,
+                                                     int               _uid)
 {
-    int * oid;
+    istringstream  is;
 
-    oid = static_cast<int *>(_oid);
+    string uid_s ;
+    string uname;
+    int    uid;
 
-    if ( oid == 0 || values == 0 || values[0] == 0 )
+    if (!(uid_s = nic->vector_value("NETWORK_UID")).empty())
     {
-        return -1;
+        is.str(uid_s);
+        is >> uid;
+
+        if( is.fail() )
+        {
+            return 0;
+        }
+    }
+    else if (!(uname = nic->vector_value("NETWORK_UNAME")).empty())
+    {
+        User *     user;
+        Nebula&    nd    = Nebula::instance();
+        UserPool * upool = nd.get_upool();
+        
+        user = upool->get(uname,true);
+        
+        if ( user == 0 )
+        {
+            return 0;
+        }
+
+        uid = user->get_oid();
+
+        user->unlock();
+    }
+    else
+    {
+        uid = _uid;        
     }
 
-    *oid = atoi(values[0]);
-
-    return 0;
+    return get(name,uid,true);
 }
-
-/* -------------------------------------------------------------------------- */
+        
 /* -------------------------------------------------------------------------- */
 
-VirtualNetwork * VirtualNetworkPool::get(const string& name, bool lock)
+VirtualNetwork * VirtualNetworkPool::get_nic_by_id(const string& id_s)
 {
-    ostringstream   oss;
+    istringstream  is;
+    int            id;
 
-    int oid = -1;
-    int rc;
+    is.str(id_s);
+    is >> id;
 
-    char * sql_name = db->escape_str(name.c_str());
-
-    if ( sql_name == 0 )
+    if( is.fail() )
     {
         return 0;
     }
 
-    set_callback(
-        static_cast<Callbackable::Callback>(&VirtualNetworkPool::get_cb),
-        static_cast<void *>(&oid));
-
-    oss << "SELECT oid FROM " << VirtualNetwork::table << " WHERE name = '"
-        << sql_name << "'";
-
-    rc = db->exec(oss, this);
-
-    unset_callback();
-
-    db->free_str(sql_name);
-
-    if (rc != 0 || oid == -1)
-    {
-        return 0;
-    }
-
-    return get(oid,lock);
+    return get(id,true);
 }
 
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-int VirtualNetworkPool::dump_cb(void * _oss,
-                                int     num,
-                                char ** values,
-                                char ** names)
-{
-    ostringstream * oss;
-
-    oss = static_cast<ostringstream *>(_oss);
-
-    return VirtualNetwork::dump(*oss, num, values, names);
-}
-
-/* -------------------------------------------------------------------------- */
-
-int VirtualNetworkPool::dump(ostringstream& oss, const string& where)
-{
-    int             rc;
-    ostringstream   cmd;
-
-    oss << "<VNET_POOL>";
-
-    set_callback(
-        static_cast<Callbackable::Callback>(&VirtualNetworkPool::dump_cb),
-        static_cast<void *>(&oss));
-
-    cmd << "SELECT " << VirtualNetwork::extended_db_names << ",COUNT("
-        << Leases::table << ".used), user_pool.user_name FROM "
-        << VirtualNetwork::table
-        << " LEFT OUTER JOIN " << Leases::table << " ON "
-        << VirtualNetwork::table << ".oid = " <<  Leases::table << ".oid"
-        << " AND " << Leases::table << ".used = 1"
-        << " LEFT OUTER JOIN (SELECT oid,user_name FROM user_pool) "
-        << " AS user_pool ON "<< VirtualNetwork::table
-        << ".uid = user_pool.oid";
-
-    if ( !where.empty() )
-    {
-        cmd << " WHERE " << where;
-    }
-
-    cmd << " GROUP BY " << VirtualNetwork::table << ".oid";
-
-    rc = db->exec(cmd,this);
-
-    oss << "</VNET_POOL>";
-
-    unset_callback();
-
-    return rc;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-int VirtualNetworkPool::nic_attribute(VectorAttribute * nic, int vid)
+int VirtualNetworkPool::nic_attribute(VectorAttribute * nic, int uid, int vid)
 {
     string           network;
     VirtualNetwork * vnet = 0;
 
-    network = nic->vector_value("NETWORK");
-
-    if (network.empty())
+    if (!(network = nic->vector_value("NETWORK")).empty())
     {
-        istringstream   is;
-        int             network_id;
-
-        network = nic->vector_value("NETWORK_ID");
-
-        if(network.empty())
-        {
-            return -2;
-        }
-
-        is.str(network);
-        is >> network_id;
-
-        if( !is.fail() )
-        {
-            vnet = get(network_id,true);
-        }
+        vnet = get_nic_by_name (nic, network, uid);
     }
-    else
+    else if (!(network = nic->vector_value("NETWORK_ID")).empty())
     {
-        vnet = get(network,true);
+        vnet = get_nic_by_id(network);
     }
+    else //Not using a pre-defined network
+    {
+        return -2;
+    } 
 
     if (vnet == 0)
     {
@@ -239,6 +227,11 @@ int VirtualNetworkPool::nic_attribute(VectorAttribute * nic, int vid)
 
     int rc = vnet->nic_attribute(nic,vid);
 
+    if ( rc == 0 )
+    {
+        update(vnet);
+    }
+
     vnet->unlock();
 
     return rc;
@@ -247,48 +240,35 @@ int VirtualNetworkPool::nic_attribute(VectorAttribute * nic, int vid)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void VirtualNetworkPool::authorize_nic(VectorAttribute * nic, AuthRequest * ar)
+void VirtualNetworkPool::authorize_nic(VectorAttribute * nic, 
+                                       int uid, 
+                                       AuthRequest * ar)
 {
     string           network;
     VirtualNetwork * vnet = 0;
+    PoolObjectAuth   perm;
 
-    network = nic->vector_value("NETWORK");
-
-    if (network.empty())
+    if (!(network = nic->vector_value("NETWORK")).empty())
     {
-        istringstream   is;
-        int             network_id;
-
-        network = nic->vector_value("NETWORK_ID");
-
-        if(network.empty())
-        {
-            return;
-        }
-
-        is.str(network);
-        is >> network_id;
-
-        if( !is.fail() )
-        {
-            vnet = get(network_id,true);
-        }
+        vnet = get_nic_by_name (nic, network, uid);
     }
-    else
+    else if (!(network = nic->vector_value("NETWORK_ID")).empty())
     {
-        vnet = get(network,true);
+        vnet = get_nic_by_id(network);
     }
+    else //Not using a pre-defined network
+    {
+        return;
+    } 
 
     if (vnet == 0)
     {
         return;
     }
 
-    ar->add_auth(AuthRequest::NET,
-                 vnet->get_vnid(),
-                 AuthRequest::USE,
-                 vnet->get_uid(),
-                 vnet->isPublic());
+    vnet->get_permissions(perm);
 
     vnet->unlock();
+
+    ar->add_auth(AuthRequest::USE, perm);
 }
